@@ -26,13 +26,14 @@ static const wxString typeNames[SERVERTYPE_MAX] = {
 	TRANSLATE_T("Default (Autodetect)"),
 	_T("Unix"),
 	_T("VMS"),
-	_T("DOS"),
+	_T("DOS with backslash separators"),
 	_T("MVS, OS/390, z/OS"),
 	_T("VxWorks"),
 	_T("z/VM"),
 	_T("HP NonStop"),
 	TRANSLATE_T("DOS-like with virtual paths"),
-	_T("Cygwin")
+	_T("Cygwin"),
+	_T("DOS with forward-slash separators"),
 };
 
 static const t_protocolInfo& GetProtocolInfo(ServerProtocol protocol)
@@ -273,6 +274,14 @@ wxString CServer::GetAccount() const
 	return m_account;
 }
 
+wxString CServer::GetKeyFile() const
+{
+	if (m_logonType != KEY)
+		return wxString();
+
+	return m_keyFile;
+}
+
 CServer& CServer::operator=(const CServer &op)
 {
 	m_protocol = op.m_protocol;
@@ -283,6 +292,7 @@ CServer& CServer::operator=(const CServer &op)
 	m_user = op.m_user;
 	m_pass = op.m_pass;
 	m_account = op.m_account;
+	m_keyFile = op.m_keyFile;
 	m_timezoneOffset = op.m_timezoneOffset;
 	m_pasvMode = op.m_pasvMode;
 	m_maximumMultipleConnections = op.m_maximumMultipleConnections;
@@ -324,6 +334,11 @@ bool CServer::operator==(const CServer &op) const
 			if (m_account != op.m_account)
 				return false;
 		}
+		else if (m_logonType == KEY)
+		{
+			if (m_keyFile != op.m_keyFile)
+				return false;
+		}
 	}
 	if (m_timezoneOffset != op.m_timezoneOffset)
 		return false;
@@ -577,6 +592,16 @@ bool CServer::SetAccount(const wxString& account)
 	return true;
 }
 
+bool CServer::SetKeyFile(const wxString& keyFile)
+{
+	if (m_logonType != KEY)
+		return false;
+
+	m_keyFile = keyFile;
+
+	return true;
+}
+
 bool CServer::SetTimezoneOffset(int minutes)
 {
 	if (minutes > (60 * 24) || minutes < (-60 * 24))
@@ -831,6 +856,8 @@ LogonType CServer::GetLogonTypeFromName(const wxString& name)
 		return NORMAL;
 	else if (name == _("Ask for password"))
 		return ASK;
+	else if (name == _("Key file"))
+		return KEY;
 	else if (name == _("Interactive"))
 		return INTERACTIVE;
 	else if (name == _("Account"))
@@ -849,6 +876,8 @@ wxString CServer::GetNameFromLogonType(LogonType type)
 		return _("Normal");
 	case ASK:
 		return _("Ask for password");
+	case KEY:
+		return _("Key file");
 	case INTERACTIVE:
 		return _("Interactive");
 	case ACCOUNT:
