@@ -20,13 +20,14 @@ static const CServerTypeTraits traits[SERVERTYPE_MAX] = {
 	{ _T("/"),   true,     0,    0,    false, 0, 0,   true,  false }, // Failsafe
 	{ _T("/"),   true,     0,    0,    false, 0, 0,   true,  false },
 	{ _T("."),   false,  '[',  ']',    false, 0, '^', false, false },
-	{ _T("\\/"), false,    0,    0,    false, 0, 0,   true,  false },
+	{ _T("\\/"), false,    0,    0,    false, 0, 0,   true,  false }, // DOS with backslashes
 	{ _T("."),   false, '\'', '\'',     true, 1, 0,   false, false },
 	{ _T("/"),   true,     0,    0,    false, 0, 0,   true,  false },
 	{ _T("/"),   true,     0,    0,    false, 0, 0,   true,  false }, // Same as Unix
 	{ _T("."),   false,    0,    0,    false, 0, 0,   false, false },
 	{ _T("\\"),  true,     0,    0,    false, 0, 0,   true,  false },
-	{ _T("/"),   true,     0,    0,    false, 0, 0,   true,  true  }  // Cygwin is like Unix but has optional prefix of form "//server"
+	{ _T("/"),   true,     0,    0,    false, 0, 0,   true,  true  },  // Cygwin is like Unix but has optional prefix of form "//server"
+	{ _T("/\\"), false,    0,    0,    false, 0, 0,   true,  false } // DOS with forwardslashes
 };
 
 bool CServerPathData::operator==(const CServerPathData& cmp) const
@@ -45,7 +46,7 @@ CServerPath::CServerPath()
 {
 }
 
-CServerPath::CServerPath(const CServerPath &path, wxString subdir)
+CServerPath::CServerPath(CServerPath const& path, wxString subdir)
 	: m_type(path.m_type)
 	, m_data(path.m_data)
 {
@@ -56,12 +57,6 @@ CServerPath::CServerPath(const CServerPath &path, wxString subdir)
 		clear();
 }
 
-CServerPath::CServerPath(const CServerPath &path)
-	: m_type(path.m_type)
-	, m_data(path.m_data)
-{
-}
-
 CServerPath::CServerPath(wxString const& path, ServerType type /*=DEFAULT*/)
 	: m_type(type)
 {
@@ -101,8 +96,7 @@ bool CServerPath::SetPath(wxString &newPath, bool isFile)
 				m_type = DOS;
 		else if (path.c_str()[0] == FTP_MVS_DOUBLE_QUOTE && path.Last() == FTP_MVS_DOUBLE_QUOTE)
 			m_type = MVS;
-		else if (path[0] == ':' && (pos1 = path.Mid(1).Find(':')) > 0)
-		{
+		else if (path[0] == ':' && (pos1 = path.Mid(1).Find(':')) > 0) {
 			int slash = path.Find('/');
 			if (slash == -1 || slash > pos1)
 				m_type = VXWORKS;
@@ -165,7 +159,7 @@ wxString CServerPath::GetPath() const
 
 	// DOS is strange.
 	// C: is current working dir on drive C, C:\ the drive root.
-	if (m_type == DOS && m_data->m_segments.size() == 1)
+	if ((m_type == DOS || m_type == DOS_FWD_SLASHES) && m_data->m_segments.size() == 1)
 		path += traits[m_type].separators[0];
 
 	return path;
@@ -188,12 +182,12 @@ CServerPath CServerPath::GetParent() const
 		return CServerPath();
 
 	CServerPath parent(*this);
-	CServerPathData& parent_data = parent.m_data.Get();
+	CServerPathData& parent_data = parent.m_data.get();
 
 	parent_data.m_segments.pop_back();
 
 	if (m_type == MVS)
-		parent_data.m_prefix = CSparseOptional<wxString>(_T("."));
+		parent_data.m_prefix = fz::sparse_optional<wxString>(_T("."));
 
 	return parent;
 }
@@ -287,7 +281,7 @@ bool CServerPath::SetSafePath(const wxString& path)
 
 bool CServerPath::DoSetSafePath(const wxString& path)
 {
-	CServerPathData& data = m_data.Get();
+	CServerPathData& data = m_data.get();
 	data.m_prefix.clear();
 	data.m_segments.clear();
 
@@ -349,7 +343,7 @@ bool CServerPath::DoSetSafePath(const wxString& path)
 	{
 		*(p + prefix_len) = 0;
 		if( *p ) {
-			data.m_prefix = CSparseOptional<wxString>(p);
+			data.m_prefix = fz::sparse_optional<wxString>(p);
 		}
 
 		p += prefix_len + 1;
@@ -487,7 +481,7 @@ bool CServerPath::DoChangePath(wxString &subdir, bool isFile)
 	}
 
 	bool const was_empty = empty();
-	CServerPathData& data = m_data.Get();
+	CServerPathData& data = m_data.get();
 
 	switch (m_type)
 	{
@@ -524,7 +518,7 @@ bool CServerPath::DoChangePath(wxString &subdir, bool isFile)
 				dir = dir.Left(pos2);
 
 				if (pos1)
-					data.m_prefix = CSparseOptional<wxString>(dir.Left(pos1));
+					data.m_prefix = fz::sparse_optional<wxString>(dir.Left(pos1));
 				dir = dir.Mid(pos1 + 1);
 
 				data.m_segments.clear();
@@ -537,6 +531,7 @@ bool CServerPath::DoChangePath(wxString &subdir, bool isFile)
 		}
 		break;
 	case DOS:
+	case DOS_FWD_SLASHES:
 		{
 			bool is_relative = false;
 			int sep = dir.find_first_of(traits[m_type].separators);
@@ -628,10 +623,10 @@ bool CServerPath::DoChangePath(wxString &subdir, bool isFile)
 			if (isFile) {
 				if (!ExtractFile(dir, file))
 					return false;
-				data.m_prefix = CSparseOptional<wxString>(_T("."));
+				data.m_prefix = fz::sparse_optional<wxString>(_T("."));
 			}
 			else if (!dir.empty() && dir.Last() == '.')
-				data.m_prefix = CSparseOptional<wxString>(_T("."));
+				data.m_prefix = fz::sparse_optional<wxString>(_T("."));
 			else
 				data.m_prefix.clear();
 		}
@@ -664,7 +659,7 @@ bool CServerPath::DoChangePath(wxString &subdir, bool isFile)
 				int colon2;
 				if ((colon2 = dir.Mid(1).Find(':')) < 1)
 					return false;
-				data.m_prefix = CSparseOptional<wxString>(dir.Left(colon2 + 2));
+				data.m_prefix = fz::sparse_optional<wxString>(dir.Left(colon2 + 2));
 				dir = dir.Mid(colon2 + 2);
 
 				data.m_segments.clear();
@@ -688,7 +683,7 @@ bool CServerPath::DoChangePath(wxString &subdir, bool isFile)
 				return false;
 			if (dir.Left(2) == _T("//"))
 			{
-				data.m_prefix = CSparseOptional<wxString>(traits[m_type].separators[0]);
+				data.m_prefix = fz::sparse_optional<wxString>(traits[m_type].separators[0]);
 				dir = dir.Mid(1);
 			}
 
@@ -858,7 +853,7 @@ bool CServerPath::AddSegment(const wxString& segment)
 		return false;
 
 	// TODO: Check for invalid characters
-	m_data.Get().m_segments.push_back(segment);
+	m_data.get().m_segments.push_back(segment);
 
 	return true;
 }
@@ -895,7 +890,7 @@ CServerPath CServerPath::GetCommonParent(const CServerPath& path) const
 	CServerPath parent;
 	parent.m_type = m_type;
 
-	CServerPathData& parentData = parent.m_data.Get();
+	CServerPathData& parentData = parent.m_data.get();
 
 	tConstSegmentIter last = m_data->m_segments.end();
 	tConstSegmentIter last2 = path.m_data->m_segments.end();
@@ -939,52 +934,68 @@ wxString CServerPath::FormatSubdir(const wxString &subdir) const
 	return res;
 }
 
-bool CServerPath::Segmentize(wxString str, tSegmentList& segments)
+bool CServerPath::SegmentizeAddSegment(wxString & segment, tSegmentList& segments, bool& append)
 {
-	bool append = false;
-	while (!str.empty()) {
-		wxString segment;
-		int pos = str.find_first_of(traits[m_type].separators);
-		if (pos == -1) {
-			segment = str,
-			str.clear();
-		}
-		else if (!pos) {
-			str = str.Mid(1);
-			continue;
-		}
-		else {
-			segment = str.Left(pos);
-			str = str.Mid(pos + 1);
-		}
-
-		if (traits[m_type].has_dots) {
-			if (segment == _T("."))
-				continue;
-			else if (segment == _T("..")) {
-				if (segments.empty())
-					return false;
-				else {
-					segments.pop_back();
-					continue;
-				}
+	if (traits[m_type].has_dots) {
+		if (segment == _T("."))
+			return true;
+		else if (segment == _T("..")) {
+			if (segments.empty())
+				return false;
+			else {
+				segments.pop_back();
+				return true;
 			}
 		}
+	}
 
-		bool append_next = false;
-		if (!segment.empty() && traits[m_type].separatorEscape && segment.Last() == traits[m_type].separatorEscape) {
-			append_next = true;
-			segment.RemoveLast();
-			segment += traits[m_type].separators[0];
+	bool append_next = false;
+	if (!segment.empty() && traits[m_type].separatorEscape && segment.Last() == traits[m_type].separatorEscape) {
+		append_next = true;
+		segment[segment.size() - 1] = traits[m_type].separators[0];
+	}
+
+	if (append)
+		segments.back() += segment;
+	else
+		segments.push_back(std::move(segment));
+
+	append = append_next;
+
+	return true;
+}
+
+bool CServerPath::Segmentize(wxString const& str, tSegmentList& segments)
+{
+	bool append = false;
+	size_t start = 0;
+
+	size_t pos;
+	while (true) {
+		pos = str.find_first_of(traits[m_type].separators, start);
+		if (pos == std::string::npos) {
+			break;
+		}
+		if (start == pos) {
+			++start;
+			continue;
 		}
 
-		if (append)
-			segments.back() += segment;
-		else
-			segments.push_back(segment);
+		wxString segment = str.substr(start, pos - start);
+		start = pos + 1;
 
-		append = append_next;
+		if (!SegmentizeAddSegment(segment, segments, append)) {
+			return false;
+		}
 	}
+
+	if (start < str.size()) {
+		wxString segment = str.substr(start);
+		if (!SegmentizeAddSegment(segment, segments, append)) {
+			return false;
+		}
+	}
+
 	if (append)
 		return false;
 
@@ -997,8 +1008,7 @@ bool CServerPath::ExtractFile(wxString& dir, wxString& file)
 	if (pos == (int)dir.Length() - 1)
 		return false;
 
-	if (pos == -1)
-	{
+	if (pos == -1) {
 		file = dir;
 		dir.clear();
 		return true;
