@@ -3,26 +3,27 @@
 #include "directorycache.h"
 #include "directorylistingparser.h"
 #include "engineprivate.h"
-#include "event_loop.h"
 #include "pathcache.h"
-#include "local_filesys.h"
-#include "fzprocess.h"
 #include "proxy.h"
 #include "servercapabilities.h"
 #include "sftpcontrolsocket.h"
 
+#include <libfilezilla/event_loop.hpp>
+#include <libfilezilla/local_filesys.hpp>
+#include <libfilezilla/process.hpp>
+
 #include <wx/filename.h>
 #include <wx/log.h>
 #include <wx/tokenzr.h>
 #include <wx/txtstrm.h>
 
-#define FZSFTP_PROTOCOL_VERSION 3
+#define FZSFTP_PROTOCOL_VERSION 5
 
 struct sftp_event_type;
-typedef CEvent<sftp_event_type> CSftpEvent;
+typedef fz::simple_event<sftp_event_type> CSftpEvent;
 
 struct terminate_event_type;
-typedef CEvent<terminate_event_type> CTerminateEvent;
+typedef fz::simple_event<terminate_event_type> CTerminateEvent;
 
 class CSftpFileTransferOpData : public CFileTransferOpData
 {
@@ -54,37 +55,28 @@ struct sftp_message
 	};
 };
 
-class CSftpInputThread final : public wxThread
+class CSftpInputThread final : public fz::thread
 {
 public:
-	CSftpInputThread(CSftpControlSocket* pOwner, CProcess& process)
-		: wxThread(wxTHREAD_JOINABLE)
-		, process_(process)
+	CSftpInputThread(CSftpControlSocket* pOwner, fz::process& proc)
+		: process_(proc)
 		, m_pOwner(pOwner)
 	{
 	}
 
 	virtual ~CSftpInputThread()
 	{
-		scoped_lock l(m_sync);
+		join();
+
+		fz::scoped_lock l(m_sync);
 		for (auto & msg : m_sftpMessages) {
 			delete msg;
 		}
 	}
 
-	bool Init()
-	{
-		if (Create() != wxTHREAD_NO_ERROR)
-			return false;
-
-		Run();
-
-		return true;
-	}
-
 	void GetMessages(std::vector<sftp_message*>& messages)
 	{
-		scoped_lock l(m_sync);
+		fz::scoped_lock l(m_sync);
 		messages.swap(m_sftpMessages);
 	}
 
@@ -95,13 +87,13 @@ protected:
 		bool sendEvent;
 
 		{
-			scoped_lock l(m_sync);
+			fz::scoped_lock l(m_sync);
 			sendEvent = m_sftpMessages.empty();
 			m_sftpMessages.push_back(message);
 		}
 
 		if (sendEvent)
-			m_pOwner->SendEvent<CSftpEvent>();
+			m_pOwner->send_event<CSftpEvent>();
 	}
 
 	int ReadNumber(bool &error)
@@ -110,7 +102,7 @@ protected:
 
 		while(true) {
 			char c;
-			int read = process_.Read(&c, 1);
+			int read = process_.read(&c, 1);
 			if (read != 1) {
 				if (!read)
 					m_pOwner->LogMessage(MessageType::Debug_Warning, _T("Unexpected EOF."));
@@ -138,7 +130,7 @@ protected:
 
 		while(true) {
 			char c;
-			int read = process_.Read(&c, 1);
+			int read = process_.read(&c, 1);
 			if (read != 1) {
 				if (!read)
 					m_pOwner->LogMessage(MessageType::Debug_Warning, _T("Unexpected EOF."));
@@ -173,12 +165,12 @@ protected:
 		return line;
 	}
 
-	virtual ExitCode Entry()
+	virtual void entry()
 	{
 		bool error = false;
 		while (!error) {
 			char readType = 0;
-			int read = process_.Read(&readType, 1);
+			int read = process_.read(&readType, 1);
 			if (read != 1)
 				break;
 
@@ -237,14 +229,17 @@ protected:
 
 						m_pOwner->SendAsyncRequest(new CHostKeyNotification(line.Mid(1), port, fingerprint, requestType == sftpReqHostkeyChanged));
 					}
-					else if (requestType == sftpReqPassword)
-					{
+					else if (requestType == sftpReqPassword) {
 						sftp_message* message = new sftp_message;
 						message->type = eventType;
 						message->reqType = sftpReqPassword;
 						message->text = line.Mid(1);
 						SendMessage(message);
 					}
+					else if (requestType == sftpReqHostkeyBetteralg) {
+						m_pOwner->LogMessage(MessageType::Error, _T("Got sftpReqHostkeyBetteralg when we shouldn't have. Aborting connection."));
+						goto loopexit;
+					}
 				}
 				break;
 			case sftpEvent::Recv:
@@ -284,20 +279,14 @@ protected:
 		}
 loopexit:
 
-		m_pOwner->SendEvent<CTerminateEvent>();
-		return reinterpret_cast<ExitCode>(Close());
+		m_pOwner->send_event<CTerminateEvent>();
 	}
 
-	int Close()
-	{
-		return 0;
-	}
-
-	CProcess& process_;
+	fz::process& process_;
 	CSftpControlSocket* m_pOwner;
 
 	std::vector<sftp_message*> m_sftpMessages;
-	mutex m_sync;
+	fz::mutex m_sync;
 };
 
 class CSftpDeleteOpData final : public COpData
@@ -313,9 +302,9 @@ public:
 	CServerPath path;
 	std::deque<wxString> files;
 
-	// Set to CDateTime::Now initially and after
+	// Set to fz::datetime::Now initially and after
 	// sending an updated listing to the UI.
-	CDateTime m_time;
+	fz::datetime m_time;
 
 	bool m_needSendListing{};
 
@@ -331,7 +320,7 @@ CSftpControlSocket::CSftpControlSocket(CFileZillaEnginePrivate & engine)
 
 CSftpControlSocket::~CSftpControlSocket()
 {
-	RemoveHandler();
+	remove_handler();
 	DoClose();
 }
 
@@ -349,7 +338,6 @@ public:
 	CSftpConnectOpData()
 		: COpData(Command::connect)
 	{
-		pLastChallenge = 0;
 		criticalFailure = false;
 		pKeyFiles = 0;
 	}
@@ -357,10 +345,9 @@ public:
 	virtual ~CSftpConnectOpData()
 	{
 		delete pKeyFiles;
-		delete pLastChallenge;
 	}
 
-	wxString *pLastChallenge;
+	wxString lastChallenge;
 	bool criticalFailure;
 
 	wxStringTokenizer* pKeyFiles;
@@ -407,27 +394,27 @@ int CSftpControlSocket::Connect(const CServer &server)
 	else
 		pData->pKeyFiles = pTokenizer;
 
-	m_pProcess = new CProcess();
+	m_pProcess = new fz::process();
 
 	engine_.GetRateLimiter().AddObject(this);
 
-	wxString executable = engine_.GetOptions().GetOption(OPTION_FZSFTP_EXECUTABLE);
+	auto executable = fz::to_native(engine_.GetOptions().GetOption(OPTION_FZSFTP_EXECUTABLE));
 	if (executable.empty())
-		executable = _T("fzsftp");
+		executable = fzT("fzsftp");
 	LogMessage(MessageType::Debug_Verbose, _T("Going to execute %s"), executable);
 
-	std::vector<wxString> args = {_T("-v")};
+	std::vector<fz::native_string> args = {fzT("-v")};
 	if (engine_.GetOptions().GetOptionVal(OPTION_SFTP_COMPRESSION)) {
-		args.push_back(_T("-C"));
+		args.push_back(fzT("-C"));
 	}
-	if (!m_pProcess->Execute(executable, args)) {
+	if (!m_pProcess->spawn(executable, args)) {
 		LogMessage(MessageType::Debug_Warning, _T("Could not create process: %s"), wxSysErrorMsg());
 		DoClose();
 		return FZ_REPLY_ERROR;
 	}
 
 	m_pInputThread = new CSftpInputThread(this, *m_pProcess);
-	if (!m_pInputThread->Init()) {
+	if (!m_pInputThread->run()) {
 		LogMessage(MessageType::Debug_Warning, _T("Thread creation failed"));
 		delete m_pInputThread;
 		m_pInputThread = 0;
@@ -599,7 +586,7 @@ void CSftpControlSocket::OnSftpEvent()
 		{
 		case sftpEvent::Reply:
 			LogMessageRaw(MessageType::Response, message->text);
-			ProcessReply(true, message->text);
+			ProcessReply(FZ_REPLY_OK, message->text);
 			break;
 		case sftpEvent::Status:
 			LogMessageRaw(MessageType::Status, message->text);
@@ -612,7 +599,17 @@ void CSftpControlSocket::OnSftpEvent()
 			break;
 		case sftpEvent::Done:
 			{
-				ProcessReply(message->text == _T("1"));
+				int result;
+				if (message->text == _T("1")) {
+					result = FZ_REPLY_OK;
+				}
+				else if (message->text == _T("2")) {
+					result = FZ_REPLY_CRITICALERROR;
+				}
+				else {
+					result = FZ_REPLY_ERROR;
+				}
+				ProcessReply(result);
 				break;
 			}
 		case sftpEvent::RequestPreamble:
@@ -625,52 +622,60 @@ void CSftpControlSocket::OnSftpEvent()
 			switch(message->reqType)
 			{
 			case sftpReqPassword:
-				if (!m_pCurOpData || m_pCurOpData->opId != Command::connect)
-				{
+				if (!m_pCurOpData || m_pCurOpData->opId != Command::connect) {
 					LogMessage(MessageType::Debug_Warning, _T("sftpReqPassword outside connect operation, ignoring."));
 					break;
 				}
-
-				if (m_pCurrentServer->GetLogonType() == INTERACTIVE)
-				{
-					wxString challenge;
-					if (!m_requestPreamble.empty())
-						challenge += m_requestPreamble + _T("\n");
-					if (!m_requestInstruction.empty())
-						challenge += m_requestInstruction + _T("\n");
-					if (message->text != _T("Password:"))
-						challenge += message->text;
-					CInteractiveLoginNotification *pNotification = new CInteractiveLoginNotification(challenge);
-					pNotification->server = *m_pCurrentServer;
-
-					SendAsyncRequest(pNotification);
-				}
-				else
-				{
+				else {
 					CSftpConnectOpData *pData = static_cast<CSftpConnectOpData*>(m_pCurOpData);
 
-					const wxString newChallenge = m_requestPreamble + _T("\n") + m_requestInstruction + message->text;
+					wxString const challengeIdentifier = m_requestPreamble + _T("\n") + m_requestInstruction + _T("\n") + message->text;
 
-					if (pData->pLastChallenge)
-					{
-						// Check for same challenge. Will most likely fail as well, so abort early.
-						if (*pData->pLastChallenge == newChallenge)
-							LogMessage(MessageType::Error, _("Authentication failed."));
-						else
-							LogMessage(MessageType::Error, _("Server sent an additional login prompt. You need to use the interactive login type."));
-						DoClose(FZ_REPLY_CRITICALERROR | FZ_REPLY_PASSWORDFAILED);
+					if (m_pCurrentServer->GetLogonType() == INTERACTIVE || m_requestPreamble == _T("SSH key passphrase")) {
+						CInteractiveLoginNotification::type t = CInteractiveLoginNotification::interactive;
+						if (m_requestPreamble == _T("SSH key passphrase")) {
+							t = CInteractiveLoginNotification::keyfile;
+						}
 
-						for (;iter != messages.end(); ++iter)
-							delete *iter;
-						return;
+						wxString challenge;
+						if (!m_requestPreamble.empty() && t != CInteractiveLoginNotification::keyfile)
+							challenge += m_requestPreamble + _T("\n");
+						if (!m_requestInstruction.empty())
+							challenge += m_requestInstruction + _T("\n");
+						if (message->text != _T("Password:"))
+							challenge += message->text;
+						CInteractiveLoginNotification *pNotification = new CInteractiveLoginNotification(t, challenge, pData->lastChallenge == challengeIdentifier);
+						pNotification->server = *m_pCurrentServer;
+
+						pData->lastChallenge = challengeIdentifier;
+
+						SendAsyncRequest(pNotification);
 					}
+					else {
+						const wxString newChallenge = m_requestPreamble + _T("\n") + m_requestInstruction + message->text;
 
-					pData->pLastChallenge = new wxString(newChallenge);
+						if (!pData->lastChallenge.empty()) {
+							// Check for same challenge. Will most likely fail as well, so abort early.
+							if (pData->lastChallenge == challengeIdentifier) {
+								LogMessage(MessageType::Error, _("Authentication failed."));
+							}
+							else {
+								LogMessage(MessageType::Error, _("Server sent an additional login prompt. You need to use the interactive login type."));
+							}
+							DoClose(FZ_REPLY_CRITICALERROR | FZ_REPLY_PASSWORDFAILED);
 
-					const wxString pass = m_pCurrentServer->GetPass();
-					wxString show = _T("Pass: ");
-					show.Append('*', pass.Length());
-					SendCommand(pass, show);
+							for (;iter != messages.end(); ++iter)
+								delete *iter;
+							return;
+						}
+
+						pData->lastChallenge = challengeIdentifier;
+
+						const wxString pass = m_pCurrentServer->GetPass();
+						wxString show = _T("Pass: ");
+						show.Append('*', pass.Length());
+						SendCommand(pass, show);
+					}
 				}
 				break;
 			default:
@@ -788,7 +793,7 @@ bool CSftpControlSocket::AddToStream(const wxString& cmd, bool force_utf8 /*=fal
 	}
 
 	unsigned int len = strlen(str);
-	return m_pProcess->Write(str, len);
+	return m_pProcess->write(str, len);
 }
 
 bool CSftpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotification)
@@ -847,8 +852,7 @@ bool CSftpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotifi
 		{
 			CInteractiveLoginNotification *pInteractiveLoginNotification = static_cast<CInteractiveLoginNotification *>(pNotification);
 
-			if (!pInteractiveLoginNotification->passwordSet)
-			{
+			if (!pInteractiveLoginNotification->passwordSet) {
 				DoClose(FZ_REPLY_CANCELED);
 				return false;
 			}
@@ -896,12 +900,15 @@ public:
 
 	CDirectoryListing directoryListing;
 	int mtime_index;
+
+	fz::monotonic_clock m_time_before_locking;
 };
 
 enum listStates
 {
 	list_init = 0,
 	list_waitcwd,
+	list_waitlock,
 	list_list,
 	list_mtime
 };
@@ -1014,11 +1021,11 @@ int CSftpControlSocket::ListParseResponse(bool successful, const wxString& reply
 				seconds += c - '0';
 			}
 			if (parsed) {
-				CDateTime date(seconds, CDateTime::seconds);
-				if (date.IsValid()) {
+				fz::datetime date(seconds, fz::datetime::seconds);
+				if (!date.empty()) {
 					wxASSERT(pData->directoryListing[pData->mtime_index].has_date());
-					CDateTime listTime = pData->directoryListing[pData->mtime_index].time;
-					listTime -= duration::from_minutes(m_pCurrentServer->GetTimezoneOffset());
+					fz::datetime listTime = pData->directoryListing[pData->mtime_index].time;
+					listTime -= fz::duration::from_minutes(m_pCurrentServer->GetTimezoneOffset());
 
 					int serveroffset = static_cast<int>((date - listTime).get_seconds());
 					if (!pData->directoryListing[pData->mtime_index].has_seconds()) {
@@ -1030,7 +1037,7 @@ int CSftpControlSocket::ListParseResponse(bool successful, const wxString& reply
 
 					LogMessage(MessageType::Status, _("Timezone offset of server is %d seconds."), -serveroffset);
 
-					duration span = duration::from_seconds(serveroffset);
+					fz::duration span = fz::duration::from_seconds(serveroffset);
 					const int count = pData->directoryListing.GetCount();
 					for (int i = 0; i < count; ++i) {
 						CDirentry& entry = pData->directoryListing[i];
@@ -1113,8 +1120,7 @@ int CSftpControlSocket::ListSubcommandResult(int prevResult)
 {
 	LogMessage(MessageType::Debug_Verbose, _T("CSftpControlSocket::ListSubcommandResult()"));
 
-	if (!m_pCurOpData)
-	{
+	if (!m_pCurOpData) {
 		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("Empty m_pCurOpData"));
 		ResetOperation(FZ_REPLY_INTERNALERROR);
 		return FZ_REPLY_ERROR;
@@ -1123,16 +1129,13 @@ int CSftpControlSocket::ListSubcommandResult(int prevResult)
 	CSftpListOpData *pData = static_cast<CSftpListOpData *>(m_pCurOpData);
 	LogMessage(MessageType::Debug_Debug, _T("  state = %d"), pData->opState);
 
-	if (pData->opState != list_waitcwd)
-	{
+	if (pData->opState != list_waitcwd) {
 		ResetOperation(FZ_REPLY_INTERNALERROR);
 		return FZ_REPLY_ERROR;
 	}
 
-	if (prevResult != FZ_REPLY_OK)
-	{
-		if (pData->fallback_to_current)
-		{
+	if (prevResult != FZ_REPLY_OK) {
+		if (pData->fallback_to_current) {
 			// List current directory instead
 			pData->fallback_to_current = false;
 			pData->path.clear();
@@ -1141,8 +1144,7 @@ int CSftpControlSocket::ListSubcommandResult(int prevResult)
 			if (res != FZ_REPLY_OK)
 				return res;
 		}
-		else
-		{
+		else {
 			ResetOperation(prevResult);
 			return FZ_REPLY_ERROR;
 		}
@@ -1151,8 +1153,7 @@ int CSftpControlSocket::ListSubcommandResult(int prevResult)
 	if (pData->path.empty())
 		pData->path = m_CurrentPath;
 
-	if (!pData->refresh)
-	{
+	if (!pData->refresh) {
 		wxASSERT(!pData->pNextOpData);
 
 		// Do a cache lookup now that we know the correct directory
@@ -1160,11 +1161,9 @@ int CSftpControlSocket::ListSubcommandResult(int prevResult)
 		int hasUnsureEntries;
 		bool is_outdated = false;
 		bool found = engine_.GetDirectoryCache().DoesExist(*m_pCurrentServer, m_CurrentPath, hasUnsureEntries, is_outdated);
-		if (found)
-		{
+		if (found) {
 			// We're done if listins is recent and has no outdated entries
-			if (!is_outdated && !hasUnsureEntries)
-			{
+			if (!is_outdated && !hasUnsureEntries) {
 				engine_.SendDirectoryListingNotification(m_CurrentPath, true, false, false);
 
 				ResetOperation(FZ_REPLY_OK);
@@ -1174,10 +1173,12 @@ int CSftpControlSocket::ListSubcommandResult(int prevResult)
 		}
 	}
 
-	if (!pData->holdsLock)
-	{
-		if (!TryLockCache(lock_list, m_CurrentPath))
+	if (!pData->holdsLock) {
+		if (!TryLockCache(lock_list, m_CurrentPath)) {
+			pData->opState = list_waitlock;
+			pData->m_time_before_locking = fz::monotonic_clock::now();
 			return FZ_REPLY_WOULDBLOCK;
+		}
 	}
 
 	pData->opState = list_list;
@@ -1189,8 +1190,7 @@ int CSftpControlSocket::ListSend()
 {
 	LogMessage(MessageType::Debug_Verbose, _T("CSftpControlSocket::ListSend()"));
 
-	if (!m_pCurOpData)
-	{
+	if (!m_pCurOpData) {
 		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("Empty m_pCurOpData"));
 		ResetOperation(FZ_REPLY_INTERNALERROR);
 		return FZ_REPLY_ERROR;
@@ -1199,7 +1199,32 @@ int CSftpControlSocket::ListSend()
 	CSftpListOpData *pData = static_cast<CSftpListOpData *>(m_pCurOpData);
 	LogMessage(MessageType::Debug_Debug, _T("  state = %d"), pData->opState);
 
-	if (pData->opState == list_list) {
+	if (pData->opState == list_waitlock) {
+		if (!pData->holdsLock) {
+			LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Warning, _T("Not holding the lock as expected"));
+			ResetOperation(FZ_REPLY_INTERNALERROR);
+			return FZ_REPLY_ERROR;
+		}
+
+		// Check if we can use already existing listing
+		CDirectoryListing listing;
+		bool is_outdated = false;
+		wxASSERT(pData->subDir.empty()); // Did do ChangeDir before trying to lock
+		bool found = engine_.GetDirectoryCache().Lookup(listing, *m_pCurrentServer, pData->path, true, is_outdated);
+		if (found && !is_outdated && !listing.get_unsure_flags() &&
+			listing.m_firstListTime >= pData->m_time_before_locking)
+		{
+			engine_.SendDirectoryListingNotification(listing.path, !pData->pNextOpData, false, false);
+
+			ResetOperation(FZ_REPLY_OK);
+			return FZ_REPLY_OK;
+		}
+
+		pData->opState = list_waitcwd;
+
+		return ListSubcommandResult(FZ_REPLY_OK);
+	}
+	else if (pData->opState == list_list) {
 		pData->pParser = new CDirectoryListingParser(this, *m_pCurrentServer, listingEncoding::unknown, true);
 		pData->pParser->SetTimezoneOffset(GetTimezoneOffset());
 		if (!SendCommand(_T("ls")))
@@ -1444,32 +1469,42 @@ int CSftpControlSocket::ChangeDirSend()
 	return FZ_REPLY_WOULDBLOCK;
 }
 
-int CSftpControlSocket::ProcessReply(bool successful, const wxString& reply /*=_T("")*/)
+void CSftpControlSocket::ProcessReply(int result, const wxString& reply /*=_T("")*/)
 {
 	Command commandId = GetCurrentCommandId();
 	switch (commandId)
 	{
 	case Command::connect:
-		return ConnectParseResponse(successful, reply);
+		ConnectParseResponse(result == FZ_REPLY_OK, reply);
+		break;
 	case Command::list:
-		return ListParseResponse(successful, reply);
+		ListParseResponse(result == FZ_REPLY_OK, reply);
+		break;
 	case Command::transfer:
-		return FileTransferParseResponse(successful, reply);
+		FileTransferParseResponse(result, reply);
+		break;
 	case Command::cwd:
-		return ChangeDirParseResponse(successful, reply);
+		ChangeDirParseResponse(result == FZ_REPLY_OK, reply);
+		break;
 	case Command::mkdir:
-		return MkdirParseResponse(successful, reply);
+		MkdirParseResponse(result == FZ_REPLY_OK, reply);
+		break;
 	case Command::del:
-		return DeleteParseResponse(successful, reply);
+		DeleteParseResponse(result == FZ_REPLY_OK, reply);
+		break;
 	case Command::removedir:
-		return RemoveDirParseResponse(successful, reply);
+		RemoveDirParseResponse(result == FZ_REPLY_OK, reply);
+		break;
 	case Command::chmod:
-		return ChmodParseResponse(successful, reply);
+		ChmodParseResponse(result == FZ_REPLY_OK, reply);
+		break;
 	case Command::rename:
-		return RenameParseResponse(successful, reply);
+		RenameParseResponse(result == FZ_REPLY_OK, reply);
+		break;
 	default:
 		LogMessage(MessageType::Debug_Warning, _T("No action for parsing replies to command %d"), (int)commandId);
-		return ResetOperation(FZ_REPLY_INTERNALERROR);
+		ResetOperation(FZ_REPLY_INTERNALERROR);
+		break;
 	}
 }
 
@@ -1571,7 +1606,7 @@ int CSftpControlSocket::FileTransfer(const wxString localFile, const CServerPath
 
 	int64_t size;
 	bool isLink;
-	if (CLocalFileSystem::GetFileInfo(pData->localFile, isLink, &size, 0, 0) == CLocalFileSystem::file)
+	if (fz::local_filesys::get_file_info(fz::to_native(pData->localFile), isLink, &size, 0, 0) == fz::local_filesys::file)
 		pData->localFileSize = size;
 
 	pData->opState = filetransfer_waitcwd;
@@ -1775,7 +1810,7 @@ int CSftpControlSocket::FileTransferSend()
 	}
 	else if (pData->opState == filetransfer_chmtime)
 	{
-		wxASSERT(pData->fileTime.IsValid());
+		wxASSERT(!pData->fileTime.empty());
 		if (pData->download)
 		{
 			LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("  filetransfer_chmtime during download"));
@@ -1785,7 +1820,7 @@ int CSftpControlSocket::FileTransferSend()
 
 		wxString quotedFilename = QuoteFilename(pData->remotePath.FormatFilename(pData->remoteFile, !pData->tryAbsolutePath));
 		// Y2K38
-		time_t ticks = pData->fileTime.GetTimeT();
+		time_t ticks = pData->fileTime.get_time_t();
 		wxString seconds = wxString::Format(_T("%d"), (int)ticks);
 		if (!SendCommand(_T("chmtime ") + seconds + _T(" ") + WildcardEscape(quotedFilename),
 			_T("chmtime ") + seconds + _T(" ") + quotedFilename))
@@ -1795,12 +1830,11 @@ int CSftpControlSocket::FileTransferSend()
 	return FZ_REPLY_WOULDBLOCK;
 }
 
-int CSftpControlSocket::FileTransferParseResponse(bool successful, const wxString& reply)
+int CSftpControlSocket::FileTransferParseResponse(int result, const wxString& reply)
 {
-	LogMessage(MessageType::Debug_Verbose, _T("FileTransferParseResponse()"));
+	LogMessage(MessageType::Debug_Verbose, _T("FileTransferParseResponse(%d)"), result);
 
-	if (!m_pCurOpData)
-	{
+	if (!m_pCurOpData) {
 		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("Empty m_pCurOpData"));
 		ResetOperation(FZ_REPLY_INTERNALERROR);
 		return FZ_REPLY_ERROR;
@@ -1809,21 +1843,21 @@ int CSftpControlSocket::FileTransferParseResponse(bool successful, const wxStrin
 	CSftpFileTransferOpData *pData = static_cast<CSftpFileTransferOpData *>(m_pCurOpData);
 
 	if (pData->opState == filetransfer_transfer) {
-		if (!successful) {
-			ResetOperation(FZ_REPLY_ERROR);
+		if (result != FZ_REPLY_OK) {
+			ResetOperation(result);
 			return FZ_REPLY_ERROR;
 		}
 
 		if (engine_.GetOptions().GetOptionVal(OPTION_PRESERVE_TIMESTAMPS)) {
 			if (pData->download) {
-				if (pData->fileTime.IsValid()) {
-					if (!CLocalFileSystem::SetModificationTime(pData->localFile, pData->fileTime))
+				if (!pData->fileTime.empty()) {
+					if (!fz::local_filesys::set_modification_time(fz::to_native(pData->localFile), pData->fileTime))
 						LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Warning, _T("Could not set modification time"));
 				}
 			}
 			else {
-				pData->fileTime = CLocalFileSystem::GetModificationTime(pData->localFile);
-				if (pData->fileTime.IsValid()) {
+				pData->fileTime = fz::local_filesys::get_modification_time(fz::to_native(pData->localFile));
+				if (!pData->fileTime.empty()) {
 					pData->opState = filetransfer_chmtime;
 					return SendNextCommand();
 				}
@@ -1831,7 +1865,7 @@ int CSftpControlSocket::FileTransferParseResponse(bool successful, const wxStrin
 		}
 	}
 	else if (pData->opState == filetransfer_mtime) {
-		if (successful && !reply.empty()) {
+		if (result == FZ_REPLY_OK && !reply.empty()) {
 			time_t seconds = 0;
 			bool parsed = true;
 			for (unsigned int i = 0; i < reply.Len(); ++i) {
@@ -1844,10 +1878,10 @@ int CSftpControlSocket::FileTransferParseResponse(bool successful, const wxStrin
 				seconds += c - '0';
 			}
 			if (parsed) {
-				CDateTime fileTime = CDateTime(seconds, CDateTime::seconds);
-				if (fileTime.IsValid()) {
+				fz::datetime fileTime = fz::datetime(seconds, fz::datetime::seconds);
+				if (!fileTime.empty()) {
 					pData->fileTime = fileTime;
-					pData->fileTime += duration::from_minutes(m_pCurrentServer->GetTimezoneOffset());
+					pData->fileTime += fz::duration::from_minutes(m_pCurrentServer->GetTimezoneOffset());
 				}
 			}
 		}
@@ -1880,17 +1914,13 @@ int CSftpControlSocket::DoClose(int nErrorCode /*=FZ_REPLY_DISCONNECTED*/)
 	engine_.GetRateLimiter().RemoveObject(this);
 
 	if (m_pProcess) {
-		m_pProcess->Kill();
+		m_pProcess->kill();
 	}
 
 	if (m_pInputThread) {
-		wxThread* pThread = m_pInputThread;
+		auto thread = m_pInputThread;
 		m_pInputThread = 0;
-
-		if (pThread) {
-			pThread->Wait(wxTHREAD_WAIT_BLOCK);
-			delete pThread;
-		}
+		delete thread;
 	}
 	if (m_pProcess) {
 		delete m_pProcess;
@@ -2127,8 +2157,8 @@ int CSftpControlSocket::DeleteParseResponse(bool successful, const wxString&)
 
 		engine_.GetDirectoryCache().RemoveFile(*m_pCurrentServer, pData->path, file);
 
-		auto const now = CDateTime::Now();
-		if (now.IsValid() && pData->m_time.IsValid() && (now - pData->m_time).get_seconds() >= 1) {
+		auto const now = fz::datetime::now();
+		if (!pData->m_time.empty() && (now - pData->m_time).get_seconds() >= 1) {
 			engine_.SendDirectoryListingNotification(pData->path, false, true, false);
 			pData->m_time = now;
 			pData->m_needSendListing = false;
@@ -2170,8 +2200,8 @@ int CSftpControlSocket::DeleteSend()
 		return FZ_REPLY_ERROR;
 	}
 
-	if (!pData->m_time.IsValid())
-		pData->m_time = CDateTime::Now();
+	if (pData->m_time.empty())
+		pData->m_time = fz::datetime::now();
 
 	engine_.GetDirectoryCache().InvalidateFile(*m_pCurrentServer, pData->path, file);
 
@@ -2633,9 +2663,9 @@ int CSftpControlSocket::ListCheckTimezoneDetection()
 	return FZ_REPLY_OK;
 }
 
-void CSftpControlSocket::operator()(CEventBase const& ev)
+void CSftpControlSocket::operator()(fz::event_base const& ev)
 {
-	if (Dispatch<CSftpEvent, CTerminateEvent>(ev, this,
+	if (fz::dispatch<CSftpEvent, CTerminateEvent>(ev, this,
 		&CSftpControlSocket::OnSftpEvent,
 		&CSftpControlSocket::OnTerminate)) {
 		return;
