@@ -1,18 +1,19 @@
 #ifndef __DIRECTORYLISTING_H__
 #define __DIRECTORYLISTING_H__
 
-#include "optional.h"
-#include "timeex.h"
+#include <libfilezilla/optional.hpp>
+#include <libfilezilla/shared.hpp>
+#include <libfilezilla/time.hpp>
 
 #include <map>
 
 class CDirentry
 {
 public:
-	wxString name;
-	wxLongLong size;
-	CRefcountObject<wxString> permissions;
-	CRefcountObject<wxString> ownerGroup;
+	std::wstring name;
+	int64_t size;
+	fz::shared_value<std::wstring> permissions;
+	fz::shared_value<std::wstring> ownerGroup;
 
 	enum _flags
 	{
@@ -26,6 +27,7 @@ public:
 	{
 		return (flags & flag_dir) != 0;
 	}
+
 	inline bool is_link() const
 	{
 		return (flags & flag_link) != 0;
@@ -38,37 +40,38 @@ public:
 
 	inline bool has_date() const
 	{
-		return time.IsValid();;
+		return !time.empty();
 	}
+
 	inline bool has_time() const
 	{
-		return time.IsValid() && time.GetAccuracy() >= CDateTime::hours;
+		return !time.empty() && time.get_accuracy() >= fz::datetime::hours;
 	}
+
 	inline bool has_seconds() const
 	{
-		return time.IsValid() && time.GetAccuracy() >= CDateTime::seconds;
+		return !time.empty() && time.get_accuracy() >= fz::datetime::seconds;
 	}
 
-	CSparseOptional<wxString> target; // Set to linktarget it link is true
+	fz::sparse_optional<std::wstring> target; // Set to linktarget it link is true
 
-	CDateTime time;
+	fz::datetime time;
 
 	wxString dump() const;
 	bool operator==(const CDirentry &op) const;
 };
 
-#include "refcount.h"
-
 class CDirectoryListing final
 {
 public:
 	typedef CDirentry value_type;
 
-	CDirectoryListing();
-	CDirectoryListing(const CDirectoryListing& listing);
+	CDirectoryListing() = default;
+	CDirectoryListing(CDirectoryListing const& listing) = default;
+	CDirectoryListing(CDirectoryListing && listing) noexcept = default;
 
-	CServerPath path;
-	CDirectoryListing& operator=(const CDirectoryListing &a);
+	CDirectoryListing& operator=(CDirectoryListing const&) = default;
+	CDirectoryListing& operator=(CDirectoryListing &&) noexcept = default;
 
 	const CDirentry& operator[](unsigned int index) const;
 
@@ -84,7 +87,8 @@ public:
 
 	void ClearFindMap();
 
-	CMonotonicTime m_firstListTime;
+	CServerPath path;
+	fz::monotonic_clock m_firstListTime;
 
 	enum
 	{
@@ -113,7 +117,7 @@ public:
 	//
 	// These bits should help the user interface to choose an appropriate sorting
 	// algorithm for modified listings
-	int m_flags;
+	int m_flags{};
 
 	int get_unsure_flags() const { return m_flags & unsure_mask; }
 	bool failed() const { return (m_flags & listing_failed) != 0; }
@@ -121,20 +125,20 @@ public:
 	bool has_perms() const { return (m_flags & listing_has_perms) != 0; }
 	bool has_usergroup() const { return (m_flags & listing_has_usergroup) != 0; }
 
-	void Assign(std::deque<CRefcountObject<CDirentry>> & entries);
+	void Assign(std::deque<fz::shared_value<CDirentry>> & entries);
 
 	bool RemoveEntry(unsigned int index);
 
-	void GetFilenames(std::vector<wxString> &names) const;
+	void GetFilenames(std::vector<std::wstring> &names) const;
 
 protected:
 
-	CRefcountObject_Uninitialized<std::vector<CRefcountObject<CDirentry> > > m_entries;
+	fz::shared_optional<std::vector<fz::shared_value<CDirentry> > > m_entries;
 
-	mutable CRefcountObject_Uninitialized<std::multimap<wxString, unsigned int> > m_searchmap_case;
-	mutable CRefcountObject_Uninitialized<std::multimap<wxString, unsigned int> > m_searchmap_nocase;
+	mutable fz::shared_optional<std::multimap<std::wstring, unsigned int> > m_searchmap_case;
+	mutable fz::shared_optional<std::multimap<std::wstring, unsigned int> > m_searchmap_nocase;
 
-	unsigned int m_entryCount;
+	unsigned int m_entryCount{};
 };
 
 #endif
