@@ -7,42 +7,16 @@
 #include "Options.h"
 #include "wrapengine.h"
 #include "buildinfo.h"
-#ifdef __WXMSW__
-#include <wx/msw/registry.h> // Needed by CheckForWin2003FirewallBug
-#endif
 #include <wx/tokenzr.h>
 #include "cmdline.h"
 #include "welcome_dialog.h"
 #include <msgbox.h>
-#include "local_filesys.h"
 
-#include <wx/xrc/xh_animatctrl.h>
-#include <wx/xrc/xh_bmpbt.h>
-#include <wx/xrc/xh_bttn.h>
-#include <wx/xrc/xh_chckb.h>
-#include <wx/xrc/xh_chckl.h>
-#include <wx/xrc/xh_choic.h>
-#include <wx/xrc/xh_dlg.h>
-#include <wx/xrc/xh_gauge.h>
-#include <wx/xrc/xh_listb.h>
-#include <wx/xrc/xh_listc.h>
-#include <wx/xrc/xh_menu.h>
-#include <wx/xrc/xh_notbk.h>
-#include <wx/xrc/xh_panel.h>
-#include <wx/xrc/xh_radbt.h>
-#include <wx/xrc/xh_scwin.h>
-#include <wx/xrc/xh_sizer.h>
-#include <wx/xrc/xh_spin.h>
-#include <wx/xrc/xh_stbmp.h>
-#include <wx/xrc/xh_stbox.h>
-#include <wx/xrc/xh_stlin.h>
-#include <wx/xrc/xh_sttxt.h>
-#include "xh_text_ex.h"
-#include <wx/xrc/xh_tree.h>
-#include <wx/xrc/xh_hyperlink.h>
-#include "xh_toolb_ex.h"
+#include <libfilezilla/local_filesys.hpp>
+
+#include "wxfilesystem_blob_handler.h"
+#include "xrc_helper.h"
 #ifdef __WXMSW__
-#include <wx/socket.h>
 #include <wx/dynlib.h>
 #endif
 #ifdef WITH_LIBDBUS
@@ -78,67 +52,10 @@ CFileZillaApp::~CFileZillaApp()
 }
 
 #ifdef __WXMSW__
-bool IsServiceRunning(const wxString& serviceName)
-{
-	SC_HANDLE hScm = OpenSCManager(0, 0, GENERIC_READ);
-	if (!hScm) {
-		//wxMessageBoxEx(_T("OpenSCManager failed"));
-		return false;
-	}
-
-	SC_HANDLE hService = OpenService(hScm, serviceName, GENERIC_READ);
-	if (!hService) {
-		CloseServiceHandle(hScm);
-		//wxMessageBoxEx(_T("OpenService failed"));
-		return false;
-	}
-
-	SERVICE_STATUS status;
-	if (!ControlService(hService, SERVICE_CONTROL_INTERROGATE, &status)) {
-		CloseServiceHandle(hService);
-		CloseServiceHandle(hScm);
-		//wxMessageBoxEx(_T("ControlService failed"));
-		return false;
-	}
-
-	CloseServiceHandle(hService);
-	CloseServiceHandle(hScm);
-
-	if (status.dwCurrentState == 0x07 || status.dwCurrentState == 0x01)
-		return false;
-
-	return true;
-}
-
-bool CheckForWin2003FirewallBug()
-{
-	const wxString os = ::wxGetOsDescription();
-	if (os.Find(_T("Windows Server 2003")) == -1)
-		return false;
-
-	if (!IsServiceRunning(_T("SharedAccess")))
-		return false;
-
-	if (!IsServiceRunning(_T("ALG")))
-		return false;
-
-	wxRegKey key(_T("HKEY_LOCAL_MACHINE\\SYSTEM\\CurrentControlSet\\Services\\SharedAccess\\Parameters\\FirewallPolicy\\StandardProfile"));
-	if (!key.Exists() || !key.Open(wxRegKey::Read))
-		return false;
-
-	long value = 0;
-	if (!key.HasValue(_T("EnableFirewall")) || !key.QueryValue(_T("EnableFirewall"), &value))
-		return false;
-
-	if (!value)
-		return false;
-
-	return true;
-}
-
+namespace {
 extern "C"
 {
-	typedef HRESULT (WINAPI *t_SetCurrentProcessExplicitAppUserModelID)(PCWSTR AppID);
+	typedef HRESULT(WINAPI *t_SetCurrentProcessExplicitAppUserModelID)(PCWSTR AppID);
 }
 
 static void SetAppId()
@@ -159,14 +76,33 @@ static void SetAppId()
 	pSetCurrentProcessExplicitAppUserModelID(_T("FileZilla.Client.AppID"));
 }
 
+static bool InitWinsock()
+{
+	WSADATA d{};
+	int res = WSAStartup((2 << 8) | 8, &d);
+	if (res != 0) {
+		int err = WSAGetLastError();
+		wxString msg = wxString::Format(_("Could not initialize Winsock (%d): %s"), err, wxSysErrorMsg(err));
+		wxMessageBoxEx(msg, _("Failed to initialize networking"), wxICON_EXCLAMATION);
+		return false;
+	}
+
+	return true;
+}
+
+static void UninitWinsock()
+{
+	WSACleanup();
+}
+}
+
 #endif //__WXMSW__
 
 void CFileZillaApp::InitLocale()
 {
 	wxString language = COptions::Get()->GetOption(OPTION_LANGUAGE);
 	const wxLanguageInfo* pInfo = wxLocale::FindLanguageInfo(language);
-	if (!language.empty())
-	{
+	if (!language.empty()) {
 #ifdef __WXGTK__
 		if (CInitializer::error) {
 			wxString error;
@@ -205,9 +141,11 @@ void CFileZillaApp::InitLocale()
 			for( language = GetFallbackLocale(language); !language.empty(); language = GetFallbackLocale(language) ) {
 				const wxLanguageInfo* fallbackInfo = wxLocale::FindLanguageInfo(language);
 				if( fallbackInfo && SetLocale(fallbackInfo->Language )) {
+					COptions::Get()->SetOption(OPTION_LANGUAGE, language);
 					return;
 				}
 			}
+			COptions::Get()->SetOption(OPTION_LANGUAGE, wxString());
 			if (pInfo && !pInfo->Description.empty())
 				wxMessageBoxEx(wxString::Format(_("Failed to set language to %s (%s), using default system language"), pInfo->Description, language), _("Failed to change language"), wxICON_EXCLAMATION);
 			else
@@ -224,8 +162,9 @@ bool CFileZillaApp::OnInit()
 	srand( (unsigned)time( NULL ) );
 
 #ifdef __WXMSW__
-	// Need to call WSAStartup. Let wx do that for us
-	wxSocketBase::Initialize();
+	if (!InitWinsock()) {
+		return false;
+	}
 
 	SetAppId();
 #endif
@@ -292,21 +231,6 @@ USE AT OWN RISK"), _T("Important Information"));
 
 	CheckExistsFzsftp();
 
-#ifdef __WXMSW__
-	if (CheckForWin2003FirewallBug()) {
-		const wxString& error = _("Warning!\n\nA bug in Windows causes problems with FileZilla\n\n\
-The bug occurs if you have\n\
-- Windows Server 2003 or XP 64\n\
-- Windows Firewall enabled\n\
-- Application Layer Gateway service enabled\n\
-See http://support.microsoft.com/kb/931130 for background information.\n\n\
-Unless you either disable Windows Firewall or the Application Layer Gateway service,\n\
-FileZilla will timeout on big transfers.\
-");
-		wxMessageBoxEx(error, _("Operating system problem detected"), wxICON_EXCLAMATION);
-	}
-#endif
-
 	// Turn off idle events, we don't need them
 	wxIdleEvent::SetMode(wxIDLE_PROCESS_SPECIFIED);
 
@@ -317,7 +241,7 @@ FileZilla will timeout on big transfers.\
 #endif
 
 	// Load the text wrapping engine
-	m_pWrapEngine = make_unique<CWrapEngine>();
+	m_pWrapEngine = std::make_unique<CWrapEngine>();
 	m_pWrapEngine->LoadCache();
 
 	CMainFrame *frame = new CMainFrame();
@@ -343,7 +267,7 @@ int CFileZillaApp::OnExit()
 	CSessionManager::Uninit();
 #endif
 #ifdef __WXMSW__
-	wxSocketBase::Shutdown();
+	UninitWinsock();
 #endif
 	return wxApp::OnExit();
 }
@@ -491,51 +415,28 @@ bool CFileZillaApp::LoadResourceFiles()
 	pResource->SetFlags(pResource->GetFlags() | wxXRC_NO_RELOADING);
 #endif
 
-	pResource->AddHandler(new wxMenuXmlHandler);
-	pResource->AddHandler(new wxMenuBarXmlHandler);
-	pResource->AddHandler(new wxDialogXmlHandler);
-	pResource->AddHandler(new wxPanelXmlHandler);
-	pResource->AddHandler(new wxSizerXmlHandler);
-	pResource->AddHandler(new wxButtonXmlHandler);
-	pResource->AddHandler(new wxBitmapButtonXmlHandler);
-	pResource->AddHandler(new wxStaticTextXmlHandler);
-	pResource->AddHandler(new wxStaticBoxXmlHandler);
-	pResource->AddHandler(new wxStaticBitmapXmlHandler);
-	pResource->AddHandler(new wxTreeCtrlXmlHandler);
-	pResource->AddHandler(new wxListCtrlXmlHandler);
-	pResource->AddHandler(new wxCheckListBoxXmlHandler);
-	pResource->AddHandler(new wxChoiceXmlHandler);
-	pResource->AddHandler(new wxGaugeXmlHandler);
-	pResource->AddHandler(new wxCheckBoxXmlHandler);
-	pResource->AddHandler(new wxSpinCtrlXmlHandler);
-	pResource->AddHandler(new wxRadioButtonXmlHandler);
-	pResource->AddHandler(new wxNotebookXmlHandler);
-	pResource->AddHandler(new wxTextCtrlXmlHandlerEx);
-	pResource->AddHandler(new wxListBoxXmlHandler);
-	pResource->AddHandler(new wxToolBarXmlHandlerEx);
-	pResource->AddHandler(new wxStaticLineXmlHandler);
-	pResource->AddHandler(new wxScrolledWindowXmlHandler);
-	pResource->AddHandler(new wxHyperlinkCtrlXmlHandler);
-	pResource->AddHandler(new wxAnimationCtrlXmlHandler);
-	pResource->AddHandler(new wxStdDialogButtonSizerXmlHandler);
+	InitHandlers(*pResource);
 
-	if (CLocalFileSystem::GetFileType(m_resourceDir.GetPath() + _T("xrc/resources.xrc")) == CLocalFileSystem::file) {
+	if (fz::local_filesys::get_file_type(fz::to_native(m_resourceDir.GetPath() + _T("xrc/resources.xrc"))) == fz::local_filesys::file) {
 		pResource->LoadFile(m_resourceDir.GetPath() + _T("xrc/resources.xrc"));
 	}
 	else {
-		CLocalFileSystem fs;
+		fz::local_filesys fs;
 		wxString dir = m_resourceDir.GetPath() + _T("xrc/");
-		bool found = fs.BeginFindFiles(dir, false);
+		bool found = fs.begin_find_files(fz::to_native(dir), false);
 		while (found) {
-			wxString name;
-			found = fs.GetNextFile(name);
-			if (name.Right(4) != _T(".xrc")) {
+			fz::native_string name;
+			found = fs.get_next_file(name);
+			if (name.size() <= 4 || name.substr(name.size() - 4) != fzT(".xrc")) {
 				continue;
 			}
 			pResource->LoadFile(dir + name);
 		}
 	}
 
+	// Useful for XRC files with embedded image data.
+	wxFileSystem::AddHandler(new wxFileSystemBlobHandler);
+
 	return true;
 }
 
@@ -593,7 +494,7 @@ bool CFileZillaApp::LoadLocales()
 bool CFileZillaApp::SetLocale(int language)
 {
 	// First check if we can load the new locale
-	auto pLocale = make_unique<wxLocale>();
+	auto pLocale = std::make_unique<wxLocale>();
 	wxLogNull log;
 	pLocale->Init(language);
 	if (!pLocale->IsOk() || !pLocale->AddCatalog(_T("filezilla"))) {
@@ -606,7 +507,7 @@ bool CFileZillaApp::SetLocale(int language)
 	m_pLocale.reset();
 
 	// Finally load new one
-	pLocale = make_unique<wxLocale>();
+	pLocale = std::make_unique<wxLocale>();
 	pLocale->Init(language);
 	if (!pLocale->IsOk() || !pLocale->AddCatalog(_T("filezilla"))) {
 		return false;
@@ -776,7 +677,7 @@ extern "C" BOOL CALLBACK EnumWindowCallback(HWND hwnd, LPARAM)
 int CFileZillaApp::ProcessCommandLine()
 {
 	AddStartupProfileRecord(_T("CFileZillaApp::ProcessCommandLine"));
-	m_pCommandLine = make_unique<CCommandLine>(argc, argv);
+	m_pCommandLine = std::make_unique<CCommandLine>(argc, argv);
 	int res = m_pCommandLine->Parse() ? 1 : -1;
 
 	if (res > 0) {
