@@ -16,14 +16,17 @@
 #include <wx/msw/registry.h>
 #include "volume_enumerator.h"
 #endif
-#include "edithandler.h"
 #include "dragdropmanager.h"
 #include "drop_target_ex.h"
-#include "local_filesys.h"
+#include "edithandler.h"
 #include "filelist_statusbar.h"
+#include "local_recursive_operation.h"
 #include "sizeformatting.h"
 #include "timeformatting.h"
 
+#include <libfilezilla/local_filesys.hpp>
+#include <libfilezilla/recursive_remove.hpp>
+
 class CLocalListViewDropTarget : public CScrollableDropTarget<wxListCtrlEx>
 {
 public:
@@ -81,7 +84,7 @@ public:
 				subdir = data->name;
 		}
 
-		CLocalPath dir = m_pLocalListView->m_pState->GetLocalDir();
+		CLocalPath dir = m_pLocalListView->m_state.GetLocalDir();
 		if (!subdir.empty())
 		{
 			if (!dir.ChangePath(subdir))
@@ -95,7 +98,7 @@ public:
 			return wxDragError;
 
 		if (m_pDataObject->GetReceivedFormat() == m_pFileDataObject->GetFormat())
-			m_pLocalListView->m_pState->HandleDroppedFiles(m_pFileDataObject, dir, def == wxDragCopy);
+			m_pLocalListView->m_state.HandleDroppedFiles(m_pFileDataObject, dir, def == wxDragCopy);
 		else
 		{
 			if (m_pRemoteDataObject->GetProcessId() != (int)wxGetProcessId())
@@ -104,13 +107,13 @@ public:
 				return wxDragNone;
 			}
 
-			if (!m_pLocalListView->m_pState->GetServer() || !m_pRemoteDataObject->GetServer().EqualsNoPass(*m_pLocalListView->m_pState->GetServer()))
+			if (!m_pLocalListView->m_state.GetServer() || !m_pRemoteDataObject->GetServer().EqualsNoPass(*m_pLocalListView->m_state.GetServer()))
 			{
 				wxMessageBoxEx(_("Drag&drop between different servers has not been implemented yet."));
 				return wxDragNone;
 			}
 
-			if (!m_pLocalListView->m_pState->DownloadDroppedFiles(m_pRemoteDataObject, dir))
+			if (!m_pLocalListView->m_state.DownloadDroppedFiles(m_pRemoteDataObject, dir))
 				return wxDragNone;
 		}
 
@@ -199,7 +202,7 @@ public:
 
 		const wxString& subdir = DoDisplayDropHighlight(wxPoint(x, y));
 
-		CLocalPath dir = m_pLocalListView->m_pState->GetLocalDir();
+		CLocalPath dir = m_pLocalListView->m_state.GetLocalDir();
 		if (subdir.empty()) {
 			const CDragDropManager* pDragDropManager = CDragDropManager::Get();
 			if (pDragDropManager && pDragDropManager->localParent == m_pLocalListView->m_dir)
@@ -258,14 +261,14 @@ BEGIN_EVENT_TABLE(CLocalListView, CFileListCtrl<CLocalFileData>)
 	EVT_MENU(XRCID("ID_CONTEXT_REFRESH"), CLocalListView::OnMenuRefresh)
 END_EVENT_TABLE()
 
-CLocalListView::CLocalListView(wxWindow* pParent, CState *pState, CQueueView *pQueue)
-	: CFileListCtrl<CLocalFileData>(pParent, pState, pQueue),
-	CStateEventHandler(pState)
+CLocalListView::CLocalListView(wxWindow* pParent, CState& state, CQueueView *pQueue)
+	: CFileListCtrl<CLocalFileData>(pParent, pQueue),
+	CStateEventHandler(state)
 {
 	wxGetApp().AddStartupProfileRecord(_T("CLocalListView::CLocalListView"));
-	m_pState->RegisterHandler(this, STATECHANGE_LOCAL_DIR);
-	m_pState->RegisterHandler(this, STATECHANGE_APPLYFILTER);
-	m_pState->RegisterHandler(this, STATECHANGE_LOCAL_REFRESH_FILE);
+	m_state.RegisterHandler(this, STATECHANGE_LOCAL_DIR);
+	m_state.RegisterHandler(this, STATECHANGE_APPLYFILTER);
+	m_state.RegisterHandler(this, STATECHANGE_LOCAL_REFRESH_FILE);
 
 	m_dropTarget = -1;
 
@@ -304,6 +307,7 @@ CLocalListView::~CLocalListView()
 
 bool CLocalListView::DisplayDir(CLocalPath const& dirname)
 {
+	
 	CancelLabelEdit();
 
 	wxString focused;
@@ -317,7 +321,7 @@ bool CLocalListView::DisplayDir(CLocalPath const& dirname)
 			ExitComparisonMode();
 
 		ClearSelection();
-		focused = m_pState->GetPreviouslyVisitedLocalSubdir();
+		focused = m_state.GetPreviouslyVisitedLocalSubdir();
 		ensureVisible = !focused.empty();
 		if (focused.empty())
 			focused = _T("..");
@@ -371,9 +375,9 @@ bool CLocalListView::DisplayDir(CLocalPath const& dirname)
 regular_dir:
 #endif
 		CFilterManager filter;
-		CLocalFileSystem local_filesystem;
+		fz::local_filesys local_filesys;
 
-		if (!local_filesystem.BeginFindFiles(m_dir.GetPath(), false)) {
+		if (!local_filesys.begin_find_files(fz::to_native(m_dir.GetPath()), false)) {
 			SetItemCount(1);
 			return false;
 		}
@@ -387,12 +391,14 @@ regular_dir:
 		int num = m_fileData.size();
 		CLocalFileData data;
 		bool wasLink;
-		while (local_filesystem.GetNextFile(data.name, wasLink, data.dir, &data.size, &data.time, &data.attributes)) {
-			if (data.name.empty()) {
+		fz::native_string name;
+		while (local_filesys.get_next_file(name, wasLink, data.dir, &data.size, &data.time, &data.attributes)) {
+			if (name.empty()) {
 				wxGetApp().DisplayEncodingWarning();
 				continue;
 			}
 
+			data.name = name;
 			m_fileData.push_back(data);
 			if (!filter.FilenameFiltered(data.name, m_dir.GetPath(), data.dir, data.size, true, data.attributes, data.time)) {
 				if (data.dir)
@@ -519,7 +525,7 @@ void CLocalListView::OnItemActivated(wxListEvent &event)
 			// Enter action
 
 			wxString error;
-			if (!m_pState->SetLocalDir(data->name, &error))
+			if (!m_state.SetLocalDir(data->name, &error))
 			{
 				if (!error.empty())
 					wxMessageBoxEx(error, _("Failed to change directory"), wxICON_INFORMATION);
@@ -556,14 +562,14 @@ void CLocalListView::OnItemActivated(wxListEvent &event)
 		return;
 	}
 
-	const CServer* pServer = m_pState->GetServer();
+	const CServer* pServer = m_state.GetServer();
 	if (!pServer)
 	{
 		wxBell();
 		return;
 	}
 
-	CServerPath path = m_pState->GetRemotePath();
+	CServerPath path = m_state.GetRemotePath();
 	if (path.empty())
 	{
 		wxBell();
@@ -596,7 +602,7 @@ void CLocalListView::OnMenuEnter(wxCommandEvent &)
 	}
 
 	wxString error;
-	if (!m_pState->SetLocalDir(data->name, &error)) {
+	if (!m_state.SetLocalDir(data->name, &error)) {
 		if (!error.empty())
 			wxMessageBoxEx(error, _("Failed to change directory"), wxICON_INFORMATION);
 		else
@@ -617,7 +623,7 @@ void CLocalListView::DisplayDrives()
 
 		CLocalFileData data;
 		data.name = drive;
-		data.label = CSparseOptional<wxString>(data.name);
+		data.label = fz::sparse_optional<wxString>(data.name);
 		data.dir = true;
 		data.size = -1;
 
@@ -673,7 +679,7 @@ void CLocalListView::DisplayShares(wxString computer)
 			CLocalFileData data;
 			data.name = p->shi1_netname;
 #ifdef __WXMSW__
-			data.label = CSparseOptional<wxString>(data.name);
+			data.label = fz::sparse_optional<wxString>(data.name);
 #endif
 			data.dir = true;
 			data.size = -1;
@@ -719,8 +725,7 @@ CFileListCtrl<CLocalFileData>::CSortComparisonObject CLocalListView::GetSortComp
 	CFileListCtrlSortBase::DirSortMode dirSortMode = GetDirSortMode();
 	CFileListCtrlSortBase::NameSortMode nameSortMode = GetNameSortMode();
 
-	if (!m_sortDirection)
-	{
+	if (!m_sortDirection) {
 		if (m_sortColumn == 1)
 			return CFileListCtrl<CLocalFileData>::CSortComparisonObject(new CFileListCtrlSortSize<std::vector<CLocalFileData>, CLocalFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 		else if (m_sortColumn == 2)
@@ -730,8 +735,7 @@ CFileListCtrl<CLocalFileData>::CSortComparisonObject CLocalListView::GetSortComp
 		else
 			return CFileListCtrl<CLocalFileData>::CSortComparisonObject(new CFileListCtrlSortName<std::vector<CLocalFileData>, CLocalFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 	}
-	else
-	{
+	else {
 		if (m_sortColumn == 1)
 			return CFileListCtrl<CLocalFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortSize<std::vector<CLocalFileData>, CLocalFileData>, CLocalFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 		else if (m_sortColumn == 2)
@@ -745,8 +749,7 @@ CFileListCtrl<CLocalFileData>::CSortComparisonObject CLocalListView::GetSortComp
 
 void CLocalListView::OnContextMenu(wxContextMenuEvent& event)
 {
-	if (GetEditControl())
-	{
+	if (GetEditControl()) {
 		event.Skip();
 		return;
 	}
@@ -755,9 +758,8 @@ void CLocalListView::OnContextMenu(wxContextMenuEvent& event)
 	if (!pMenu)
 		return;
 
-	const bool connected = m_pState->IsRemoteConnected();
-	if (!connected)
-	{
+	const bool connected = m_state.IsRemoteConnected();
+	if (!connected) {
 		pMenu->Enable(XRCID("ID_EDIT"), COptions::Get()->GetOptionVal(OPTION_EDIT_TRACK_LOCAL) == 0);
 		pMenu->Enable(XRCID("ID_UPLOAD"), false);
 		pMenu->Enable(XRCID("ID_ADDTOQUEUE"), false);
@@ -767,12 +769,10 @@ void CLocalListView::OnContextMenu(wxContextMenuEvent& event)
 	int count = 0;
 	int fillCount = 0;
 	bool selectedDir = false;
-	while (index != -1)
-	{
-		count++;
+	while (index != -1) {
+		++count;
 		const CLocalFileData* const data = GetData(index);
-		if (!data || (!index && m_hasParent))
-		{
+		if (!data || (!index && m_hasParent)) {
 			pMenu->Enable(XRCID("ID_OPEN"), false);
 			pMenu->Enable(XRCID("ID_RENAME"), false);
 			pMenu->Enable(XRCID("ID_EDIT"), false);
@@ -783,8 +783,7 @@ void CLocalListView::OnContextMenu(wxContextMenuEvent& event)
 			selectedDir = true;
 		index = GetNextItem(index, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 	}
-	if (!count || fillCount == count)
-	{
+	if (!count || fillCount == count) {
 		pMenu->Delete(XRCID("ID_ENTER"));
 		pMenu->Enable(XRCID("ID_UPLOAD"), false);
 		pMenu->Enable(XRCID("ID_ADDTOQUEUE"), false);
@@ -792,19 +791,22 @@ void CLocalListView::OnContextMenu(wxContextMenuEvent& event)
 		pMenu->Enable(XRCID("ID_RENAME"), false);
 		pMenu->Enable(XRCID("ID_EDIT"), false);
 	}
-	else if (count > 1)
-	{
+	else if (count > 1) {
 		pMenu->Delete(XRCID("ID_ENTER"));
 		pMenu->Enable(XRCID("ID_RENAME"), false);
 	}
-	else
-	{
+	else {
 		// Exactly one item selected
 		if (!selectedDir)
 			pMenu->Delete(XRCID("ID_ENTER"));
 	}
-	if (selectedDir)
+	if (selectedDir) {
 		pMenu->Enable(XRCID("ID_EDIT"), false);
+		if (m_state.GetLocalRecursiveOperation() && m_state.GetLocalRecursiveOperation()->IsActive()) {
+			pMenu->Enable(XRCID("ID_UPLOAD"), false);
+			pMenu->Enable(XRCID("ID_ADDTOQUEUE"), false);
+		}
+	}
 
 	PopupMenu(pMenu);
 	delete pMenu;
@@ -812,7 +814,7 @@ void CLocalListView::OnContextMenu(wxContextMenuEvent& event)
 
 void CLocalListView::OnMenuUpload(wxCommandEvent& event)
 {
-	const CServer* pServer = m_pState->GetServer();
+	const CServer* pServer = m_state.GetServer();
 	if (!pServer) {
 		wxBell();
 		return;
@@ -822,6 +824,14 @@ void CLocalListView::OnMenuUpload(wxCommandEvent& event)
 
 	bool queue_only = event.GetId() == XRCID("ID_ADDTOQUEUE");
 
+	auto recursiveOperation = m_state.GetLocalRecursiveOperation();
+	if (!recursiveOperation || recursiveOperation->IsActive()) {
+		wxBell();
+		return;
+	}
+
+	local_recursion_root root;
+
 	long item = -1;
 	for (;;) {
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
@@ -837,28 +847,38 @@ void CLocalListView::OnMenuUpload(wxCommandEvent& event)
 		if (data->comparison_flags == fill)
 			continue;
 
-		CServerPath path = m_pState->GetRemotePath();
-		if (path.empty()) {
+		CServerPath remotePath = m_state.GetRemotePath();
+		if (remotePath.empty()) {
 			wxBell();
 			break;
 		}
 
 		if (data->dir) {
-			if (!path.ChangePath(data->name)) {
+			CLocalPath localPath = m_dir;
+			if (!localPath.ChangePath(data->name)) {
+				continue;
+			}
+			if (!remotePath.ChangePath(data->name)) {
 				continue;
 			}
 
-			CLocalPath localPath(m_dir);
-			localPath.AddSegment(data->name);
-			m_pQueue->QueueFolder(event.GetId() == XRCID("ID_ADDTOQUEUE"), false, localPath, path, *pServer);
+			root.add_dir_to_visit(localPath, remotePath);
 		}
 		else {
-			m_pQueue->QueueFile(queue_only, false, data->name, wxEmptyString, m_dir, path, *pServer, data->size);
+			m_pQueue->QueueFile(queue_only, false, data->name, wxEmptyString, m_dir, remotePath, *pServer, data->size);
 			added = true;
 		}
 	}
+
 	if (added)
 		m_pQueue->QueueFile_Finish(!queue_only);
+
+	if (!root.empty()) {
+		recursiveOperation->AddRecursionRoot(std::move(root));
+		CFilterManager filter;
+		recursiveOperation->StartRecursiveOperation(queue_only ? CRecursiveOperation::recursive_addtoqueue : CRecursiveOperation::recursive_transfer,
+			filter.GetActiveFilters(true));
+	}
 }
 
 // Create a new Directory
@@ -866,7 +886,7 @@ void CLocalListView::OnMenuMkdir(wxCommandEvent&)
 {
 	wxString newdir = MenuMkdir();
 	if (!newdir.empty()) {
-		m_pState->RefreshLocal();
+		m_state.RefreshLocal();
 	}
 }
 
@@ -880,7 +900,7 @@ void CLocalListView::OnMenuMkdirChgDir(wxCommandEvent&)
 
 	// OnMenuEnter
 	wxString error;
-	if (!m_pState->SetLocalDir(newdir, &error))
+	if (!m_state.SetLocalDir(newdir, &error))
 	{
 		if (!error.empty())
 			wxMessageBoxEx(error, _("Failed to change directory"), wxICON_INFORMATION);
@@ -927,7 +947,7 @@ wxString CLocalListView::MenuMkdir()
 
 void CLocalListView::OnMenuDelete(wxCommandEvent&)
 {
-	std::list<wxString> pathsToDelete;
+	std::list<fz::native_string> pathsToDelete;
 	long item = -1;
 	while ((item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1) {
 		if (!item && m_hasParent)
@@ -940,12 +960,12 @@ void CLocalListView::OnMenuDelete(wxCommandEvent&)
 		if (data->comparison_flags == fill)
 			continue;
 
-		pathsToDelete.push_back(m_dir.GetPath() + data->name);
+		pathsToDelete.push_back(fz::to_native(m_dir.GetPath() + data->name));
 	}
-	if (!CLocalFileSystem::RecursiveDelete(pathsToDelete, this))
-		wxGetApp().DisplayEncodingWarning();
+	gui_recursive_remove rmd(this);
+	rmd.remove(pathsToDelete);
 
-	m_pState->SetLocalDir(m_dir);
+	m_state.SetLocalDir(m_dir);
 }
 
 void CLocalListView::OnMenuRename(wxCommandEvent&)
@@ -1016,7 +1036,7 @@ void CLocalListView::OnKeyDown(wxKeyEvent& event)
 
 bool CLocalListView::OnBeginRename(const wxListEvent& event)
 {
-	if (!m_pState->GetLocalDir().IsWriteable())
+	if (!m_state.GetLocalDir().IsWriteable())
 		return false;
 
 	if (event.GetIndex() == 0 && m_hasParent)
@@ -1038,7 +1058,7 @@ bool CLocalListView::OnAcceptRename(const wxListEvent& event)
 	if (event.GetLabel().empty())
 		return false;
 
-	if (!m_pState->GetLocalDir().IsWriteable())
+	if (!m_state.GetLocalDir().IsWriteable())
 		return false;
 
 	CLocalFileData *const data = GetData(event.GetIndex());
@@ -1060,7 +1080,7 @@ bool CLocalListView::OnAcceptRename(const wxListEvent& event)
 #ifdef __WXMSW__
 	data->label.clear();
 #endif
-	m_pState->RefreshLocal();
+	m_state.RefreshLocal();
 
 	return true;
 }
@@ -1239,14 +1259,13 @@ void CLocalListView::ReselectItems(const std::list<wxString>& selectedNames, wxS
 	}
 }
 
-void CLocalListView::OnStateChange(CState*, enum t_statechange_notifications notification, const wxString& data, const void*)
+void CLocalListView::OnStateChange(t_statechange_notifications notification, const wxString& data, const void*)
 {
 	if (notification == STATECHANGE_LOCAL_DIR)
-		DisplayDir(m_pState->GetLocalDir());
+		DisplayDir(m_state.GetLocalDir());
 	else if (notification == STATECHANGE_APPLYFILTER)
 		ApplyCurrentFilter();
-	else
-	{
+	else {
 		wxASSERT(notification == STATECHANGE_LOCAL_REFRESH_FILE);
 		RefreshFile(data);
 	}
@@ -1306,7 +1325,7 @@ void CLocalListView::OnBeginDrag(wxListEvent&)
 	if (!handled_internally && (res == wxDragCopy || res == wxDragMove)) {
 		// We only need to refresh local side if the operation got handled
 		// externally, the internal handlers do this for us already
-		m_pState->RefreshLocal();
+		m_state.RefreshLocal();
 	}
 }
 
@@ -1315,12 +1334,12 @@ void CLocalListView::RefreshFile(const wxString& file)
 	CLocalFileData data;
 
 	bool wasLink;
-	enum CLocalFileSystem::local_fileType type = CLocalFileSystem::GetFileInfo(m_dir.GetPath() + file, wasLink, &data.size, &data.time, &data.attributes);
-	if (type == CLocalFileSystem::unknown)
+	enum fz::local_filesys::type type = fz::local_filesys::get_file_info(fz::to_native(m_dir.GetPath() + file), wasLink, &data.size, &data.time, &data.attributes);
+	if (type == fz::local_filesys::unknown)
 		return;
 
 	data.name = file;
-	data.dir = type == CLocalFileSystem::dir;
+	data.dir = type == fz::local_filesys::dir;
 
 	CFilterManager filter;
 	if (filter.FilenameFiltered(data.name, m_dir.GetPath(), data.dir, data.size, true, data.attributes, data.time))
@@ -1396,10 +1415,11 @@ void CLocalListView::RefreshFile(const wxString& file)
 	wxString focused;
 	std::list<wxString> selectedNames;
 	if (IsComparing()) {
-		wxASSERT(!m_originalIndexMapping.empty());
 		selectedNames = RememberSelectedItems(focused);
-		m_indexMapping.clear();
-		m_originalIndexMapping.swap(m_indexMapping);
+		if (!m_originalIndexMapping.empty()) {
+			m_indexMapping.clear();
+			m_originalIndexMapping.swap(m_indexMapping);
+		}
 	}
 
 	// Insert new entry
@@ -1495,7 +1515,7 @@ void CLocalListView::StartComparison()
 	}
 }
 
-bool CLocalListView::GetNextFile(wxString& name, bool& dir, wxLongLong& size, CDateTime& date)
+bool CLocalListView::get_next_file(wxString& name, bool& dir, int64_t& size, fz::datetime& date)
 {
 	if (++m_comparisonIndex >= (int)m_originalIndexMapping.size())
 		return false;
@@ -1529,7 +1549,7 @@ void CLocalListView::FinishComparison()
 	pOther->ScrollTopItem(GetTopItem());
 }
 
-bool CLocalListView::CanStartComparison(wxString*)
+bool CLocalListView::CanStartComparison()
 {
 	return true;
 }
@@ -1576,16 +1596,16 @@ void CLocalListView::OnMenuEdit(wxCommandEvent&)
 	CServer server;
 	CServerPath path;
 
-	if (!m_pState->GetServer()) {
+	if (!m_state.GetServer()) {
 		if (COptions::Get()->GetOptionVal(OPTION_EDIT_TRACK_LOCAL)) {
 			wxMessageBoxEx(_("Cannot edit file, not connected to any server."), _("Editing failed"), wxICON_EXCLAMATION);
 			return;
 		}
 	}
 	else {
-		server = *m_pState->GetServer();
+		server = *m_state.GetServer();
 
-		path = m_pState->GetRemotePath();
+		path = m_state.GetRemotePath();
 		if (path.empty()) {
 			wxMessageBoxEx(_("Cannot edit file, remote path unknown."), _("Editing failed"), wxICON_EXCLAMATION);
 			return;
@@ -1747,7 +1767,7 @@ void CLocalListView::OnVolumesEnumerated(wxCommandEvent& event)
 		if (item >= m_indexMapping.size())
 			continue;
 
-		m_fileData[index].label = CSparseOptional<wxString>(drive + _T(" (") + iter->volumeName + _T(")"));
+		m_fileData[index].label = fz::sparse_optional<wxString>(drive + _T(" (") + iter->volumeName + _T(")"));
 
 		RefreshItem(item);
 	}
@@ -1757,7 +1777,7 @@ void CLocalListView::OnVolumesEnumerated(wxCommandEvent& event)
 
 void CLocalListView::OnMenuRefresh(wxCommandEvent&)
 {
-	m_pState->RefreshLocal();
+	m_state.RefreshLocal();
 }
 
 void CLocalListView::OnNavigationEvent(bool forward)
@@ -1771,7 +1791,7 @@ void CLocalListView::OnNavigationEvent(bool forward)
 		}
 
 		wxString error;
-		if (!m_pState->SetLocalDir(_T(".."), &error))
+		if (!m_state.SetLocalDir(_T(".."), &error))
 		{
 			if (!error.empty())
 				wxMessageBoxEx(error, _("Failed to change directory"), wxICON_INFORMATION);
