@@ -7,11 +7,12 @@
 #include <wx/dnd.h>
 #include "dndobjects.h"
 #include "inputdialog.h"
-#include "local_filesys.h"
 #include "dragdropmanager.h"
 #include "drop_target_ex.h"
 #include "Options.h"
 
+#include <libfilezilla/local_filesys.hpp>
+
 #ifdef __WXMSW__
 #include <wx/msw/registry.h>
 #include <shlobj.h>
@@ -19,6 +20,8 @@
 #include "volume_enumerator.h"
 #endif
 
+#include <algorithm>
+
 class CTreeItemData : public wxTreeItemData
 {
 public:
@@ -99,7 +102,7 @@ public:
 			pDragDropManager->pDropTarget = m_pLocalTreeView;
 
 		if (m_pDataObject->GetReceivedFormat() == m_pFileDataObject->GetFormat())
-			m_pLocalTreeView->m_pState->HandleDroppedFiles(m_pFileDataObject, path, def == wxDragCopy);
+			m_pLocalTreeView->m_state.HandleDroppedFiles(m_pFileDataObject, path, def == wxDragCopy);
 		else
 		{
 			if (m_pRemoteDataObject->GetProcessId() != (int)wxGetProcessId())
@@ -108,13 +111,13 @@ public:
 				return wxDragNone;
 			}
 
-			if (!m_pLocalTreeView->m_pState->GetServer() || !m_pRemoteDataObject->GetServer().EqualsNoPass(*m_pLocalTreeView->m_pState->GetServer()))
+			if (!m_pLocalTreeView->m_state.GetServer() || !m_pRemoteDataObject->GetServer().EqualsNoPass(*m_pLocalTreeView->m_state.GetServer()))
 			{
 				wxMessageBoxEx(_("Drag&drop between different servers has not been implemented yet."));
 				return wxDragNone;
 			}
 
-			if (!m_pLocalTreeView->m_pState->DownloadDroppedFiles(m_pRemoteDataObject, path))
+			if (!m_pLocalTreeView->m_state.DownloadDroppedFiles(m_pRemoteDataObject, path))
 				return wxDragNone;
 		}
 
@@ -233,10 +236,10 @@ EVT_CHAR(CLocalTreeView::OnChar)
 EVT_MENU(XRCID("ID_OPEN"), CLocalTreeView::OnMenuOpen)
 END_EVENT_TABLE()
 
-CLocalTreeView::CLocalTreeView(wxWindow* parent, wxWindowID id, CState *pState, CQueueView *pQueueView)
+CLocalTreeView::CLocalTreeView(wxWindow* parent, wxWindowID id, CState& state, CQueueView *pQueueView)
 	: wxTreeCtrlEx(parent, id, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxTR_EDIT_LABELS | wxTR_LINES_AT_ROOT | wxTR_HAS_BUTTONS | wxNO_BORDER),
 	CSystemImageList(16),
-	CStateEventHandler(pState),
+	CStateEventHandler(state),
 	m_pQueueView(pQueueView)
 {
 	wxGetApp().AddStartupProfileRecord(_T("CLocalTreeView::CLocalTreeView"));
@@ -244,8 +247,8 @@ CLocalTreeView::CLocalTreeView(wxWindow* parent, wxWindowID id, CState *pState,
 	SetFont(wxSystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT));
 #endif
 
-	pState->RegisterHandler(this, STATECHANGE_LOCAL_DIR);
-	pState->RegisterHandler(this, STATECHANGE_APPLYFILTER);
+	state.RegisterHandler(this, STATECHANGE_LOCAL_DIR);
+	state.RegisterHandler(this, STATECHANGE_APPLYFILTER);
 
 	SetImageList(GetSystemImageList());
 
@@ -425,14 +428,12 @@ bool CLocalTreeView::DisplayDrives(wxTreeItemId parent)
 
 void CLocalTreeView::DisplayDir(wxTreeItemId parent, const wxString& dirname, const wxString& knownSubdir /*=_T("")*/)
 {
-	CLocalFileSystem local_filesystem;
+	fz::local_filesys local_filesys;
 
 	{
 		wxLogNull log;
-		if (!local_filesystem.BeginFindFiles(dirname, true))
-		{
-			if (!knownSubdir.empty())
-			{
+		if (!local_filesys.begin_find_files(fz::to_native(dirname), true)) {
+			if (!knownSubdir.empty()) {
 				wxTreeItemId item = GetSubdir(parent, knownSubdir);
 				if (item != wxTreeItemId())
 					return;
@@ -466,35 +467,34 @@ void CLocalTreeView::DisplayDir(wxTreeItemId parent, const wxString& dirname, co
 
 	bool matchedKnown = false;
 
-	wxString file;
+	fz::native_string file;
 	bool wasLink;
 	int attributes;
 	bool is_dir;
-	const wxLongLong size(-1);
-	CDateTime date;
-	while (local_filesystem.GetNextFile(file, wasLink, is_dir, 0, &date, &attributes))
-	{
+	static int64_t const size(-1);
+	fz::datetime date;
+	while (local_filesys.get_next_file(file, wasLink, is_dir, 0, &date, &attributes)) {
 		wxASSERT(is_dir);
-		if (file.empty())
-		{
+		if (file.empty()) {
 			wxGetApp().DisplayEncodingWarning();
 			continue;
 		}
+		wxString file_wx = file;
 
-		wxString fullName = dirname + file;
+		wxString fullName = dirname + file_wx;
 #ifdef __WXMSW__
-		if (file.CmpNoCase(knownSubdir))
+		if (file_wx.CmpNoCase(knownSubdir))
 #else
-		if (file != knownSubdir)
+		if (file_wx != knownSubdir)
 #endif
 		{
-			if (filter.FilenameFiltered(file, dirname, true, size, true, attributes, date))
+			if (filter.FilenameFiltered(file_wx, dirname, true, size, true, attributes, date))
 				continue;
 		}
 		else
 			matchedKnown = true;
 
-		wxTreeItemId item = AppendItem(parent, file, GetIconIndex(iconType::dir, fullName),
+		wxTreeItemId item = AppendItem(parent, file_wx, GetIconIndex(iconType::dir, fullName),
 #ifdef __WXMSW__
 				-1
 #else
@@ -505,8 +505,7 @@ void CLocalTreeView::DisplayDir(wxTreeItemId parent, const wxString& dirname, co
 		CheckSubdirStatus(item, fullName);
 	}
 
-	if (!matchedKnown && !knownSubdir.empty())
-	{
+	if (!matchedKnown && !knownSubdir.empty()) {
 		const wxString fullName = dirname + knownSubdir;
 		wxTreeItemId item = AppendItem(parent, knownSubdir, GetIconIndex(iconType::dir, fullName),
 #ifdef __WXMSW__
@@ -528,29 +527,28 @@ wxString CLocalTreeView::HasSubdir(const wxString& dirname)
 
 	CFilterManager filter;
 
-	CLocalFileSystem local_filesystem;
-	if (!local_filesystem.BeginFindFiles(dirname, true))
+	fz::local_filesys local_filesys;
+	if (!local_filesys.begin_find_files(fz::to_native(dirname), true))
 		return wxString();
 
-	wxString file;
+	fz::native_string file;
 	bool wasLink;
 	int attributes;
 	bool is_dir;
-	const wxLongLong size(-1);
-	CDateTime date;
-	while (local_filesystem.GetNextFile(file, wasLink, is_dir, 0, &date, &attributes))
-	{
+	static int64_t const size(-1);
+	fz::datetime date;
+	while (local_filesys.get_next_file(file, wasLink, is_dir, 0, &date, &attributes)) {
 		wxASSERT(is_dir);
-		if (file.empty())
-		{
+		if (file.empty()) {
 			wxGetApp().DisplayEncodingWarning();
 			continue;
 		}
 
-		if (filter.FilenameFiltered(file, dirname, true, size, true, attributes, date))
+		wxString file_wx = file;
+		if (filter.FilenameFiltered(file_wx, dirname, true, size, true, attributes, date))
 			continue;
 
-		return file;
+		return file_wx;
 	}
 
 	return wxString();
@@ -560,17 +558,14 @@ wxTreeItemId CLocalTreeView::MakeSubdirs(wxTreeItemId parent, wxString dirname,
 {
 	const wxString& separator = wxFileName::GetPathSeparator();
 
-	while (!subDir.empty())
-	{
+	while (!subDir.empty()) {
 		int pos = subDir.Find(separator);
 		wxString segment;
-		if (pos == -1)
-		{
+		if (pos == -1) {
 			segment = subDir;
 			subDir = _T("");
 		}
-		else
-		{
+		else {
 			segment = subDir.Left(pos);
 			subDir = subDir.Mid(pos + 1);
 		}
@@ -716,8 +711,8 @@ void CLocalTreeView::RefreshListing()
 		dirsToCheck.pop_front();
 
 		// Step 1: Check if directory exists
-		CLocalFileSystem local_filesystem;
-		if (!local_filesystem.BeginFindFiles(dir.dir, true)) {
+		fz::local_filesys local_filesys;
+		if (!local_filesys.begin_find_files(fz::to_native(dir.dir), true)) {
 			// Dir does exist (listed in parent) but may not be accessible.
 			// Recurse into children anyhow, they might be accessible again.
 			wxTreeItemIdValue value;
@@ -731,32 +726,34 @@ void CLocalTreeView::RefreshListing()
 		}
 
 		// Step 2: Enumerate subdirectories on disk and sort them
-		wxArrayString dirs;
+		std::vector<wxString> dirs;
 
-		wxString file;
-		const wxLongLong size(-1);
+		fz::native_string file;
+		static int64_t const size(-1);
 		bool was_link;
 		bool is_dir;
 		int attributes;
-		CDateTime date;
-		while (local_filesystem.GetNextFile(file, was_link, is_dir, 0, &date, &attributes)) {
+		fz::datetime date;
+		while (local_filesys.get_next_file(file, was_link, is_dir, 0, &date, &attributes)) {
 			if (file.empty()) {
 				wxGetApp().DisplayEncodingWarning();
 				continue;
 			}
 
-			if (filter.FilenameFiltered(file, dir.dir, true, size, true, attributes, date))
+			wxString file_wx = file;
+			if (filter.FilenameFiltered(file_wx, dir.dir, true, size, true, attributes, date))
 				continue;
 
-			dirs.push_back(file);
+			dirs.push_back(file_wx);
 		}
-		dirs.Sort(CFileListCtrlSortBase::GetCmpFunction(m_nameSortMode));
+		auto const& sortFunc = CFileListCtrlSortBase::GetCmpFunction(m_nameSortMode);
+		std::sort(dirs.begin(), dirs.end(), [&](auto const& lhs, auto const& rhs) { return sortFunc(lhs, rhs) < 0; });
 
 		bool inserted = false;
 
 		// Step 3: Merge list of subdirectories with subtree.
 		wxTreeItemId child = GetLastChild(dir.item);
-		wxArrayString::reverse_iterator iter = dirs.rbegin();
+		auto iter = dirs.rbegin();
 		while (child || iter != dirs.rend()) {
 			int cmp;
 			if (child && iter != dirs.rend())
@@ -841,7 +838,7 @@ void CLocalTreeView::OnSelectionChanged(wxTreeEvent& event)
 	wxString dir = GetDirFromItem(item);
 
 	wxString error;
-	if (!m_pState->SetLocalDir(dir, &error)) {
+	if (!m_state.SetLocalDir(dir, &error)) {
 		if (!error.empty())
 			wxMessageBoxEx(error, _("Failed to change directory"), wxICON_INFORMATION);
 		else
@@ -852,10 +849,10 @@ void CLocalTreeView::OnSelectionChanged(wxTreeEvent& event)
 	}
 }
 
-void CLocalTreeView::OnStateChange(CState*, enum t_statechange_notifications notification, const wxString&, const void*)
+void CLocalTreeView::OnStateChange(t_statechange_notifications notification, const wxString&, const void*)
 {
 	if (notification == STATECHANGE_LOCAL_DIR)
-		SetDir(m_pState->GetLocalDir().GetPath());
+		SetDir(m_state.GetLocalDir().GetPath());
 	else {
 		wxASSERT(notification == STATECHANGE_APPLYFILTER);
 		RefreshListing();
@@ -908,7 +905,7 @@ void CLocalTreeView::OnBeginDrag(wxTreeEvent& event)
 	{
 		// We only need to refresh local side if the operation got handled
 		// externally, the internal handlers do this for us already
-		m_pState->RefreshLocal();
+		m_state.RefreshLocal();
 	}
 }
 
@@ -1037,10 +1034,12 @@ void CLocalTreeView::OnContextMenu(wxTreeEvent& event)
 	const bool hasParent = path.HasParent();
 	const bool writeable = path.IsWriteable();
 
-	const bool remoteConnected = m_pState->IsRemoteConnected() && !m_pState->GetRemotePath().empty();
+	const bool remoteConnected = m_state.IsRemoteConnected() && !m_state.GetRemotePath().empty();
 
-	pMenu->Enable(XRCID("ID_UPLOAD"), hasParent && remoteConnected);
-	pMenu->Enable(XRCID("ID_ADDTOQUEUE"), hasParent && remoteConnected);
+	bool const idle = m_state.GetLocalRecursiveOperation() && !m_state.GetLocalRecursiveOperation()->IsActive();
+
+	pMenu->Enable(XRCID("ID_UPLOAD"), hasParent && remoteConnected && idle);
+	pMenu->Enable(XRCID("ID_ADDTOQUEUE"), hasParent && remoteConnected && idle);
 	pMenu->Enable(XRCID("ID_MKDIR"), writeable);
 	pMenu->Enable(XRCID("ID_DELETE"), writeable && hasParent);
 	pMenu->Enable(XRCID("ID_RENAME"), writeable && hasParent);
@@ -1051,6 +1050,11 @@ void CLocalTreeView::OnContextMenu(wxTreeEvent& event)
 
 void CLocalTreeView::OnMenuUpload(wxCommandEvent& event)
 {
+	auto recursiveOperation = m_state.GetLocalRecursiveOperation();
+	if (recursiveOperation->IsActive()) {
+		return;
+	}
+
 	if (!m_contextMenuItem.IsOk())
 		return;
 
@@ -1059,18 +1063,26 @@ void CLocalTreeView::OnMenuUpload(wxCommandEvent& event)
 	if (!path.HasParent())
 		return;
 
-	if (!m_pState->IsRemoteConnected())
+	if (!m_state.IsRemoteConnected())
 		return;
 
-	const CServer server = *m_pState->GetServer();
-	CServerPath remotePath = m_pState->GetRemotePath();
+	const CServer server = *m_state.GetServer();
+	CServerPath remotePath = m_state.GetRemotePath();
 	if (remotePath.empty())
 		return;
 
 	if (!remotePath.ChangePath(GetItemText(m_contextMenuItem)))
 		return;
 
-	m_pQueueView->QueueFolder(event.GetId() == XRCID("ID_ADDTOQUEUE"), false, path, remotePath, server);
+	local_recursion_root root;
+	root.add_dir_to_visit(path, remotePath);
+	recursiveOperation->AddRecursionRoot(std::move(root));
+	
+	bool const queue_only = event.GetId() == XRCID("ID_ADDTOQUEUE");
+
+	CFilterManager filter;
+	recursiveOperation->StartRecursiveOperation(queue_only ? CRecursiveOperation::recursive_addtoqueue : CRecursiveOperation::recursive_transfer,
+		filter.GetActiveFilters(true));
 }
 
 // Create a new Directory
@@ -1079,7 +1091,7 @@ void CLocalTreeView::OnMenuMkdir(wxCommandEvent&)
 	wxString newdir = MenuMkdir();
 	if (!newdir.empty()) {
 		RefreshListing();
-		m_pState->RefreshLocal();
+		m_state.RefreshLocal();
 	}
 }
 
@@ -1093,7 +1105,7 @@ void CLocalTreeView::OnMenuMkdirChgDir(wxCommandEvent&)
 
 	// OnMenuEnter
 	wxString error;
-	if (!m_pState->SetLocalDir(newdir, &error))
+	if (!m_state.SetLocalDir(newdir, &error))
 	{
 		if (!error.empty())
 			wxMessageBoxEx(error, _("Failed to change directory"), wxICON_INFORMATION);
@@ -1183,8 +1195,8 @@ void CLocalTreeView::OnMenuDelete(wxCommandEvent&)
 	if (!local_path.HasParent() || !local_path.IsWriteable())
 		return;
 
-	if (!CLocalFileSystem::RecursiveDelete(path, this))
-		wxGetApp().DisplayEncodingWarning();
+	gui_recursive_remove rmd(this);
+	rmd.remove(fz::to_native(path));
 
 	wxTreeItemId item = GetSelection();
 	while (item && item != m_contextMenuItem)
@@ -1192,7 +1204,7 @@ void CLocalTreeView::OnMenuDelete(wxCommandEvent&)
 
 	if (!item) {
 		if (GetItemParent(m_contextMenuItem) == GetSelection())
-			m_pState->RefreshLocal();
+			m_state.RefreshLocal();
 		else
 			RefreshListing();
 		return;
@@ -1206,7 +1218,7 @@ void CLocalTreeView::OnMenuDelete(wxCommandEvent&)
 	else
 		path = path.Left(pos);
 
-	m_pState->SetLocalDir(path);
+	m_state.SetLocalDir(path);
 	RefreshListing();
 }
 
@@ -1298,7 +1310,7 @@ void CLocalTreeView::OnEndLabelEdit(wxTreeEvent& event)
 	}
 
 	if (item == currentSel) {
-		m_pState->SetLocalDir(parent + newName);
+		m_state.SetLocalDir(parent + newName);
 		return;
 	}
 
@@ -1316,7 +1328,7 @@ void CLocalTreeView::OnEndLabelEdit(wxTreeEvent& event)
 	}
 
 	// Current selection below renamed item
-	m_pState->SetLocalDir(parent + newName + sub);
+	m_state.SetLocalDir(parent + newName + sub);
 }
 
 void CLocalTreeView::OnChar(wxKeyEvent& event)
@@ -1337,7 +1349,7 @@ bool CLocalTreeView::CheckSubdirStatus(wxTreeItemId& item, const wxString& path)
 	wxTreeItemIdValue value;
 	wxTreeItemId child = GetFirstChild(item, value);
 
-	static const wxLongLong size(-1);
+	static int64_t const size(-1);
 
 #ifdef __WXMAC__
 	// By default, OS X has a list of servers mounted into /net,
@@ -1347,7 +1359,7 @@ bool CLocalTreeView::CheckSubdirStatus(wxTreeItemId& item, const wxString& path)
 		CFilterManager filter;
 
 		const int attributes = S_IRUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH;
-		if (!filter.FilenameFiltered(_T("localhost"), path, true, size, true, attributes, CDateTime()))
+		if (!filter.FilenameFiltered(_T("localhost"), path, true, size, true, attributes, fz::datetime()))
 		{
 			if (!child)
 				AppendItem(item, _T(""));
@@ -1364,13 +1376,13 @@ bool CLocalTreeView::CheckSubdirStatus(wxTreeItemId& item, const wxString& path)
 		if (pData) {
 			bool wasLink;
 			int attributes;
-			enum CLocalFileSystem::local_fileType type;
-			CDateTime date;
-			if (!path.empty() && path.Last() == CLocalFileSystem::path_separator)
-				type = CLocalFileSystem::GetFileInfo(path + pData->m_known_subdir, wasLink, 0, &date, &attributes);
+			enum fz::local_filesys::type type;
+			fz::datetime date;
+			if (!path.empty() && path.Last() == fz::local_filesys::path_separator)
+				type = fz::local_filesys::get_file_info(fz::to_native(path + pData->m_known_subdir), wasLink, 0, &date, &attributes);
 			else
-				type = CLocalFileSystem::GetFileInfo(path + CLocalFileSystem::path_separator + pData->m_known_subdir, wasLink, 0, &date, &attributes);
-			if (type == CLocalFileSystem::dir) {
+				type = fz::local_filesys::get_file_info(fz::to_native(path + fz::local_filesys::path_separator + pData->m_known_subdir), wasLink, 0, &date, &attributes);
+			if (type == fz::local_filesys::dir) {
 				CFilterManager filter;
 				if (!filter.FilenameFiltered(pData->m_known_subdir, path, true, size, true, attributes, date))
 					return true;
@@ -1422,8 +1434,8 @@ void CLocalTreeView::OnDevicechange(WPARAM wParam, LPARAM lParam)
 			mask <<= 1;
 		}
 
-		if (GetSelection() == m_drives && m_pState)
-			m_pState->RefreshLocal();
+		if (GetSelection() == m_drives)
+			m_state.RefreshLocal();
 	}
 }
 
@@ -1433,14 +1445,14 @@ wxTreeItemId CLocalTreeView::AddDrive(wxChar letter)
 	drive += _T(":");
 
 	long drivesToHide = CVolumeDescriptionEnumeratorThread::GetDrivesToHide();
-	if( CVolumeDescriptionEnumeratorThread::IsHidden(drive, drivesToHide) ) {
+	if( CVolumeDescriptionEnumeratorThread::IsHidden(drive.wc_str(), drivesToHide) ) {
 		return wxTreeItemId();
 	}
 
 	// Get the label of the drive
 	wxChar volumeName[501];
 	int oldErrorMode = SetErrorMode(SEM_FAILCRITICALERRORS);
-	BOOL res = GetVolumeInformation(drive + _T("\\"), volumeName, 500, 0, 0, 0, 0, 0);
+	BOOL res = GetVolumeInformation((drive + _T("\\")).wc_str(), volumeName, 500, 0, 0, 0, 0, 0);
 	SetErrorMode(oldErrorMode);
 
 	wxString itemLabel = drive;
