@@ -1,54 +1,56 @@
 #include <filezilla.h>
 #include "Mainfrm.h"
 
+#include "aboutdialog.h"
+#include "asyncrequestqueue.h"
+#include "auto_ascii_files.h"
+#include "bookmarks_dialog.h"
+#include "buildinfo.h"
+#include "clearprivatedata.h"
+#include "cmdline.h"
+#include "commandqueue.h"
+#include "conditionaldialog.h"
+#include "context_control.h"
+#include "defaultfileexistsdlg.h"
+#include "edithandler.h"
+#include "export.h"
+#include "filelist_statusbar.h"
+#include "filezillaapp.h"
+#include "filter.h"
+#include "import.h"
+#include "inputdialog.h"
+#include "led.h"
+#include "local_recursive_operation.h"
 #include "LocalListView.h"
 #include "LocalTreeView.h"
+#include "loginmanager.h"
+#include "manual_transfer.h"
+#include "menu_bar.h"
+#include "netconfwizard.h"
+#include "Options.h"
+#include "power_management.h"
 #include "queue.h"
+#include "quickconnectbar.h"
+#include "remote_recursive_operation.h"
 #include "RemoteListView.h"
 #include "RemoteTreeView.h"
+#include "search.h"
+#include "settings/settingsdialog.h"
+#include "sitemanager_dialog.h"
+#include "speedlimits_dialog.h"
+#include "splitter.h"
 #include "StatusView.h"
 #include "state.h"
-#include "Options.h"
-#include "asyncrequestqueue.h"
-#include "commandqueue.h"
-#include "led.h"
-#include "sitemanager_dialog.h"
-#include "settings/settingsdialog.h"
 #include "themeprovider.h"
-#include "filezillaapp.h"
+#include "toolbar.h"
+#include "update_dialog.h"
+#include "updater.h"
 #include "view.h"
 #include "viewheader.h"
-#include "aboutdialog.h"
-#include "filter.h"
-#include "netconfwizard.h"
-#include "quickconnectbar.h"
-#include "updater.h"
-#include "update_dialog.h"
-#include "defaultfileexistsdlg.h"
-#include "loginmanager.h"
-#include "conditionaldialog.h"
-#include "clearprivatedata.h"
-#include "export.h"
-#include "import.h"
-#include "recursive_operation.h"
-#include <wx/tokenzr.h>
-#include "edithandler.h"
-#include "inputdialog.h"
-#include "window_state_manager.h"
-#include "cmdline.h"
-#include "buildinfo.h"
-#include "filelist_statusbar.h"
-#include "manual_transfer.h"
-#include "auto_ascii_files.h"
-#include "splitter.h"
-#include "bookmarks_dialog.h"
-#include "search.h"
-#include "power_management.h"
 #include "welcome_dialog.h"
-#include "context_control.h"
-#include "speedlimits_dialog.h"
-#include "toolbar.h"
-#include "menu_bar.h"
+#include "window_state_manager.h"
+
+#include <wx/tokenzr.h>
 
 #ifdef __WXMSW__
 #include <wx/module.h>
@@ -117,7 +119,6 @@ bool HandleKeyboardCommand(wxCommandEvent& event, wxWindow& parent)
 BEGIN_EVENT_TABLE(CMainFrame, wxNavigationEnabled<wxFrame>)
 	EVT_SIZE(CMainFrame::OnSize)
 	EVT_MENU(wxID_ANY, CMainFrame::OnMenuHandler)
-	EVT_FZ_NOTIFICATION(wxID_ANY, CMainFrame::OnEngineEvent)
 	EVT_COMMAND(wxID_ANY, fzEVT_UPDATE_LED_TOOLTIP, CMainFrame::OnUpdateLedTooltip)
 	EVT_TOOL(XRCID("ID_TOOLBAR_DISCONNECT"), CMainFrame::OnDisconnect)
 	EVT_MENU(XRCID("ID_MENU_SERVER_DISCONNECT"), CMainFrame::OnDisconnect)
@@ -181,11 +182,10 @@ BEGIN_EVENT_TABLE(CMainFrame, wxNavigationEnabled<wxFrame>)
 	EVT_MENU(XRCID("ID_MENU_FILE_CLOSETAB"), CMainFrame::OnMenuCloseTab)
 END_EVENT_TABLE()
 
-class CMainFrameStateEventHandler : public CStateEventHandler
+class CMainFrameStateEventHandler : public CGlobalStateEventHandler
 {
 public:
 	CMainFrameStateEventHandler(CMainFrame* pMainFrame)
-		: CStateEventHandler(0)
 	{
 		m_pMainFrame = pMainFrame;
 
@@ -196,7 +196,7 @@ public:
 	}
 
 protected:
-	virtual void OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString&, const void*)
+	virtual void OnStateChange(CState* pState, t_statechange_notifications notification, const wxString&, const void*)
 	{
 		if (notification == STATECHANGE_CHANGEDCONTEXT) {
 			// Update window title
@@ -212,6 +212,10 @@ protected:
 		if (!pState)
 			return;
 
+		if (!m_pMainFrame->m_pContextControl) {
+			return;
+		}
+
 		CContextControl::_context_controls* controls = m_pMainFrame->m_pContextControl->GetControlsFromState(pState);
 		if (!controls)
 			return;
@@ -441,6 +445,9 @@ CMainFrame::~CMainFrame()
 
 	delete m_pStateEventHandler;
 
+	delete m_pContextControl;
+	m_pContextControl = 0;
+
 	CContextManager::Get()->DestroyAllStates();
 	delete m_pAsyncRequestQueue;
 #if FZ_MANUALUPDATECHECK
@@ -865,12 +872,17 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 	}
 }
 
-void CMainFrame::OnEngineEvent(wxFzEvent &event)
+void CMainFrame::OnEngineEvent(CFileZillaEngine* engine)
+{
+	CallAfter(&CMainFrame::DoOnEngineEvent, engine);
+}
+
+void CMainFrame::DoOnEngineEvent(CFileZillaEngine* engine)
 {
 	const std::vector<CState*> *pStates = CContextManager::Get()->GetAllStates();
 	CState* pState = 0;
 	for (std::vector<CState*>::const_iterator iter = pStates->begin(); iter != pStates->end(); ++iter) {
-		if ((*iter)->m_pEngine != event.engine_)
+		if ((*iter)->m_pEngine != engine)
 			continue;
 
 		pState = *iter;
@@ -1015,10 +1027,9 @@ void CMainFrame::OnCancel(wxCommandEvent&)
 	if (!pState || pState->m_pCommandQueue->Idle())
 		return;
 
-	if (wxMessageBoxEx(_("Really cancel current operation?"), _T("FileZilla"), wxYES_NO | wxICON_QUESTION) == wxYES)
-	{
+	if (wxMessageBoxEx(_("Really cancel current operation?"), _T("FileZilla"), wxYES_NO | wxICON_QUESTION) == wxYES) {
 		pState->m_pCommandQueue->Cancel();
-		pState->GetRecursiveOperationHandler()->StopRecursiveOperation();
+		pState->GetRemoteRecursiveOperation()->StopRecursiveOperation();
 	}
 }
 
@@ -1208,8 +1219,11 @@ void CMainFrame::OnClose(wxCloseEvent &event)
 	bool res = true;
 	const std::vector<CState*> *pStates = CContextManager::Get()->GetAllStates();
 	for (CState* pState : *pStates) {
-		if( pState->GetRecursiveOperationHandler() ) {
-			pState->GetRecursiveOperationHandler()->StopRecursiveOperation();
+		if (pState->GetLocalRecursiveOperation()) {
+			pState->GetLocalRecursiveOperation()->StopRecursiveOperation();
+		}
+		if (pState->GetRemoteRecursiveOperation()) {
+			pState->GetRemoteRecursiveOperation()->StopRecursiveOperation();
 		}
 
 		if (pState->m_pCommandQueue) {
@@ -2485,7 +2499,7 @@ void CMainFrame::OnSearch(wxCommandEvent&)
 		return;
 	}
 
-	CSearchDialog dlg(this, pState, m_pQueueView);
+	CSearchDialog dlg(this, *pState, m_pQueueView);
 	if (!dlg.Load())
 		return;
 
