@@ -122,6 +122,7 @@ static const t_Option options[OPTIONS_NUM] =
 	{ "Update Check", number, _T("1"), normal },
 	{ "Update Check Interval", number, _T("7"), normal },
 	{ "Last automatic update check", string, _T(""), normal },
+	{ "Last automatic update version", string, _T(""), normal },
 	{ "Update Check New Version", string, _T(""), normal },
 	{ "Update Check Check Beta", number, _T("0"), normal },
 	{ "Show debug menu", number, _T("0"), normal },
@@ -129,6 +130,7 @@ static const t_Option options[OPTIONS_NUM] =
 	{ "File exists action upload", number, _T("0"), normal },
 	{ "Allow ascii resume", number, _T("0"), normal },
 	{ "Greeting version", string, _T(""), normal },
+	{ "Greeting resources", string, _T(""), normal },
 	{ "Onetime Dialogs", string, _T(""), normal },
 	{ "Show Tree Local", number, _T("1"), normal },
 	{ "Show Tree Remote", number, _T("1"), normal },
