@@ -26,6 +26,7 @@ enum interfaceOptions
 	OPTION_UPDATECHECK,
 	OPTION_UPDATECHECK_INTERVAL,
 	OPTION_UPDATECHECK_LASTDATE,
+	OPTION_UPDATECHECK_LASTVERSION,
 	OPTION_UPDATECHECK_NEWVERSION,
 	OPTION_UPDATECHECK_CHECKBETA,
 	OPTION_DEBUG_MENU,
@@ -33,6 +34,7 @@ enum interfaceOptions
 	OPTION_FILEEXISTS_UPLOAD,
 	OPTION_ASCIIRESUME,
 	OPTION_GREETINGVERSION,
+	OPTION_GREETINGRESOURCES,
 	OPTION_ONETIME_DIALOGS,
 	OPTION_SHOW_TREE_LOCAL,
 	OPTION_SHOW_TREE_REMOTE,
