@@ -14,7 +14,7 @@
 #include <wx/dnd.h>
 #include "dndobjects.h"
 #include "Options.h"
-#include "recursive_operation.h"
+#include "remote_recursive_operation.h"
 #include "edithandler.h"
 #include "dragdropmanager.h"
 #include "drop_target_ex.h"
@@ -91,7 +91,7 @@ public:
 				}
 			}
 
-			m_pRemoteListView->m_pState->UploadDroppedFiles(m_pFileDataObject, subdir, false);
+			m_pRemoteListView->m_state.UploadDroppedFiles(m_pFileDataObject, subdir, false);
 			return wxDragCopy;
 		}
 
@@ -103,7 +103,7 @@ public:
 			return wxDragNone;
 		}
 
-		if (!m_pRemoteDataObject->GetServer().EqualsNoPass(*m_pRemoteListView->m_pState->GetServer()))
+		if (!m_pRemoteDataObject->GetServer().EqualsNoPass(*m_pRemoteListView->m_state.GetServer()))
 		{
 			wxMessageBoxEx(_("Drag&drop between different servers has not been implemented yet."));
 			return wxDragNone;
@@ -163,13 +163,13 @@ public:
 		for (std::list<CRemoteDataObject::t_fileInfo>::const_iterator iter = files.begin(); iter != files.end(); ++iter)
 		{
 			const CRemoteDataObject::t_fileInfo& info = *iter;
-			m_pRemoteListView->m_pState->m_pCommandQueue->ProcessCommand(
+			m_pRemoteListView->m_state.m_pCommandQueue->ProcessCommand(
 				new CRenameCommand(m_pRemoteDataObject->GetServerPath(), info.name, target, info.name)
 				);
 		}
 
 		// Refresh remote listing
-		m_pRemoteListView->m_pState->m_pCommandQueue->ProcessCommand(
+		m_pRemoteListView->m_state.m_pCommandQueue->ProcessCommand(
 			new CListCommand()
 			);
 
@@ -253,7 +253,7 @@ public:
 			return wxDragNone;
 		}
 
-		const CServer* const pServer = m_pRemoteListView->m_pState->GetServer();
+		const CServer* const pServer = m_pRemoteListView->m_state.GetServer();
 		wxASSERT(pServer);
 
 		int hit = DoDisplayDropHighlight(wxPoint(x, y));
@@ -355,14 +355,13 @@ BEGIN_EVENT_TABLE(CRemoteListView, CFileListCtrl<CGenericFileData>)
 	EVT_MENU(XRCID("ID_CONTEXT_REFRESH"), CRemoteListView::OnMenuRefresh)
 END_EVENT_TABLE()
 
-CRemoteListView::CRemoteListView(wxWindow* pParent, CState *pState, CQueueView* pQueue)
-	: CFileListCtrl<CGenericFileData>(pParent, pState, pQueue),
-	CStateEventHandler(pState)
+CRemoteListView::CRemoteListView(wxWindow* pParent, CState& state, CQueueView* pQueue)
+	: CFileListCtrl<CGenericFileData>(pParent, pQueue),
+	CStateEventHandler(state)
 {
-	pState->RegisterHandler(this, STATECHANGE_REMOTE_DIR);
-	pState->RegisterHandler(this, STATECHANGE_REMOTE_DIR_MODIFIED);
-	pState->RegisterHandler(this, STATECHANGE_APPLYFILTER);
-	pState->RegisterHandler(this, STATECHANGE_REMOTE_LINKNOTDIR);
+	state.RegisterHandler(this, STATECHANGE_REMOTE_DIR);
+	state.RegisterHandler(this, STATECHANGE_APPLYFILTER);
+	state.RegisterHandler(this, STATECHANGE_REMOTE_LINKNOTDIR);
 
 	m_dropTarget = -1;
 
@@ -474,7 +473,7 @@ void CRemoteListView::UpdateDirectoryListing_Added(std::shared_ptr<CDirectoryLis
 			if (entry.is_dir())
 				m_pFilelistStatusBar->AddDirectory();
 			else
-				m_pFilelistStatusBar->AddFile(entry.size.GetValue());
+				m_pFilelistStatusBar->AddFile(entry.size);
 		}
 
 		// Find correct position in index mapping
@@ -527,39 +526,40 @@ void CRemoteListView::UpdateDirectoryListing_Added(std::shared_ptr<CDirectoryLis
 
 void CRemoteListView::UpdateDirectoryListing_Removed(std::shared_ptr<CDirectoryListing> const& pDirectoryListing)
 {
-	const unsigned int removed = m_pDirectoryListing->GetCount() - pDirectoryListing->GetCount();
-	if (!removed) {
+	unsigned int const countRemoved = m_pDirectoryListing->GetCount() - pDirectoryListing->GetCount();
+	if (!countRemoved) {
 		m_pDirectoryListing = pDirectoryListing;
 		return;
 	}
 	wxASSERT(!IsComparing());
 
 	std::list<unsigned int> removedItems;
+	{
+		// Get indexes of the removed items in the listing
+		unsigned int j = 0;
+		unsigned int i = 0;
+		while (i < pDirectoryListing->GetCount() && j < m_pDirectoryListing->GetCount()) {
+			const CDirentry& oldEntry = (*m_pDirectoryListing)[j];
+			const wxString& oldName = oldEntry.name;
+			const wxString& newName = (*pDirectoryListing)[i].name;
+			if (oldName == newName) {
+				++i;
+				++j;
+				continue;
+			}
 
-	// Get indexes of the removed items in the listing
-	unsigned int j = 0;
-	unsigned int i = 0;
-	while (i < pDirectoryListing->GetCount() && j < m_pDirectoryListing->GetCount()) {
-		const CDirentry& oldEntry = (*m_pDirectoryListing)[j];
-		const wxString& oldName = oldEntry.name;
-		const wxString& newName = (*pDirectoryListing)[i].name;
-		if (oldName == newName) {
-			++i;
-			++j;
-			continue;
+			removedItems.push_back(j++);
 		}
+		for (; j < m_pDirectoryListing->GetCount(); ++j)
+			removedItems.push_back(j);
 
-		removedItems.push_back(j++);
+		wxASSERT(removedItems.size() == countRemoved);
 	}
-	for (; j < m_pDirectoryListing->GetCount(); ++j)
-		removedItems.push_back(j);
-
-	wxASSERT(removedItems.size() == removed);
 
 	std::list<int> selectedItems;
 
 	// Number of items left to remove
-	unsigned int toRemove = removed;
+	unsigned int toRemove = countRemoved;
 
 	std::list<int> removedIndexes;
 
@@ -593,12 +593,12 @@ void CRemoteListView::UpdateDirectoryListing_Removed(std::shared_ptr<CDirectoryL
 				if (oldEntry.is_dir())
 					m_pFilelistStatusBar->UnselectDirectory();
 				else
-					m_pFilelistStatusBar->UnselectFile(oldEntry.size.GetValue());
+					m_pFilelistStatusBar->UnselectFile(oldEntry.size);
 			}
 			if (oldEntry.is_dir())
 				m_pFilelistStatusBar->RemoveDirectory();
 			else
-				m_pFilelistStatusBar->RemoveFile(oldEntry.size.GetValue());
+				m_pFilelistStatusBar->RemoveFile(oldEntry.size);
 		}
 
 		// Update index
@@ -633,7 +633,7 @@ void CRemoteListView::UpdateDirectoryListing_Removed(std::shared_ptr<CDirectoryL
 
 	// Erase indexes
 	wxASSERT(!toRemove);
-	wxASSERT(removedIndexes.size() == removed);
+	wxASSERT(removedIndexes.size() == countRemoved);
 	for (auto const& removedIndex : removedIndexes) {
 		m_indexMapping.erase(m_indexMapping.begin() + removedIndex);
 	}
@@ -729,12 +729,12 @@ void CRemoteListView::SetDirectoryListing(std::shared_ptr<CDirectoryListing> con
 	if (reset) {
 		ResetSearchPrefix();
 
-		if (IsComparing())
+		if (IsComparing() && m_pDirectoryListing)
 			ExitComparisonMode();
 
 		ClearSelection();
 
-		prevFocused = m_pState->GetPreviouslyVisitedRemoteSubdir();
+		prevFocused = m_state.GetPreviouslyVisitedRemoteSubdir();
 		ensureVisible = !prevFocused.empty();
 	}
 	else {
@@ -790,7 +790,7 @@ void CRemoteListView::SetDirectoryListing(std::shared_ptr<CDirectoryListing> con
 				if (entry.size == -1)
 					++unknown_sizes;
 				else
-					totalSize += entry.size.GetValue();
+					totalSize += entry.size;
 				++totalFileCount;
 			}
 
@@ -877,7 +877,8 @@ wxString StripVMSRevision(const wxString& name)
 
 void CRemoteListView::OnItemActivated(wxListEvent &event)
 {
-	if (!m_pState->IsRemoteIdle()) {
+	int const action = COptions::Get()->GetOptionVal(OPTION_DOUBLECLICK_ACTION_DIRECTORY);
+	if (!m_state.IsRemoteIdle(action ? false : true)) {
 		wxBell();
 		return;
 	}
@@ -927,14 +928,13 @@ void CRemoteListView::OnItemActivated(wxListEvent &event)
 		const CDirentry& entry = (*m_pDirectoryListing)[index];
 		const wxString& name = entry.name;
 
-		const CServer* pServer = m_pState->GetServer();
+		const CServer* pServer = m_state.GetServer();
 		if (!pServer) {
 			wxBell();
 			return;
 		}
 
 		if (entry.is_dir()) {
-			const int action = COptions::Get()->GetOptionVal(OPTION_DOUBLECLICK_ACTION_DIRECTORY);
 			if (action == 3) {
 				// No action
 				wxBell();
@@ -946,10 +946,10 @@ void CRemoteListView::OnItemActivated(wxListEvent &event)
 					m_pLinkResolveState.reset(new t_linkResolveState);
 					m_pLinkResolveState->remote_path = m_pDirectoryListing->path;
 					m_pLinkResolveState->link = name;
-					m_pLinkResolveState->local_path = m_pState->GetLocalDir();
+					m_pLinkResolveState->local_path = m_state.GetLocalDir();
 					m_pLinkResolveState->server = *pServer;
 				}
-				m_pState->ChangeRemoteDir(m_pDirectoryListing->path, name, entry.is_link() ? LIST_FLAG_LINK : 0);
+				m_state.ChangeRemoteDir(m_pDirectoryListing->path, name, entry.is_link() ? LIST_FLAG_LINK : 0);
 			}
 			else {
 				wxCommandEvent evt(0, action == 1 ? XRCID("ID_DOWNLOAD") : XRCID("ID_ADDTOQUEUE"));
@@ -973,7 +973,7 @@ void CRemoteListView::OnItemActivated(wxListEvent &event)
 
 			const bool queue_only = action == 1;
 
-			const CLocalPath local_path = m_pState->GetLocalDir();
+			const CLocalPath local_path = m_state.GetLocalDir();
 			if (!local_path.IsWriteable()) {
 				wxBell();
 				return;
@@ -984,18 +984,18 @@ void CRemoteListView::OnItemActivated(wxListEvent &event)
 				localFile = StripVMSRevision(localFile);
 			m_pQueue->QueueFile(queue_only, true, name,
 				(name == localFile) ? wxString() : localFile,
-				local_path, m_pDirectoryListing->path, *pServer, entry.size.GetValue());
+				local_path, m_pDirectoryListing->path, *pServer, entry.size);
 			m_pQueue->QueueFile_Finish(true);
 		}
 	}
 	else {
-		m_pState->ChangeRemoteDir(m_pDirectoryListing->path, _T(".."));
+		m_state.ChangeRemoteDir(m_pDirectoryListing->path, _T(".."));
 	}
 }
 
 void CRemoteListView::OnMenuEnter(wxCommandEvent &)
 {
-	if (!m_pState->IsRemoteIdle()) {
+	if (!m_state.IsRemoteIdle(true)) {
 		wxBell();
 		return;
 	}
@@ -1021,7 +1021,7 @@ void CRemoteListView::OnMenuEnter(wxCommandEvent &)
 		const CDirentry& entry = (*m_pDirectoryListing)[index];
 		const wxString& name = entry.name;
 
-		const CServer* pServer = m_pState->GetServer();
+		const CServer* pServer = m_state.GetServer();
 		if (!pServer) {
 			wxBell();
 			return;
@@ -1036,20 +1036,19 @@ void CRemoteListView::OnMenuEnter(wxCommandEvent &)
 			m_pLinkResolveState.reset(new t_linkResolveState);
 			m_pLinkResolveState->remote_path = m_pDirectoryListing->path;
 			m_pLinkResolveState->link = name;
-			m_pLinkResolveState->local_path = m_pState->GetLocalDir();
+			m_pLinkResolveState->local_path = m_state.GetLocalDir();
 			m_pLinkResolveState->server = *pServer;
 		}
-		m_pState->ChangeRemoteDir(m_pDirectoryListing->path, name, entry.is_link() ? LIST_FLAG_LINK : 0);
+		m_state.ChangeRemoteDir(m_pDirectoryListing->path, name, entry.is_link() ? LIST_FLAG_LINK : 0);
 	}
 	else {
-		m_pState->ChangeRemoteDir(m_pDirectoryListing->path, _T(".."));
+		m_state.ChangeRemoteDir(m_pDirectoryListing->path, _T(".."));
 	}
 }
 
 void CRemoteListView::OnContextMenu(wxContextMenuEvent& event)
 {
-	if (GetEditControl())
-	{
+	if (GetEditControl()) {
 		event.Skip();
 		return;
 	}
@@ -1058,9 +1057,28 @@ void CRemoteListView::OnContextMenu(wxContextMenuEvent& event)
 	if (!pMenu)
 		return;
 
-	if (!m_pState->IsRemoteConnected() || !m_pState->IsRemoteIdle())
-	{
-		pMenu->Delete(XRCID("ID_ENTER"));
+	bool const idle = m_state.IsRemoteIdle();
+	bool const userIdle = m_state.IsRemoteIdle(true);
+	if (!m_state.IsRemoteConnected() || !idle) {
+		bool canEnter = false;
+		if (userIdle) {
+			int item = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
+			if (item > 0 && GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED) == -1) {
+				int index = GetItemIndex(item);
+				if (index != -1 && m_fileData[index].comparison_flags != fill) {
+					if ((*m_pDirectoryListing)[index].is_dir()) {
+						canEnter = true;
+					}
+				}
+				canEnter = true;
+			}
+		}
+		else {
+			pMenu->Enable(XRCID("ID_ENTER"), false);
+		}
+		if (!canEnter) {
+			pMenu->Delete(XRCID("ID_ENTER"));
+		}
 		pMenu->Enable(XRCID("ID_DOWNLOAD"), false);
 		pMenu->Enable(XRCID("ID_ADDTOQUEUE"), false);
 		pMenu->Enable(XRCID("ID_MKDIR"), false);
@@ -1073,8 +1091,7 @@ void CRemoteListView::OnContextMenu(wxContextMenuEvent& event)
 		pMenu->Enable(XRCID("ID_CONTEXT_REFRESH"), false);
 		pMenu->Enable(XRCID("ID_NEW_FILE"), false);
 	}
-	else if (GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED) == -1)
-	{
+	else if (GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED) == -1) {
 		pMenu->Delete(XRCID("ID_ENTER"));
 		pMenu->Enable(XRCID("ID_DOWNLOAD"), false);
 		pMenu->Enable(XRCID("ID_ADDTOQUEUE"), false);
@@ -1084,10 +1101,8 @@ void CRemoteListView::OnContextMenu(wxContextMenuEvent& event)
 		pMenu->Enable(XRCID("ID_EDIT"), false);
 		pMenu->Enable(XRCID("ID_GETURL"), false);
 	}
-	else
-	{
-		if ((GetItemCount() && GetItemState(0, wxLIST_STATE_SELECTED)))
-		{
+	else {
+		if ((GetItemCount() && GetItemState(0, wxLIST_STATE_SELECTED))) {
 			pMenu->Enable(XRCID("ID_RENAME"), false);
 			pMenu->Enable(XRCID("ID_CHMOD"), false);
 			pMenu->Enable(XRCID("ID_EDIT"), false);
@@ -1098,10 +1113,8 @@ void CRemoteListView::OnContextMenu(wxContextMenuEvent& event)
 		int fillCount = 0;
 		bool selectedDir = false;
 		int item = -1;
-		while ((item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1)
-		{
-			if (!item)
-			{
+		while ((item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1) {
+			if (!item) {
 				++count;
 				++fillCount;
 				continue;
@@ -1111,16 +1124,14 @@ void CRemoteListView::OnContextMenu(wxContextMenuEvent& event)
 			if (index == -1)
 				continue;
 			count++;
-			if (m_fileData[index].comparison_flags == fill)
-			{
+			if (m_fileData[index].comparison_flags == fill) {
 				fillCount++;
 				continue;
 			}
 			if ((*m_pDirectoryListing)[index].is_dir())
 				selectedDir = true;
 		}
-		if (!count || fillCount == count)
-		{
+		if (!count || fillCount == count) {
 			pMenu->Delete(XRCID("ID_ENTER"));
 			pMenu->Enable(XRCID("ID_DOWNLOAD"), false);
 			pMenu->Enable(XRCID("ID_ADDTOQUEUE"), false);
@@ -1130,21 +1141,18 @@ void CRemoteListView::OnContextMenu(wxContextMenuEvent& event)
 			pMenu->Enable(XRCID("ID_EDIT"), false);
 			pMenu->Enable(XRCID("ID_GETURL"), false);
 		}
-		else
-		{
+		else {
 			if (selectedDir)
 				pMenu->Enable(XRCID("ID_EDIT"), false);
 			else
 				pMenu->Delete(XRCID("ID_ENTER"));
-			if (count > 1)
-			{
+			if (count > 1) {
 				if (selectedDir)
 					pMenu->Delete(XRCID("ID_ENTER"));
 				pMenu->Enable(XRCID("ID_RENAME"), false);
 			}
 
-			if (!m_pState->GetLocalDir().IsWriteable())
-			{
+			if (!m_state.GetLocalDir().IsWriteable()) {
 				pMenu->Enable(XRCID("ID_DOWNLOAD"), false);
 				pMenu->Enable(XRCID("ID_ADDTOQUEUE"), false);
 			}
@@ -1158,18 +1166,16 @@ void CRemoteListView::OnContextMenu(wxContextMenuEvent& event)
 void CRemoteListView::OnMenuDownload(wxCommandEvent& event)
 {
 	// Make sure selection is valid
-	bool idle = m_pState->IsRemoteIdle();
+	bool idle = m_state.IsRemoteIdle();
 
-	const CLocalPath localDir = m_pState->GetLocalDir();
-	if (!localDir.IsWriteable())
-	{
+	const CLocalPath localDir = m_state.GetLocalDir();
+	if (!localDir.IsWriteable()) {
 		wxBell();
 		return;
 	}
 
 	long item = -1;
-	for (;;)
-	{
+	for (;;) {
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 		if (item == -1)
 			break;
@@ -1182,8 +1188,7 @@ void CRemoteListView::OnMenuDownload(wxCommandEvent& event)
 			continue;
 		if (m_fileData[index].comparison_flags == fill)
 			continue;
-		if ((*m_pDirectoryListing)[index].is_dir() && !idle)
-		{
+		if ((*m_pDirectoryListing)[index].is_dir() && !idle) {
 			wxBell();
 			return;
 		}
@@ -1192,27 +1197,26 @@ void CRemoteListView::OnMenuDownload(wxCommandEvent& event)
 	TransferSelectedFiles(localDir, event.GetId() == XRCID("ID_ADDTOQUEUE"));
 }
 
-void CRemoteListView::TransferSelectedFiles(const CLocalPath& local_parent, bool queueOnly)
+void CRemoteListView::TransferSelectedFiles(const CLocalPath& local_parent, bool queue_only)
 {
-	bool idle = m_pState->IsRemoteIdle();
+	bool idle = m_state.IsRemoteIdle();
 
-	CRecursiveOperation* pRecursiveOperation = m_pState->GetRecursiveOperationHandler();
+	CRemoteRecursiveOperation* pRecursiveOperation = m_state.GetRemoteRecursiveOperation();
 	wxASSERT(pRecursiveOperation);
 
 	wxASSERT(local_parent.IsWriteable());
 
-	const CServer* pServer = m_pState->GetServer();
-	if (!pServer)
-	{
+	const CServer* pServer = m_state.GetServer();
+	if (!pServer) {
 		wxBell();
 		return;
 	}
 
 	bool added = false;
-	bool startRecursive = false;
 	long item = -1;
-	for (;;)
-	{
+
+	recursion_root root(m_pDirectoryListing->path, false);
+	for (;;) {
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 		if (item == -1)
 			break;
@@ -1228,39 +1232,36 @@ void CRemoteListView::TransferSelectedFiles(const CLocalPath& local_parent, bool
 		const CDirentry& entry = (*m_pDirectoryListing)[index];
 		const wxString& name = entry.name;
 
-		if (entry.is_dir())
-		{
+		if (entry.is_dir()) {
 			if (!idle)
 				continue;
 			CLocalPath local_path(local_parent);
 			local_path.AddSegment(CQueueView::ReplaceInvalidCharacters(name));
 			CServerPath remotePath = m_pDirectoryListing->path;
-			if (remotePath.AddSegment(name))
-			{
-				pRecursiveOperation->AddDirectoryToVisit(m_pDirectoryListing->path, name, local_path, entry.is_link());
-				startRecursive = true;
+			if (remotePath.AddSegment(name)) {
+				root.add_dir_to_visit(m_pDirectoryListing->path, name, local_path, entry.is_link());
 			}
 		}
-		else
-		{
+		else {
 			wxString localFile = CQueueView::ReplaceInvalidCharacters(name);
 			if (m_pDirectoryListing->path.GetType() == VMS && COptions::Get()->GetOptionVal(OPTION_STRIP_VMS_REVISION))
 				localFile = StripVMSRevision(localFile);
-			m_pQueue->QueueFile(queueOnly, true,
+			m_pQueue->QueueFile(queue_only, true,
 				name, (name == localFile) ? wxString() : localFile,
-				local_parent, m_pDirectoryListing->path, *pServer, entry.size.GetValue());
+				local_parent, m_pDirectoryListing->path, *pServer, entry.size);
 			added = true;
 		}
 	}
 	if (added)
-		m_pQueue->QueueFile_Finish(!queueOnly);
+		m_pQueue->QueueFile_Finish(!queue_only);
 
-	if (startRecursive)
-	{
+	if (!root.empty()) {
+		pRecursiveOperation->AddRecursionRoot(std::move(root));
 		if (IsComparing())
 			ExitComparisonMode();
 		CFilterManager filter;
-		pRecursiveOperation->StartRecursiveOperation(queueOnly ? CRecursiveOperation::recursive_addtoqueue : CRecursiveOperation::recursive_download, m_pDirectoryListing->path, filter.GetActiveFilters(false));
+		pRecursiveOperation->StartRecursiveOperation(queue_only ? CRecursiveOperation::recursive_addtoqueue : CRecursiveOperation::recursive_transfer,
+													 filter.GetActiveFilters(false), m_pDirectoryListing->path);
 	}
 }
 
@@ -1275,7 +1276,7 @@ void CRemoteListView::OnMenuMkdirChgDir(wxCommandEvent&)
 {
 	CServerPath newdir = MenuMkdir();
 	if (!newdir.empty()) {
-		m_pState->ChangeRemoteDir(newdir, wxString(), 0, true);
+		m_state.ChangeRemoteDir(newdir, wxString(), 0, true);
 	}
 }
 
@@ -1283,7 +1284,7 @@ void CRemoteListView::OnMenuMkdirChgDir(wxCommandEvent&)
 // Returns the name of the new directory
 CServerPath CRemoteListView::MenuMkdir()
 {
-	if (!m_pDirectoryListing || !m_pState->IsRemoteIdle()) {
+	if (!m_pDirectoryListing || !m_state.IsRemoteIdle()) {
 		wxBell();
 		return CServerPath();
 	}
@@ -1314,7 +1315,7 @@ CServerPath CRemoteListView::MenuMkdir()
 		return CServerPath();
 
 	if (!m_pDirectoryListing || oldPath != m_pDirectoryListing->path ||
-		!m_pState->IsRemoteIdle())
+		!m_state.IsRemoteIdle())
 	{
 		wxBell();
 		return CServerPath();
@@ -1326,7 +1327,7 @@ CServerPath CRemoteListView::MenuMkdir()
 		return CServerPath();
 	}
 
-	m_pState->m_pCommandQueue->ProcessCommand(new CMkdirCommand(path));
+	m_state.m_pCommandQueue->ProcessCommand(new CMkdirCommand(path));
 
 	// Return name of the New Directory
 	return path;
@@ -1334,7 +1335,7 @@ CServerPath CRemoteListView::MenuMkdir()
 
 void CRemoteListView::OnMenuDelete(wxCommandEvent&)
 {
-	if (!m_pState->IsRemoteIdle()) {
+	if (!m_state.IsRemoteIdle()) {
 		wxBell();
 		return;
 	}
@@ -1404,15 +1405,13 @@ void CRemoteListView::OnMenuDelete(wxCommandEvent&)
 		follow_symlink = XRCCTRL(dlg, "ID_RECURSE", wxRadioButton)->GetValue();
 	}
 
-	CRecursiveOperation* pRecursiveOperation = m_pState->GetRecursiveOperationHandler();
+	CRemoteRecursiveOperation* pRecursiveOperation = m_state.GetRemoteRecursiveOperation();
 	wxASSERT(pRecursiveOperation);
 
 	std::deque<wxString> filesToDelete;
 
-	bool startRecursive = false;
-	item = -1;
-	for (;;)
-	{
+	recursion_root root(m_pDirectoryListing->path, false);
+	for (item = -1; ;) {
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 		if (!item)
 			continue;
@@ -1428,13 +1427,10 @@ void CRemoteListView::OnMenuDelete(wxCommandEvent&)
 		const CDirentry& entry = (*m_pDirectoryListing)[index];
 		const wxString& name = entry.name;
 
-		if (entry.is_dir() && (follow_symlink || !entry.is_link()))
-		{
+		if (entry.is_dir() && (follow_symlink || !entry.is_link())) {
 			CServerPath remotePath = m_pDirectoryListing->path;
-			if (remotePath.AddSegment(name))
-			{
-				pRecursiveOperation->AddDirectoryToVisit(m_pDirectoryListing->path, name, CLocalPath(), true);
-				startRecursive = true;
+			if (remotePath.AddSegment(name)) {
+				root.add_dir_to_visit(m_pDirectoryListing->path, name, CLocalPath(), true);;
 			}
 		}
 		else
@@ -1442,14 +1438,16 @@ void CRemoteListView::OnMenuDelete(wxCommandEvent&)
 	}
 
 	if (!filesToDelete.empty())
-		m_pState->m_pCommandQueue->ProcessCommand(new CDeleteCommand(m_pDirectoryListing->path, std::move(filesToDelete)));
+		m_state.m_pCommandQueue->ProcessCommand(new CDeleteCommand(m_pDirectoryListing->path, std::move(filesToDelete)));
 
-	if (startRecursive)
-	{
+	if (!root.empty()) {
 		if (IsComparing())
 			ExitComparisonMode();
+		pRecursiveOperation->AddRecursionRoot(std::move(root));
+		
 		CFilterManager filter;
-		pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_delete, m_pDirectoryListing->path, filter.GetActiveFilters(false));
+		pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_delete,
+													 filter.GetActiveFilters(false), m_pDirectoryListing->path);
 	}
 }
 
@@ -1460,7 +1458,7 @@ void CRemoteListView::OnMenuRename(wxCommandEvent&)
 		return;
 	}
 
-	if (!m_pState->IsRemoteIdle()) {
+	if (!m_state.IsRemoteIdle()) {
 		wxBell();
 		return;
 	}
@@ -1526,7 +1524,7 @@ void CRemoteListView::OnKeyDown(wxKeyEvent& event)
 
 bool CRemoteListView::OnBeginRename(const wxListEvent& event)
 {
-	if (!m_pState->IsRemoteIdle())
+	if (!m_state.IsRemoteIdle())
 	{
 		wxBell();
 		return false;
@@ -1551,7 +1549,7 @@ bool CRemoteListView::OnBeginRename(const wxListEvent& event)
 
 bool CRemoteListView::OnAcceptRename(const wxListEvent& event)
 {
-	if (!m_pState->IsRemoteIdle())
+	if (!m_state.IsRemoteIdle())
 	{
 		wxBell();
 		return false;
@@ -1603,15 +1601,14 @@ bool CRemoteListView::OnAcceptRename(const wxListEvent& event)
 		}
 	}
 
-	m_pState->m_pCommandQueue->ProcessCommand(new CRenameCommand(m_pDirectoryListing->path, entry.name, newPath, newFile));
+	m_state.m_pCommandQueue->ProcessCommand(new CRenameCommand(m_pDirectoryListing->path, entry.name, newPath, newFile));
 
 	return true;
 }
 
 void CRemoteListView::OnMenuChmod(wxCommandEvent&)
 {
-	if (!m_pState->IsRemoteConnected() || !m_pState->IsRemoteIdle())
-	{
+	if (!m_state.IsRemoteConnected() || !m_state.IsRemoteIdle()) {
 		wxBell();
 		return;
 	}
@@ -1620,11 +1617,10 @@ void CRemoteListView::OnMenuChmod(wxCommandEvent&)
 	int dirCount = 0;
 	wxString name;
 
-	char permissions[9] = {0};
+	char permissions[9] = {};
 
 	long item = -1;
-	for (;;)
-	{
+	for (;;) {
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 		if (item == -1)
 			break;
@@ -1647,10 +1643,8 @@ void CRemoteListView::OnMenuChmod(wxCommandEvent&)
 		name = entry.name;
 
 		char file_perms[9];
-		if (CChmodDialog::ConvertPermissions(*entry.permissions, file_perms))
-		{
-			for (int i = 0; i < 9; i++)
-			{
+		if (CChmodDialog::ConvertPermissions(*entry.permissions, file_perms)) {
+			for (int i = 0; i < 9; i++) {
 				if (!permissions[i] || permissions[i] == file_perms[i])
 					permissions[i] = file_perms[i];
 				else
@@ -1658,8 +1652,7 @@ void CRemoteListView::OnMenuChmod(wxCommandEvent&)
 			}
 		}
 	}
-	if (!dirCount && !fileCount)
-	{
+	if (!dirCount && !fileCount) {
 		wxBell();
 		return;
 	}
@@ -1669,51 +1662,43 @@ void CRemoteListView::OnMenuChmod(wxCommandEvent&)
 			permissions[i] = 0;
 
 	CChmodDialog* pChmodDlg = new CChmodDialog;
-	if (!pChmodDlg->Create(this, fileCount, dirCount, name, permissions))
-	{
+	if (!pChmodDlg->Create(this, fileCount, dirCount, name, permissions)) {
 		pChmodDlg->Destroy();
-		pChmodDlg = 0;
 		return;
 	}
 
-	if (pChmodDlg->ShowModal() != wxID_OK)
-	{
+	if (pChmodDlg->ShowModal() != wxID_OK) {
 		pChmodDlg->Destroy();
-		pChmodDlg = 0;
 		return;
 	}
 
 	// State may have changed while chmod dialog was shown
-	if (!m_pState->IsRemoteConnected() || !m_pState->IsRemoteIdle())
-	{
+	if (!m_state.IsRemoteConnected() || !m_state.IsRemoteIdle()) {
 		pChmodDlg->Destroy();
-		pChmodDlg = 0;
 		wxBell();
 		return;
 	}
 
 	const int applyType = pChmodDlg->GetApplyType();
 
-	CRecursiveOperation* pRecursiveOperation = m_pState->GetRecursiveOperationHandler();
+	CRemoteRecursiveOperation* pRecursiveOperation = m_state.GetRemoteRecursiveOperation();
 	wxASSERT(pRecursiveOperation);
+	recursion_root root(m_pDirectoryListing->path, false);
 
 	item = -1;
-	for (;;)
-	{
+	for (;;) {
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 		if (item == -1)
 			break;
 
-		if (!item)
-		{
+		if (!item) {
 			pChmodDlg->Destroy();
 			pChmodDlg = 0;
 			return;
 		}
 
 		int index = GetItemIndex(item);
-		if (index == -1)
-		{
+		if (index == -1) {
 			pChmodDlg->Destroy();
 			pChmodDlg = 0;
 			return;
@@ -1727,35 +1712,35 @@ void CRemoteListView::OnMenuChmod(wxCommandEvent&)
 			(!entry.is_dir() && applyType == 1) ||
 			(entry.is_dir() && applyType == 2))
 		{
-			char permissions[9];
-			bool res = pChmodDlg->ConvertPermissions(*entry.permissions, permissions);
-			wxString newPerms = pChmodDlg->GetPermissions(res ? permissions : 0, entry.is_dir());
+			char newPermissions[9]{};
+			bool res = pChmodDlg->ConvertPermissions(*entry.permissions, newPermissions);
+			wxString newPerms = pChmodDlg->GetPermissions(res ? newPermissions : 0, entry.is_dir());
 
-			m_pState->m_pCommandQueue->ProcessCommand(new CChmodCommand(m_pDirectoryListing->path, entry.name, newPerms));
+			m_state.m_pCommandQueue->ProcessCommand(new CChmodCommand(m_pDirectoryListing->path, entry.name, newPerms));
 		}
 
 		if (pChmodDlg->Recursive() && entry.is_dir())
-			pRecursiveOperation->AddDirectoryToVisit(m_pDirectoryListing->path, entry.name);
+			root.add_dir_to_visit(m_pDirectoryListing->path, entry.name);
 	}
 
-	if (pChmodDlg->Recursive())
-	{
+	if (pChmodDlg->Recursive()) {
 		if (IsComparing())
 			ExitComparisonMode();
 
 		pRecursiveOperation->SetChmodDialog(pChmodDlg);
+		pRecursiveOperation->AddRecursionRoot(std::move(root));
 		CFilterManager filter;
-		pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_chmod, m_pDirectoryListing->path, filter.GetActiveFilters(false));
+		pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_chmod,
+													 filter.GetActiveFilters(false), m_pDirectoryListing->path);
 
 		// Refresh listing. This gets done implicitely by the recursive operation, so
 		// only it if not recursing.
 		if (pRecursiveOperation->GetOperationMode() != CRecursiveOperation::recursive_chmod)
-			m_pState->ChangeRemoteDir(m_pDirectoryListing->path);
+			m_state.ChangeRemoteDir(m_pDirectoryListing->path);
 	}
-	else
-	{
+	else {
 		pChmodDlg->Destroy();
-		m_pState->ChangeRemoteDir(m_pDirectoryListing->path, _T(""), 0, true);
+		m_state.ChangeRemoteDir(m_pDirectoryListing->path, _T(""), 0, true);
 	}
 
 }
@@ -1800,7 +1785,7 @@ void CRemoteListView::ApplyCurrentFilter()
 			if (entry.size == -1)
 				++unknown_sizes;
 			else
-				totalSize += entry.size.GetValue();
+				totalSize += entry.size;
 			++totalFileCount;
 		}
 
@@ -1862,8 +1847,7 @@ std::list<wxString> CRemoteListView::RememberSelectedItems(wxString& focused)
 	}
 
 	int item = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_FOCUSED);
-	if (item != -1)
-	{
+	if (item != -1) {
 		int index = GetItemIndex(item);
 		if (index != -1 && m_fileData[index].comparison_flags != fill)
 		{
@@ -1884,25 +1868,21 @@ void CRemoteListView::ReselectItems(std::list<wxString>& selectedNames, wxString
 	if (!GetItemCount())
 		return;
 
-	if (focused == _T(".."))
-	{
+	if (focused == _T("..")) {
 		focused = _T("");
 		SetItemState(0, wxLIST_STATE_FOCUSED, wxLIST_STATE_FOCUSED);
 	}
 
-	if (selectedNames.empty())
-	{
+	if (selectedNames.empty()) {
 		if (focused.empty())
 			return;
 
-		for (unsigned int i = 1; i < m_indexMapping.size(); i++)
-		{
+		for (unsigned int i = 1; i < m_indexMapping.size(); ++i) {
 			const int index = m_indexMapping[i];
 			if (m_fileData[index].comparison_flags == fill)
 				continue;
 
-			if ((*m_pDirectoryListing)[index].name == focused)
-			{
+			if ((*m_pDirectoryListing)[index].name == focused) {
 				SetItemState(i, wxLIST_STATE_FOCUSED, wxLIST_STATE_FOCUSED);
 				if (ensureVisible)
 					EnsureVisible(i);
@@ -1912,8 +1892,7 @@ void CRemoteListView::ReselectItems(std::list<wxString>& selectedNames, wxString
 		return;
 	}
 
-	if (selectedNames.front() == _T(".."))
-	{
+	if (selectedNames.front() == _T("..")) {
 		selectedNames.pop_front();
 		SetItemState(0, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
 	}
@@ -1952,7 +1931,7 @@ void CRemoteListView::ReselectItems(std::list<wxString>& selectedNames, wxString
 				if (firstSelected == -1)
 					firstSelected = i;
 				if (m_pFilelistStatusBar)
-					m_pFilelistStatusBar->SelectFile(entry.size.GetValue());
+					m_pFilelistStatusBar->SelectFile(entry.size);
 				SetSelection(i, true);
 				break;
 			}
@@ -2009,20 +1988,15 @@ void CRemoteListView::RepositionInfoText()
 
 }
 
-void CRemoteListView::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2)
+void CRemoteListView::OnStateChange(t_statechange_notifications notification, const wxString& data, const void* data2)
 {
-	wxASSERT(pState);
 	if (notification == STATECHANGE_REMOTE_DIR)
-		SetDirectoryListing(pState->GetRemoteDir());
-	else if (notification == STATECHANGE_REMOTE_DIR_MODIFIED)
-		SetDirectoryListing(pState->GetRemoteDir());
-	else if (notification == STATECHANGE_REMOTE_LINKNOTDIR)
-	{
+		SetDirectoryListing(m_state.GetRemoteDir());
+	else if (notification == STATECHANGE_REMOTE_LINKNOTDIR) {
 		wxASSERT(data2);
 		LinkIsNotDir(*(CServerPath*)data2, data);
 	}
-	else
-	{
+	else {
 		wxASSERT(notification == STATECHANGE_APPLYFILTER);
 		ApplyCurrentFilter();
 	}
@@ -2031,8 +2005,7 @@ void CRemoteListView::OnStateChange(CState* pState, enum t_statechange_notificat
 void CRemoteListView::SetInfoText()
 {
 	wxString text;
-	if (!IsComparing())
-	{
+	if (!IsComparing()) {
 		if (!m_pDirectoryListing)
 			text = _("Not connected to any server");
 		else if (m_pDirectoryListing->failed())
@@ -2041,15 +2014,13 @@ void CRemoteListView::SetInfoText()
 			text = _("Empty directory listing");
 	}
 
-	if (text.empty())
-	{
+	if (text.empty()) {
 		delete m_pInfoText;
 		m_pInfoText = 0;
 		return;
 	}
 
-	if (!m_pInfoText)
-	{
+	if (!m_pInfoText) {
 		m_pInfoText = new CInfoText(this, text);
 #ifdef __WXMSW__
 		if (GetLayoutDirection() != wxLayout_RightToLeft)
@@ -2066,30 +2037,26 @@ void CRemoteListView::SetInfoText()
 
 void CRemoteListView::OnBeginDrag(wxListEvent&)
 {
-	if (!m_pState->IsRemoteIdle())
-	{
+	if (!m_state.IsRemoteIdle()) {
 		wxBell();
 		return;
 	}
 
-	if (GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED) == -1)
-	{
+	if (GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED) == -1) {
 		// Nothing selected
 		return;
 	}
 
-	bool idle = m_pState->m_pCommandQueue->Idle();
+	bool idle = m_state.m_pCommandQueue->Idle();
 
 	long item = -1;
 	int count = 0;
-	for (;;)
-	{
+	for (;;) {
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 		if (item == -1)
 			break;
 
-		if (!item)
-		{
+		if (!item) {
 			// Can't drag ".."
 			wxBell();
 			return;
@@ -2098,27 +2065,25 @@ void CRemoteListView::OnBeginDrag(wxListEvent&)
 		int index = GetItemIndex(item);
 		if (index == -1 || m_fileData[index].comparison_flags == fill)
 			continue;
-		if ((*m_pDirectoryListing)[index].is_dir() && !idle)
-		{
+		if ((*m_pDirectoryListing)[index].is_dir() && !idle) {
 			// Drag could result in recursive operation, don't allow at this point
 			wxBell();
 			return;
 		}
-		count++;
+		++count;
 	}
-	if (!count)
-	{
+	if (!count) {
 		wxBell();
 		return;
 	}
 
 	wxDataObjectComposite object;
 
-	const CServer* const pServer = m_pState->GetServer();
+	CServer const* pServer = m_state.GetServer();
 	if (!pServer)
 		return;
-	const CServer server = *pServer;
-	const CServerPath path = m_pDirectoryListing->path;
+	CServer const server = *pServer;
+	CServerPath const path = m_pDirectoryListing->path;
 
 	CRemoteDataObject *pRemoteDataObject = new CRemoteDataObject(*pServer, m_pDirectoryListing->path);
 
@@ -2129,8 +2094,7 @@ void CRemoteListView::OnBeginDrag(wxListEvent&)
 
 	// Add files to remote data object
 	item = -1;
-	for (;;)
-	{
+	for (;;) {
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 		if (item == -1)
 			break;
@@ -2148,9 +2112,8 @@ void CRemoteListView::OnBeginDrag(wxListEvent&)
 	object.Add(pRemoteDataObject, true);
 
 #if FZ3_USESHELLEXT
-	CShellExtensionInterface* ext = CShellExtensionInterface::CreateInitialized();
-	if (ext)
-	{
+	std::unique_ptr<CShellExtensionInterface> ext = CShellExtensionInterface::CreateInitialized();
+	if (ext) {
 		const wxString& file = ext->GetDragDirectory();
 
 		wxASSERT(!file.empty());
@@ -2171,88 +2134,64 @@ void CRemoteListView::OnBeginDrag(wxListEvent&)
 
 	pDragDropManager->Release();
 
-	if (res != wxDragCopy)
-	{
-#if FZ3_USESHELLEXT
-		delete ext;
-		ext = 0;
-#endif
+	if (res != wxDragCopy) {
 		return;
 	}
 
 #if FZ3_USESHELLEXT
-	if (ext)
-	{
-		if (!pRemoteDataObject->DidSendData())
-		{
-			const CServer* pServer = m_pState->GetServer();
-			if (!m_pState->IsRemoteIdle() ||
+	if (ext) {
+		if (!pRemoteDataObject->DidSendData()) {
+			pServer = m_state.GetServer();
+			if (!m_state.IsRemoteIdle() ||
 				!pServer || *pServer != server ||
 				!m_pDirectoryListing || m_pDirectoryListing->path != path)
 			{
 				// Remote listing has changed since drag started
 				wxBell();
-				delete ext;
-				ext = 0;
 				return;
 			}
 
 			// Same checks as before
-			bool idle = m_pState->m_pCommandQueue->Idle();
+			idle = m_state.m_pCommandQueue->Idle();
 
-			long item = -1;
-			for (;;)
-			{
+			item = -1;
+			for (;;) {
 				item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 				if (item == -1)
 					break;
 
-				if (!item)
-				{
+				if (!item) {
 					// Can't drag ".."
 					wxBell();
-					delete ext;
-					ext = 0;
 					return;
 				}
 
 				int index = GetItemIndex(item);
 				if (index == -1 || m_fileData[index].comparison_flags == fill)
 					continue;
-				if ((*m_pDirectoryListing)[index].is_dir() && !idle)
-				{
+				if ((*m_pDirectoryListing)[index].is_dir() && !idle) {
 					// Drag could result in recursive operation, don't allow at this point
 					wxBell();
-					delete ext;
-					ext = 0;
 					return;
 				}
 			}
 
 			CLocalPath target(ext->GetTarget());
-			if (target.empty())
-			{
-				delete ext;
-				ext = 0;
+			if (target.empty()) {
+				ext.reset(); // Release extension before the modal message box
 				wxMessageBoxEx(_("Could not determine the target of the Drag&Drop operation.\nEither the shell extension is not installed properly or you didn't drop the files into an Explorer window."));
 				return;
 			}
 
 			TransferSelectedFiles(target, false);
-
-			delete ext;
-			ext = 0;
-			return;
 		}
-		delete ext;
-		ext = 0;
 	}
 #endif
 }
 
 void CRemoteListView::OnMenuEdit(wxCommandEvent&)
 {
-	if (!m_pState->IsRemoteConnected() || !m_pDirectoryListing) {
+	if (!m_state.IsRemoteConnected() || !m_pDirectoryListing) {
 		wxBell();
 		return;
 	}
@@ -2276,7 +2215,7 @@ void CRemoteListView::OnMenuEdit(wxCommandEvent&)
 			return;
 		}
 
-		selected_items.push_back({entry.name, entry.size.GetValue()});
+		selected_items.push_back({entry.name, entry.size});
 	}
 
 	CEditHandler* pEditHandler = CEditHandler::Get();
@@ -2286,7 +2225,7 @@ void CRemoteListView::OnMenuEdit(wxCommandEvent&)
 	}
 
 	const CServerPath path = m_pDirectoryListing->path;
-	const CServer server = *m_pState->GetServer();
+	const CServer server = *m_state.GetServer();
 	pEditHandler->Edit(CEditHandler::remote, selected_items, path, server, this);
 }
 
@@ -2322,15 +2261,9 @@ void CRemoteListView::ValidateIndexMapping()
 }
 #endif
 
-bool CRemoteListView::CanStartComparison(wxString* pError)
+bool CRemoteListView::CanStartComparison()
 {
-	if (!m_pDirectoryListing) {
-		if (pError)
-			*pError = _("Cannot compare directories, not connected to a server.");
-		return false;
-	}
-
-	return true;
+	return m_pDirectoryListing != 0;
 }
 
 void CRemoteListView::StartComparison()
@@ -2360,7 +2293,7 @@ void CRemoteListView::StartComparison()
 	}
 }
 
-bool CRemoteListView::GetNextFile(wxString& name, bool& dir, wxLongLong& size, CDateTime& date)
+bool CRemoteListView::get_next_file(wxString& name, bool& dir, int64_t& size, fz::datetime& date)
 {
 	if (++m_comparisonIndex >= (int)m_originalIndexMapping.size())
 		return false;
@@ -2369,8 +2302,7 @@ bool CRemoteListView::GetNextFile(wxString& name, bool& dir, wxLongLong& size, C
 	if (index >= m_fileData.size())
 		return false;
 
-	if (index == m_pDirectoryListing->GetCount())
-	{
+	if (index == m_pDirectoryListing->GetCount()) {
 		name = _T("..");
 		dir = true;
 		size = -1;
@@ -2453,7 +2385,7 @@ wxString CRemoteListView::GetItemText(int item, unsigned int column)
 		if (entry.is_dir() || entry.size < 0)
 			return wxString();
 		else
-			return CSizeFormat::Format(entry.size.GetValue());
+			return CSizeFormat::Format(entry.size);
 	}
 	else if (column == 2)
 	{
@@ -2532,7 +2464,7 @@ bool CRemoteListView::ItemIsDir(int index) const
 
 int64_t CRemoteListView::ItemGetSize(int index) const
 {
-	return (*m_pDirectoryListing)[index].size.GetValue();
+	return (*m_pDirectoryListing)[index].size;
 }
 
 void CRemoteListView::LinkIsNotDir(const CServerPath& path, const wxString& link)
@@ -2555,7 +2487,7 @@ void CRemoteListView::OnMenuGeturl(wxCommandEvent&)
 	if (!m_pDirectoryListing)
 		return;
 
-	const CServer* pServer = m_pState->GetServer();
+	const CServer* pServer = m_state.GetServer();
 	if (!pServer)
 		return;
 
@@ -2644,14 +2576,13 @@ int CRemoteListView::GetOverlayIndex(int item)
 
 void CRemoteListView::OnMenuRefresh(wxCommandEvent&)
 {
-	if (m_pState)
-		m_pState->RefreshRemote();
+	m_state.RefreshRemote();
 }
 
 void CRemoteListView::OnNavigationEvent(bool forward)
 {
 	if (!forward) {
-		if (!m_pState->IsRemoteIdle()) {
+		if (!m_state.IsRemoteIdle(true)) {
 			wxBell();
 			return;
 		}
@@ -2661,13 +2592,13 @@ void CRemoteListView::OnNavigationEvent(bool forward)
 			return;
 		}
 
-		m_pState->ChangeRemoteDir(m_pDirectoryListing->path, _T(".."));
+		m_state.ChangeRemoteDir(m_pDirectoryListing->path, _T(".."));
 	}
 }
 
 void CRemoteListView::OnMenuNewfile(wxCommandEvent&)
 {
-	if (!m_pState->IsRemoteIdle() || !m_pDirectoryListing) {
+	if (!m_state.IsRemoteIdle() || !m_pDirectoryListing) {
 		wxBell();
 		return;
 	}
@@ -2706,12 +2637,12 @@ void CRemoteListView::OnMenuNewfile(wxCommandEvent&)
 		file.Create(emptyfile);
 	}
 
-	const CServer* pServer = m_pState->GetServer();
+	const CServer* pServer = m_state.GetServer();
 	if (!pServer) {
 		wxBell();
 		return;
 	}
 
 	CFileTransferCommand *cmd = new CFileTransferCommand(emptyfile, m_pDirectoryListing->path, newFileName, false, CFileTransferCommand::t_transferSettings());
-	m_pState->m_pCommandQueue->ProcessCommand(cmd);
+	m_state.m_pCommandQueue->ProcessCommand(cmd);
 }
