@@ -14,12 +14,12 @@ class CRemoteListView : public CFileListCtrl<CGenericFileData>, CStateEventHandl
 {
 	friend class CRemoteListViewDropTarget;
 public:
-	CRemoteListView(wxWindow* pParent, CState* pState, CQueueView* pQueue);
+	CRemoteListView(wxWindow* pParent, CState& state, CQueueView* pQueue);
 	virtual ~CRemoteListView();
 
-	virtual bool CanStartComparison(wxString* pError);
+	virtual bool CanStartComparison();
 	virtual void StartComparison();
-	virtual bool GetNextFile(wxString& name, bool &dir, wxLongLong &size, CDateTime& date);
+	virtual bool get_next_file(wxString& name, bool &dir, int64_t &size, fz::datetime& date);
 	virtual void FinishComparison();
 	virtual void OnExitComparisonMode();
 
@@ -51,7 +51,7 @@ protected:
 
 	virtual CSortComparisonObject GetSortComparisonObject();
 
-	virtual void OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2);
+	virtual void OnStateChange(t_statechange_notifications notification, const wxString& data, const void* data2);
 	void ApplyCurrentFilter();
 	void SetDirectoryListing(std::shared_ptr<CDirectoryListing> const& pDirectoryListing);
 	bool UpdateDirectoryListing(std::shared_ptr<CDirectoryListing> const& pDirectoryListing);
@@ -67,7 +67,7 @@ protected:
 	std::shared_ptr<CDirectoryListing> m_pDirectoryListing;
 
 	// Caller is responsible to check selection is valid!
-	void TransferSelectedFiles(const CLocalPath& local_parent, bool queueOnly);
+	void TransferSelectedFiles(const CLocalPath& local_parent, bool queue_only);
 
 	// Cache icon for directories, no need to calculate it multiple times
 	int m_dirIcon;
