@@ -1,25 +1,27 @@
 #ifndef __REMOTETREEVIEW_H__
 #define __REMOTETREEVIEW_H__
 
+#include <option_change_event_handler.h>
 #include "systemimagelist.h"
 #include "state.h"
 #include "filter.h"
+#include "treectrlex.h"
 
 class CQueueView;
-class CRemoteTreeView : public wxTreeCtrl, CSystemImageList, CStateEventHandler
+class CRemoteTreeView : public wxTreeCtrlEx, CSystemImageList, CStateEventHandler, COptionChangeEventHandler
 {
 	DECLARE_CLASS(CRemoteTreeView)
 
 	friend class CRemoteTreeViewDropTarget;
 
 public:
-	CRemoteTreeView(wxWindow* parent, wxWindowID id, CState* pState, CQueueView* pQueue);
+	CRemoteTreeView(wxWindow* parent, wxWindowID id, CState& state, CQueueView* pQueue);
 	virtual ~CRemoteTreeView();
 
 protected:
 	wxTreeItemId MakeParent(CServerPath path, bool select);
-	void SetDirectoryListing(const CSharedPointer<const CDirectoryListing> &pListing, bool modified);
-	virtual void OnStateChange(enum t_statechange_notifications notification, const wxString& data);
+	void SetDirectoryListing(std::shared_ptr<CDirectoryListing> const& pListing, bool modified);
+	virtual void OnStateChange(t_statechange_notifications notification, const wxString&, const void*);
 
 	void DisplayItem(wxTreeItemId parent, const CDirectoryListing& listing);
 	void RefreshItem(wxTreeItemId parent, const CDirectoryListing& listing, bool will_select_parent);
@@ -28,13 +30,11 @@ protected:
 
 	bool HasSubdirs(const CDirectoryListing& listing, const CFilterManager& filter);
 
-	virtual int OnCompareItems(const wxTreeItemId& item1, const wxTreeItemId& item2);
-
 	CServerPath GetPathFromItem(const wxTreeItemId& item) const;
 
 	bool ListExpand(wxTreeItemId item);
 
-	void ApplyFilters();
+	void ApplyFilters(bool resort);
 
 	CQueueView* m_pQueue;
 
@@ -50,24 +50,28 @@ protected:
 
 	wxTreeItemId m_dropHighlight;
 
+	CServerPath MenuMkdir();
+
+	void UpdateSortMode();
+
+	virtual void OnOptionsChanged(changed_options_t const& options);
+
 	DECLARE_EVENT_TABLE()
 	void OnItemExpanding(wxTreeEvent& event);
 	void OnSelectionChanged(wxTreeEvent& event);
 	void OnItemActivated(wxTreeEvent& event);
 	void OnBeginDrag(wxTreeEvent& event);
-#ifndef __WXMSW__
-	void OnKeyDown(wxKeyEvent& event);
-#endif
 	void OnContextMenu(wxTreeEvent& event);
-	void OnMenuChmod(wxCommandEvent& event);
+	void OnMenuChmod(wxCommandEvent&);
 	void OnMenuDownload(wxCommandEvent& event);
-	void OnMenuDelete(wxCommandEvent& event);
-	void OnMenuRename(wxCommandEvent& event);
+	void OnMenuDelete(wxCommandEvent&);
+	void OnMenuRename(wxCommandEvent&);
 	void OnBeginLabelEdit(wxTreeEvent& event);
 	void OnEndLabelEdit(wxTreeEvent& event);
-	void OnMkdir(wxCommandEvent& event);
+	void OnMkdir(wxCommandEvent&);
+	void OnMenuMkdirChgDir(wxCommandEvent&);
 	void OnChar(wxKeyEvent& event);
-	void OnMenuGeturl(wxCommandEvent& event);
+	void OnMenuGeturl(wxCommandEvent&);
 
 	wxTreeItemId m_contextMenuItem;
 };
