@@ -13,14 +13,14 @@ class CSearchDialog : protected CFilterConditionsDialog, public CStateEventHandl
 {
 	friend class CSearchDialogFileList;
 public:
-	CSearchDialog(wxWindow* parent, CState* pState, CQueueView* pQueue);
+	CSearchDialog(wxWindow* parent, CState& state, CQueueView* pQueue);
 	virtual ~CSearchDialog();
 
 	bool Load();
 	void Run();
 
 protected:
-	void ProcessDirectoryListing();
+	void ProcessDirectoryListing(std::shared_ptr<CDirectoryListing> const& listing);
 
 	void SetCtrlState();
 
@@ -31,7 +31,7 @@ protected:
 	CSearchDialogFileList *m_results{};
 	CQueueView* m_pQueue;
 
-	virtual void OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2);
+	virtual void OnStateChange(t_statechange_notifications notification, const wxString& data, const void* data2);
 
 	CWindowStateManager* m_pWindowStateManager{};
 
@@ -42,7 +42,7 @@ protected:
 	CServerPath m_original_dir;
 	CLocalPath m_local_target;
 
-	void ProcessSelection(std::list<int> &selected_files, std::list<CServerPath> &selected_dirs);
+	void ProcessSelection(std::list<int> &selected_files, std::deque<CServerPath> &selected_dirs);
 
 	DECLARE_EVENT_TABLE()
 	void OnSearch(wxCommandEvent& event);
