@@ -6,7 +6,8 @@
 #include "Mainfrm.h"
 #include "queue.h"
 #include "filezillaapp.h"
-#include "recursive_operation.h"
+#include "local_recursive_operation.h"
+#include "remote_recursive_operation.h"
 #include "listingcomparison.h"
 #include "xrc_helper.h"
 
@@ -44,8 +45,7 @@ void CContextManager::DestroyState(CState* pState)
 		m_contexts.erase(m_contexts.begin() + i);
 		if ((int)i < m_current_context)
 			m_current_context--;
-		else if ((int)i == m_current_context)
-		{
+		else if ((int)i == m_current_context) {
 			if (i >= m_contexts.size())
 				m_current_context--;
 			NotifyHandlers(GetCurrentContext(), STATECHANGE_CHANGEDCONTEXT, _T(""), 0);
@@ -63,8 +63,7 @@ void CContextManager::SetCurrentContext(CState* pState)
 	if (GetCurrentContext() == pState)
 		return;
 
-	for (unsigned int i = 0; i < m_contexts.size(); i++)
-	{
+	for (unsigned int i = 0; i < m_contexts.size(); ++i) {
 		if (m_contexts[i] != pState)
 			continue;
 
@@ -78,8 +77,7 @@ void CContextManager::DestroyAllStates()
 	m_current_context = -1;
 	NotifyHandlers(GetCurrentContext(), STATECHANGE_CHANGEDCONTEXT, _T(""), 0);
 
-	while (!m_contexts.empty())
-	{
+	while (!m_contexts.empty()) {
 		CState* pState = m_contexts.back();
 		m_contexts.pop_back();
 
@@ -88,7 +86,7 @@ void CContextManager::DestroyAllStates()
 	}
 }
 
-void CContextManager::RegisterHandler(CStateEventHandler* pHandler, enum t_statechange_notifications notification, bool current_only)
+void CContextManager::RegisterHandler(CGlobalStateEventHandler* pHandler, t_statechange_notifications notification, bool current_only)
 {
 	wxASSERT(pHandler);
 	wxASSERT(notification != STATECHANGE_MAX && notification != STATECHANGE_NONE);
@@ -106,7 +104,7 @@ void CContextManager::RegisterHandler(CStateEventHandler* pHandler, enum t_state
 	handlers.push_back(handler);
 }
 
-void CContextManager::UnregisterHandler(CStateEventHandler* pHandler, enum t_statechange_notifications notification)
+void CContextManager::UnregisterHandler(CGlobalStateEventHandler* pHandler, t_statechange_notifications notification)
 {
 	wxASSERT(pHandler);
 	wxASSERT(notification != STATECHANGE_MAX);
@@ -160,13 +158,13 @@ CState* CContextManager::GetCurrentContext()
 	return m_contexts[m_current_context];
 }
 
-void CContextManager::NotifyAllHandlers(enum t_statechange_notifications notification, const wxString& data /*=_T("")*/, const void* data2 /*=0*/)
+void CContextManager::NotifyAllHandlers(t_statechange_notifications notification, const wxString& data /*=_T("")*/, const void* data2 /*=0*/)
 {
 	for (unsigned int i = 0; i < m_contexts.size(); i++)
 		m_contexts[i]->NotifyHandlers(notification, data, data2);
 }
 
-void CContextManager::NotifyGlobalHandlers(enum t_statechange_notifications notification, const wxString& data /*=_T("")*/, const void* data2 /*=0*/)
+void CContextManager::NotifyGlobalHandlers(t_statechange_notifications notification, const wxString& data /*=_T("")*/, const void* data2 /*=0*/)
 {
 	auto const& handlers = m_handlers[notification];
 	for (auto const& handler : handlers) {
@@ -179,9 +177,10 @@ CState::CState(CMainFrame &mainFrame)
 {
 	m_title = _("Not connected");
 
-	m_pComparisonManager = new CComparisonManager(this);
+	m_pComparisonManager = new CComparisonManager(*this);
 
-	m_pRecursiveOperation = new CRecursiveOperation(this);
+	m_pLocalRecursiveOperation = new CLocalRecursiveOperation(*this);
+	m_pRemoteRecursiveOperation = new CRemoteRecursiveOperation(*this);
 
 	m_sync_browse.is_changing = false;
 	m_sync_browse.compare = false;
@@ -196,17 +195,13 @@ CState::~CState()
 	delete m_pComparisonManager;
 	delete m_pCommandQueue;
 	delete m_pEngine;
+	delete m_pLocalRecursiveOperation;
+	delete m_pRemoteRecursiveOperation;
 
 	// Unregister all handlers
-	for (int i = 0; i < STATECHANGE_MAX; i++)
-	{
-		for (auto iter = m_handlers[i].begin(); iter != m_handlers[i].end(); ++iter)
-		{
-			iter->pHandler->m_pState = 0;
-		}
+	for (int i = 0; i < STATECHANGE_MAX; ++i) {
+		wxASSERT(m_handlers[i].empty());
 	}
-
-	delete m_pRecursiveOperation;
 }
 
 CLocalPath CState::GetLocalDir() const
@@ -498,13 +493,13 @@ wxString CState::GetTitle() const
 	return m_title;
 }
 
-bool CState::Connect(const CServer& server, const CServerPath& path /*=CServerPath()*/)
+bool CState::Connect(const CServer& server, const CServerPath& path)
 {
 	if (!m_pEngine)
 		return false;
 	if (m_pEngine->IsConnected() || m_pEngine->IsBusy() || !m_pCommandQueue->Idle())
 		m_pCommandQueue->Cancel();
-	m_pRecursiveOperation->StopRecursiveOperation();
+	m_pRemoteRecursiveOperation->StopRecursiveOperation();
 	SetSyncBrowse(false);
 
 	m_pCommandQueue->ProcessCommand(new CConnectCommand(server));
@@ -538,10 +533,9 @@ bool CState::CreateEngine()
 	if (m_pEngine)
 		return true;
 
-	m_pEngine = new CFileZillaEngine(m_mainFrame.GetEngineContext());
-	m_pEngine->Init(&m_mainFrame);
+	m_pEngine = new CFileZillaEngine(m_mainFrame.GetEngineContext(), m_mainFrame);
 
-	m_pCommandQueue = new CCommandQueue(m_pEngine, &m_mainFrame, this);
+	m_pCommandQueue = new CCommandQueue(m_pEngine, &m_mainFrame, *this);
 
 	return true;
 }
@@ -554,27 +548,35 @@ void CState::DestroyEngine()
 	m_pEngine = 0;
 }
 
-void CState::RegisterHandler(CStateEventHandler* pHandler, enum t_statechange_notifications notification)
+void CState::RegisterHandler(CStateEventHandler* pHandler, t_statechange_notifications notification, CStateEventHandler* insertBefore)
 {
 	wxASSERT(pHandler);
-	wxASSERT(pHandler->m_pState == this);
-	if (pHandler->m_pState != this)
+	wxASSERT(&pHandler->m_state == this);
+	if (!pHandler || &pHandler->m_state != this)
 		return;
 	wxASSERT(notification != STATECHANGE_MAX && notification != STATECHANGE_NONE);
+	wxASSERT(pHandler != insertBefore);
 
+	
 	auto &handlers = m_handlers[notification];
-	for (auto const& it : handlers) {
-		if (it.pHandler == pHandler) {
+	auto insertionPoint = handlers.end();
+
+	for (auto it = handlers.begin(); it != handlers.end(); ++it) {
+		if (it->pHandler == insertBefore) {
+			insertionPoint = it;
+		}
+		if (it->pHandler == pHandler) {
+			wxASSERT(insertionPoint == handlers.end());
 			return;
 		}
 	}
 
 	t_handler handler;
 	handler.pHandler = pHandler;
-	handlers.push_back(handler);
+	handlers.insert(insertionPoint, handler);
 }
 
-void CState::UnregisterHandler(CStateEventHandler* pHandler, enum t_statechange_notifications notification)
+void CState::UnregisterHandler(CStateEventHandler* pHandler, t_statechange_notifications notification)
 {
 	wxASSERT(pHandler);
 	wxASSERT(notification != STATECHANGE_MAX);
@@ -601,30 +603,31 @@ void CState::UnregisterHandler(CStateEventHandler* pHandler, enum t_statechange_
 	}
 }
 
-void CState::NotifyHandlers(enum t_statechange_notifications notification, const wxString& data, const void* data2)
+void CState::NotifyHandlers(t_statechange_notifications notification, const wxString& data, const void* data2)
 {
 	wxASSERT(notification != STATECHANGE_NONE && notification != STATECHANGE_MAX);
 
 	auto const& handlers = m_handlers[notification];
 	for (auto const& handler : handlers) {
-		handler.pHandler->OnStateChange(this, notification, data, data2);
+		handler.pHandler->OnStateChange(notification, data, data2);
 	}
 
 	CContextManager::Get()->NotifyHandlers(this, notification, data, data2);
 }
 
-CStateEventHandler::CStateEventHandler(CState* pState)
-	: m_pState(pState)
+CStateEventHandler::CStateEventHandler(CState &state)
+	: m_state(state)
 {
 }
 
 CStateEventHandler::~CStateEventHandler()
 {
-	CContextManager::Get()->UnregisterHandler(this, STATECHANGE_NONE);
+	m_state.UnregisterHandler(this, STATECHANGE_NONE);
+}
 
-	const std::vector<CState*> *states = CContextManager::Get()->GetAllStates();
-	for (std::vector<CState*>::const_iterator iter = states->begin(); iter != states->end(); ++iter)
-		(*iter)->UnregisterHandler(this, STATECHANGE_NONE);
+CGlobalStateEventHandler::~CGlobalStateEventHandler()
+{
+	CContextManager::Get()->UnregisterHandler(this, STATECHANGE_NONE);
 }
 
 void CState::UploadDroppedFiles(const wxFileDataObject* pFileDataObject, const wxString& subdir, bool queueOnly)
@@ -646,6 +649,16 @@ void CState::UploadDroppedFiles(const wxFileDataObject* pFileDataObject, const C
 	if (!m_pServer)
 		return;
 
+	if (path.empty()) {
+		return;
+	}
+
+	auto recursiveOperation = GetLocalRecursiveOperation();
+	if (!recursiveOperation || recursiveOperation->IsActive()) {
+		wxBell();
+		return;
+	}
+
 	const wxArrayString& files = pFileDataObject->GetFilenames();
 
 	for (unsigned int i = 0; i < files.Count(); ++i) {
@@ -660,14 +673,23 @@ void CState::UploadDroppedFiles(const wxFileDataObject* pFileDataObject, const C
 		}
 		else if (type == fz::local_filesys::dir) {
 			CLocalPath localPath(files[i]);
-			if (localPath.HasParent())
-			{
-				CServerPath target = path;
-				target.AddSegment(localPath.GetLastSegment());
-				m_mainFrame.GetQueue()->QueueFolder(queueOnly, false, localPath, target, *m_pServer);
+			if (localPath.HasParent()) {
+				
+				CServerPath remotePath = path;
+				if (!remotePath.ChangePath(localPath.GetLastSegment())) {
+					continue;
+				}
+
+				local_recursion_root root;
+				root.add_dir_to_visit(localPath, remotePath);
+				recursiveOperation->AddRecursionRoot(std::move(root));
 			}
 		}
 	}
+
+	CFilterManager filter;
+	recursiveOperation->StartRecursiveOperation(queueOnly ? CRecursiveOperation::recursive_addtoqueue : CRecursiveOperation::recursive_transfer,
+		filter.GetActiveFilters(true));
 }
 
 void CState::HandleDroppedFiles(const wxFileDataObject* pFileDataObject, const CLocalPath& path, bool copy)
@@ -844,10 +866,10 @@ bool CState::DownloadDroppedFiles(const CRemoteDataObject* pRemoteDataObject, co
 		if (m_pComparisonManager->IsComparing())
 			m_pComparisonManager->ExitComparisonMode();
 
-		m_pRecursiveOperation->AddRecursionRoot(std::move(root));
+		m_pRemoteRecursiveOperation->AddRecursionRoot(std::move(root));
 
 		CFilterManager filter;
-		m_pRecursiveOperation->StartRecursiveOperation(queueOnly ? CRecursiveOperation::recursive_addtoqueue : CRecursiveOperation::recursive_download,
+		m_pRemoteRecursiveOperation->StartRecursiveOperation(queueOnly ? CRecursiveOperation::recursive_addtoqueue : CRecursiveOperation::recursive_transfer,
 			filter.GetActiveFilters(false), pRemoteDataObject->GetServerPath());
 	}
 
@@ -864,7 +886,7 @@ bool CState::IsRemoteConnected() const
 
 bool CState::IsRemoteIdle(bool ignore_recursive) const
 {
-	if (!ignore_recursive && m_pRecursiveOperation->GetOperationMode() != CRecursiveOperation::recursive_none)
+	if (!ignore_recursive && m_pRemoteRecursiveOperation->GetOperationMode() != CRecursiveOperation::recursive_none)
 		return false;
 
 	if (!m_pCommandQueue)
