@@ -84,14 +84,14 @@ void CStatusLineCtrl::OnPaint(wxPaintEvent&)
 	if (!m_data.IsOk() || rect.GetWidth() != m_data.GetWidth() || rect.GetHeight() != m_data.GetHeight()) {
 		m_mdc.reset();
 		m_data = wxBitmap(rect.GetWidth(), rect.GetHeight());
-		m_mdc = make_unique<wxMemoryDC>(m_data);
+		m_mdc = std::make_unique<wxMemoryDC>(m_data);
 		// Use same layout direction as the DC which bitmap is drawn on.
 		// This avoids problem with mirrored characters on RTL locales.
 		m_mdc->SetLayoutDirection(dc.GetLayoutDirection());
 		refresh = 31;
 	}
 
-	wxTimeSpan elapsed;
+	fz::duration elapsed;
 	int left = -1;
 	wxFileOffset rate;
 	wxString bytes_and_rate;
@@ -119,9 +119,9 @@ void CStatusLineCtrl::OnPaint(wxPaintEvent&)
 		}
 
 		int elapsed_milli_seconds = 0;
-		if (status_.started.IsValid()) {
-			elapsed = wxDateTime::UNow().Subtract(status_.started);
-			elapsed_milli_seconds = elapsed.GetMilliseconds().GetLo(); // Assume GetHi is always 0
+		if (!status_.started.empty()) {
+			elapsed = fz::datetime::now() - status_.started;
+			elapsed_milli_seconds = static_cast<int>(elapsed.get_milliseconds()); // Assume it doesn't overflow
 		}
 
 		if (elapsed_milli_seconds / 1000 != m_last_elapsed_seconds) {
@@ -197,7 +197,7 @@ void CStatusLineCtrl::OnPaint(wxPaintEvent&)
 
 		if (refresh & 1) {
 			m_mdc->DrawRectangle(0, 0, m_fieldOffsets[0], rect.GetHeight());
-			DrawRightAlignedText(*m_mdc, elapsed.Format(_("%H:%M:%S elapsed")), m_fieldOffsets[0], h);
+			DrawRightAlignedText(*m_mdc, wxTimeSpan::Milliseconds(elapsed.get_milliseconds()).Format(_("%H:%M:%S elapsed")), m_fieldOffsets[0], h);
 		}
 		if (refresh & 2) {
 			m_mdc->DrawRectangle(m_fieldOffsets[0], 0, m_fieldOffsets[1] - m_fieldOffsets[0], rect.GetHeight());
@@ -343,7 +343,7 @@ void CStatusLineCtrl::DrawProgressBar(wxDC& dc, int x, int y, int height, int ba
 	// Draw percentage-done text
 	wxString text;
 	if (permill > 1000) {
-		text = _T("> 100.0%%");
+		text = _T("> 100.0%");
 	}
 	else {
 		text = wxString::Format(_T("%d.%d%%"), permill / 10, permill % 10);
