@@ -7,6 +7,8 @@
 #include "file_utils.h"
 #include "update_dialog.h"
 #include "themeprovider.h"
+#include "xrc_helper.h"
+#include "Options.h"
 
 #include <wx/animate.h>
 #include <wx/hyperlink.h>
@@ -77,6 +79,8 @@ int CUpdateDialog::ShowModal()
 	XRCCTRL(*this, "ID_WAIT_DOWNLOAD", wxAnimationCtrl)->SetAnimation(a);
 	XRCCTRL(*this, "ID_WAIT_DOWNLOAD", wxAnimationCtrl)->Play();
 
+	InitFooter();
+
 	Wrap();
 
 	XRCCTRL(*this, "ID_DETAILS", wxTextCtrl)->Hide();
@@ -94,6 +98,60 @@ int CUpdateDialog::ShowModal()
 	return ret;
 }
 
+namespace {
+void MakeLinksFromTooltips(wxWindow& parent)
+{
+	// Iterates over all children.
+	// If tooltip is a URL, make the child have the hand cursor
+	// and launch URL in browser upon click.
+	wxWindowList children = parent.GetChildren();
+	for (auto child : children) {
+		wxString const tooltip = child->GetToolTipText();
+		if (tooltip.find(_T("http://")) == 0 || tooltip.find(_T("https://")) == 0) {
+			child->SetCursor(wxCURSOR_HAND);
+			child->Bind(wxEVT_LEFT_UP, std::bind(&wxLaunchDefaultBrowser, tooltip, 0));
+		}
+		MakeLinksFromTooltips(*child);
+	}
+}
+}
+
+
+void CUpdateDialog::InitFooter()
+{
+	bool hideFooter = true;
+#if FZ_WINDOWS
+	if (CBuildInfo::GetBuildType() == _T("official")) {
+		wxString const resources = updater_.GetResources();
+		if (!resources.empty()) {
+			wxLogNull null;
+
+			wxXmlResource res(wxXRC_NO_RELOADING);
+			InitHandlers(res);
+			if (res.Load(_T("blob:") + resources)) {
+				wxWindow* parent = XRCCTRL(*this, "ID_FOOTERMESSAGE_PANEL", wxPanel);
+				if (parent) {
+					wxPanel* p = new wxPanel();
+					if (res.LoadPanel(p, parent, _T("ID_UPDATE_FOOTER"))) {
+						wxSize minSize = p->GetSizer()->GetMinSize();
+						parent->SetInitialSize(minSize);
+						hideFooter = false;
+						MakeLinksFromTooltips(*p);
+					}
+					else {
+						delete p;
+					}
+				}
+			}
+		}
+	}
+#endif
+	if (hideFooter) {
+		XRCCTRL(*this, "ID_FOOTERMESSAGE_PANEL", wxPanel)->Hide();
+	}
+
+}
+
 void CUpdateDialog::Wrap()
 {
 	wxPanel* parentPanel = XRCCTRL(*this, "ID_CONTENT", wxPanel);
@@ -221,8 +279,9 @@ void CUpdateDialog::OnInstall(wxCommandEvent&)
 	if( f.empty() ) {
 		return;
 	}
+	COptions::Get()->SetOption(OPTION_GREETINGRESOURCES, updater_.GetResources());
 #ifdef __WXMSW__
-	wxExecute(_T("\"") + f +  _T("\" /update"));
+	wxExecute(_T("\"") + f +  _T("\" /update /NCRC"));
 	wxWindow* p = parent_;
 	while( p->GetParent() ) {
 		p = p->GetParent();
@@ -243,12 +302,12 @@ void CUpdateDialog::OnInstall(wxCommandEvent&)
 
 void CUpdateDialog::OnTimer(wxTimerEvent&)
 {
-	wxULongLong size = updater_.AvailableBuild().size_;
-	wxULongLong downloaded = updater_.BytesDownloaded();
+	int64_t size = updater_.AvailableBuild().size_;
+	int64_t downloaded = updater_.BytesDownloaded();
 
 	unsigned int percent = 0;
-	if( size > 0 ) {
-		percent = ((downloaded * 100) / size).GetLo();
+	if (size > 0 && downloaded >= 0) {
+		percent = static_cast<unsigned int>((downloaded * 100) / size);
 	}
 
 	XRCCTRL(*this, "ID_DOWNLOAD_PROGRESS", wxStaticText)->SetLabel(wxString::Format(_("(%u%% downloaded)"), percent));
@@ -269,7 +328,7 @@ void CUpdateDialog::Retry(wxHyperlinkEvent&)
 
 void CUpdateDialog::OnDebugLog(wxCommandEvent&)
 {
-	wxMessageBox(updater_.GetLog());
+	wxMessageBoxEx(updater_.GetLog());
 }
 
 #endif
