@@ -6,23 +6,22 @@
 #include "updater.h"
 #include "Options.h"
 #include "file_utils.h"
-#include <local_filesys.h>
 #include <wx/base64.h>
 #include <wx/tokenzr.h>
 #include <string>
 
+#ifdef __WXMSW__
+#include <wx/msw/registry.h>
+#endif
+
+#include <libfilezilla/file.hpp>
+#include <libfilezilla/local_filesys.hpp>
+
 #include <string>
 
-// This is ugly but does the job
-#define SHA512_STANDALONE
-typedef unsigned int uint32;
-namespace {
-#include "../putty/int64.h"
-#include "../putty/sshsh512.c"
-}
+#include <nettle/sha2.h>
 
 BEGIN_EVENT_TABLE(CUpdater, wxEvtHandler)
-EVT_FZ_NOTIFICATION(wxID_ANY, CUpdater::OnEngineEvent)
 EVT_TIMER(wxID_ANY, CUpdater::OnTimer)
 END_EVENT_TABLE()
 
@@ -81,15 +80,14 @@ static CUpdater* instance = 0;
 
 CUpdater::CUpdater(CUpdateHandler& parent, CFileZillaEngineContext& engine_context)
 	: state_(UpdaterState::idle)
-	, engine_(new CFileZillaEngine(engine_context))
+	, engine_(new CFileZillaEngine(engine_context, *this))
 {
 	AddHandler(parent);
-	engine_->Init(this);
 }
 
 void CUpdater::Init()
 {
-	if( state_ == UpdaterState::checking || state_ == UpdaterState::newversion_downloading ) {
+	if (state_ == UpdaterState::checking || state_ == UpdaterState::newversion_downloading) {
 		return;
 	}
 
@@ -150,94 +148,29 @@ bool CUpdater::LongTimeSinceLastCheck() const
 	if (lastCheckStr.empty())
 		return true;
 
-	wxDateTime lastCheck;
-	lastCheck.ParseDateTime(lastCheckStr);
-	if (!lastCheck.IsValid())
+	fz::datetime lastCheck(lastCheckStr.ToStdWstring(), fz::datetime::utc);
+	if (lastCheck.empty())
 		return true;
 
-	wxTimeSpan const span = wxDateTime::Now() - lastCheck;
+	auto const span = fz::datetime::now() - lastCheck;
 
-	if (span.GetSeconds() < 0)
+	if (span.get_seconds() < 0)
 		// Last check in future
 		return true;
 
 	int days = 1;
 	if (!CBuildInfo::IsUnstable())
 		days = COptions::Get()->GetOptionVal(OPTION_UPDATECHECK_INTERVAL);
-	return span.GetDays() >= days;
+	return span.get_days() >= days;
 }
 
-#if defined(__i386__) || defined(__x86_64__) || defined(_M_X64) || defined(_M_IX86)
-#define HAVE_CPUID
-#endif
-
-#ifdef HAVE_CPUID
-
-#ifdef _MSC_VER
-namespace {
-void cpuid(int f, int reg[4])
-{
-	__cpuid(reg, f);
-}
-}
-#else
-#include <cpuid.h>
-namespace {
-void cpuid(int f, int reg[4])
-{
-	__cpuid(f, reg[0], reg[1], reg[2], reg[3]);
-}
-}
-#endif
-
-namespace {
-wxString GetCPUCaps()
-{
-	wxString ret;
-
-	int reg[4];
-	cpuid(0, reg);
-
-	int const max = reg[0];
-
-	// function, register, bit, description
-	std::tuple<int, int, int, wxString> const caps[] = {
-		std::make_tuple(1, 3, 25, _T("sse")),
-		std::make_tuple(1, 3, 26, _T("sse2")),
-		std::make_tuple(1, 2, 0, _T("sse3")),
-		std::make_tuple(1, 2, 9, _T("ssse3")),
-		std::make_tuple(1, 2, 19, _T("sse4.1")),
-		std::make_tuple(1, 2, 20, _T("sse4.2")),
-		std::make_tuple(1, 2, 28, _T("avx")),
-		std::make_tuple(1, 2, 25, _T("aes")),
-		std::make_tuple(1, 2, 1, _T("pclmulqdq")),
-		std::make_tuple(1, 2, 30, _T("rdrnd"))
-	};
-
-	for (auto const& cap : caps) {
-		if (max >= std::get<0>(cap)) {
-			cpuid(std::get<0>(cap), reg);
-			if (reg[std::get<1>(cap)] & (1 << std::get<2>(cap))) {
-				if (!ret.empty()) {
-					ret += ',';
-				}
-				ret += std::get<3>(cap);
-			}
-		}
-	}
-
-	return ret;
-}
-}
-#endif
-
 wxString CUpdater::GetUrl()
 {
 	wxString host = CBuildInfo::GetHostname();
 	if (host.empty())
 		host = _T("unknown");
 
-	wxString version(PACKAGE_VERSION, wxConvLocal);
+	wxString version = CBuildInfo::GetVersion();
 	version.Replace(_T(" "), _T("%20"));
 
 	wxString url = wxString::Format(_T("https://update.filezilla-project.org/update.php?platform=%s&version=%s"), host, version);
@@ -252,11 +185,42 @@ wxString CUpdater::GetUrl()
 		url += _T("&osarch=64");
 	else
 		url += _T("&osarch=32");
+
+	// Add information about package
+	{
+		wxLogNull log;
+
+		// Installer always writes to 32bit section
+		auto key = std::make_unique<wxRegKey>(_T("HKEY_CURRENT_USER\\Software\\FileZilla Client"), wxRegKey::WOW64ViewMode_32);
+		if (!key->Exists()) {
+			// wxRegKey is sad, it doesn't even have a copy constructor.
+			key = std::make_unique<wxRegKey>(_T("HKEY_LOCAL_MACHINE\\Software\\FileZilla Client"), wxRegKey::WOW64ViewMode_32);
+		}
+
+		long updated{};
+		if (key->GetValueType(_T("Updated")) == wxRegKey::Type_Dword && key->QueryValue(_T("Updated"), &updated)) {
+			url += wxString::Format(_T("&updated=%d"), updated);
+		}
+
+		long package{};
+		if (key->GetValueType(_T("Package")) == wxRegKey::Type_Dword && key->QueryValue(_T("Package"), &package)) {
+			url += wxString::Format(_T("&package=%d"), package);
+		}
+	}
 #endif
 
-#ifdef HAVE_CPUID
-	url += _T("&cpuid=") + GetCPUCaps();
-#endif
+	wxString const cpuCaps = CBuildInfo::GetCPUCaps(',');
+	if (!cpuCaps.empty()) {
+		url += _T("&cpuid=") + cpuCaps;
+	}
+
+	wxString const lastVersion = COptions::Get()->GetOption(OPTION_UPDATECHECK_LASTVERSION);
+	if (lastVersion != CBuildInfo::GetVersion()) {
+		url += _T("&initial=1");
+	}
+	else {
+		url += _T("&initial=0");
+	}
 
 	return url;
 }
@@ -269,11 +233,11 @@ bool CUpdater::Run()
 		return false;
 	}
 
-	wxDateTime const t = wxDateTime::Now();
-	COptions::Get()->SetOption(OPTION_UPDATECHECK_LASTDATE, t.Format(_T("%Y-%m-%d %H:%M:%S")));
+	auto  const t = fz::datetime::now();
+	COptions::Get()->SetOption(OPTION_UPDATECHECK_LASTDATE, t.format(_T("%Y-%m-%d %H:%M:%S"), fz::datetime::utc));
 
 	local_file_.clear();
-	log_ = wxString::Format(_("Started update check on %s\n"), t.Format());
+	log_ = wxString::Format(_("Started update check on %s\n"), t.format(_T("%Y-%m-%d %H:%M:%S"), fz::datetime::local));
 
 	wxString build = CBuildInfo::GetBuildType();
 	if( build.empty() ) {
@@ -351,9 +315,14 @@ bool CUpdater::CreateTransferCommand(wxString const& url, wxString const& local_
 	return true;
 }
 
-void CUpdater::OnEngineEvent(wxFzEvent& event)
+void CUpdater::OnEngineEvent(CFileZillaEngine* engine)
 {
-	if (!engine_ || engine_ != event.engine_)
+	CallAfter(&CUpdater::DoOnEngineEvent, engine);
+}
+
+void CUpdater::DoOnEngineEvent(CFileZillaEngine* engine)
+{
+	if (!engine_ || engine_ != engine)
 		return;
 
 	std::unique_ptr<CNotification> notification;
@@ -422,14 +391,14 @@ UpdaterState CUpdater::ProcessFinishedData(bool can_download)
 
 	ParseData();
 
-	if( version_information_.available_.version_.empty() ) {
+	if (version_information_.available_.version_.empty()) {
 		s = UpdaterState::idle;
 	}
-	else if( !version_information_.available_.url_.empty() ) {
+	else if (!version_information_.available_.url_.empty()) {
 
 		wxString const temp = GetTempFile();
 		wxString const local_file = GetLocalFile(version_information_.available_, true);
-		if( !local_file.empty() && CLocalFileSystem::GetFileType(local_file) != CLocalFileSystem::unknown) {
+		if (!local_file.empty() && fz::local_filesys::get_file_type(fz::to_native(local_file)) != fz::local_filesys::unknown) {
 			local_file_ = local_file;
 			log_ += wxString::Format(_("Local file is %s\n"), local_file);
 			s = UpdaterState::newversion_ready;
@@ -438,13 +407,13 @@ UpdaterState CUpdater::ProcessFinishedData(bool can_download)
 			// We got a checksum over a secure channel already.
 			m_use_internal_rootcert = false;
 
-			if( temp.empty() || local_file.empty() ) {
+			if (temp.empty() || local_file.empty()) {
 				s = UpdaterState::newversion;
 			}
 			else {
 				s = UpdaterState::newversion_downloading;
-				wxLongLong size = CLocalFileSystem::GetSize(temp);
-				if( size >= 0 && static_cast<unsigned long long>(size.GetValue()) >= version_information_.available_.size_ ) {
+				auto size = fz::local_filesys::get_size(fz::to_native(temp));
+				if (size >= 0 && size >= version_information_.available_.size_) {
 					s = ProcessFinishedDownload();
 				}
 				else if( !can_download || Download( version_information_.available_.url_, GetTempFile() ) != FZ_REPLY_WOULDBLOCK ) {
@@ -462,7 +431,7 @@ UpdaterState CUpdater::ProcessFinishedData(bool can_download)
 
 void CUpdater::ProcessOperation(COperationNotification const& operation)
 {
-	if( state_ != UpdaterState::checking && state_ != UpdaterState::newversion_downloading ) {
+	if (state_ != UpdaterState::checking && state_ != UpdaterState::newversion_downloading) {
 		return;
 	}
 
@@ -488,7 +457,8 @@ void CUpdater::ProcessOperation(COperationNotification const& operation)
 			s = UpdaterState::newversion;
 		}
 	}
-	else if( state_ == UpdaterState::checking ) {
+	else if (state_ == UpdaterState::checking) {
+		COptions::Get()->SetOption(OPTION_UPDATECHECK_LASTVERSION, CBuildInfo::GetVersion());
 		s = ProcessFinishedData(true);
 	}
 	else {
@@ -529,7 +499,7 @@ UpdaterState CUpdater::ProcessFinishedDownload()
 	return s;
 }
 
-wxString CUpdater::GetLocalFile( build const& b, bool allow_existing )
+wxString CUpdater::GetLocalFile(build const& b, bool allow_existing)
 {
 	wxString const fn = GetFilename( b.url_ );
 	wxString const dl = GetDownloadDir().GetPath();
@@ -537,20 +507,20 @@ wxString CUpdater::GetLocalFile( build const& b, bool allow_existing )
 	int i = 1;
 	wxString f = dl + fn;
 
-	while( CLocalFileSystem::GetFileType(f) != CLocalFileSystem::unknown && (!allow_existing || !VerifyChecksum(f, b.size_, b.hash_))) {
-		if( ++i > 99 ) {
+	while (fz::local_filesys::get_file_type(fz::to_native(f)) != fz::local_filesys::unknown && (!allow_existing || !VerifyChecksum(f, b.size_, b.hash_))) {
+		if (++i > 99) {
 			return wxString();
 		}
 		wxString ext;
 		int pos;
-		if( !fn.Right(8).CmpNoCase(_T(".tar.bz2")) ) {
+		if (!fn.Right(8).CmpNoCase(_T(".tar.bz2"))) {
 			pos = fn.size() - 8;
 		}
 		else {
 			pos = fn.Find('.', true);
 		}
 
-		if( pos == -1 ) {
+		if (pos == -1) {
 			f = dl + fn + wxString::Format(_T(" (%d)"), i);
 		}
 		else {
@@ -574,7 +544,7 @@ void CUpdater::ProcessData(CDataNotification& dataNotification)
 		log_ += wxString::Format(_T("ProcessData %d\n"), len);
 	}
 
-	if( raw_version_information_.size() + len > 131072 ) {
+	if( raw_version_information_.size() + len > 0x40000) {
 		log_ += _("Received version information is too large");
 		engine_->Cancel();
 		SetState(UpdaterState::failed);
@@ -598,7 +568,7 @@ void CUpdater::ProcessData(CDataNotification& dataNotification)
 
 void CUpdater::ParseData()
 {
-	const wxLongLong ownVersionNumber = CBuildInfo::ConvertToVersionNumber(CBuildInfo::GetVersion().c_str());
+	int64_t const ownVersionNumber = CBuildInfo::ConvertToVersionNumber(CBuildInfo::GetVersion().c_str());
 	version_information_ = version_information();
 
 	wxString raw_version_information = raw_version_information_;
@@ -618,39 +588,48 @@ void CUpdater::ParseData()
 		}
 
 		wxStringTokenizer tokens(line, _T(" \t\n"),  wxTOKEN_STRTOK);
-		if( !tokens.CountTokens() ) {
+		if (!tokens.CountTokens()) {
 			// After empty line, changelog follows
-			version_information_.changelog = raw_version_information;
-			version_information_.changelog.Trim(true);
-			version_information_.changelog.Trim(false);
+			version_information_.changelog_ = raw_version_information;
+			version_information_.changelog_.Trim(true);
+			version_information_.changelog_.Trim(false);
 
-			if( COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4 ) {
-				log_ += wxString::Format(_T("Changelog: %s\n"), version_information_.changelog);
+			if (COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4) {
+				log_ += wxString::Format(_T("Changelog: %s\n"), version_information_.changelog_);
 			}
 			break;
 		}
 
-		if( tokens.CountTokens() != 2 && tokens.CountTokens() != 6 ) {
-			if( COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4 ) {
-				log_ += wxString::Format(_T("Skipping line with %d tokens\n"), static_cast<int>(tokens.CountTokens()));
+		wxString const type = tokens.GetNextToken();
+		if (type == _T("resources")) {
+			if (tokens.CountTokens() == 2) {
+				if (UpdatableBuild()) {
+					version_information_.resources_ = tokens.GetNextToken();
+				}
+			}
+			continue;
+		}
+
+		if (tokens.CountTokens() != 1 && tokens.CountTokens() != 5) {
+			if (COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4) {
+				log_ += wxString::Format(_T("Skipping line with %d tokens\n"), static_cast<int>(tokens.CountTokens() + 1));
 			}
 			continue;
 		}
 
-		wxString type = tokens.GetNextToken();
 		wxString versionOrDate = tokens.GetNextToken();
 
 		if (type == _T("nightly")) {
-			wxDateTime nightlyDate;
-			if( !nightlyDate.ParseFormat(versionOrDate, _T("%Y-%m-%d")) ) {
-				if( COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4 ) {
+			fz::datetime nightlyDate(versionOrDate.ToStdWstring(), fz::datetime::utc);
+			if (nightlyDate.empty()) {
+				if (COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4) {
 					log_ += _T("Could not parse nightly date\n");
 				}
 				continue;
 			}
 
-			wxDateTime buildDate = CBuildInfo::GetBuildDate();
-			if (!buildDate.IsValid() || !nightlyDate.IsValid() || nightlyDate <= buildDate) {
+			fz::datetime buildDate = CBuildInfo::GetBuildDate();
+			if (buildDate.empty() || nightlyDate.empty() || nightlyDate <= buildDate) {
 				if( COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4 ) {
 					log_ += _T("Nightly isn't newer\n");
 				}
@@ -658,7 +637,7 @@ void CUpdater::ParseData()
 			}
 		}
 		else {
-			wxLongLong v = CBuildInfo::ConvertToVersionNumber(versionOrDate.c_str());
+			int64_t v = CBuildInfo::ConvertToVersionNumber(versionOrDate.c_str());
 			if (v <= ownVersionNumber)
 				continue;
 		}
@@ -722,30 +701,29 @@ void CUpdater::OnTimer(wxTimerEvent&)
 	AutoRunIfNeeded();
 }
 
-bool CUpdater::VerifyChecksum( wxString const& file, wxULongLong size, wxString const& checksum )
+bool CUpdater::VerifyChecksum(wxString const& file, int64_t size, wxString const& checksum)
 {
-	if( file.empty() || checksum.empty() ) {
+	if (file.empty() || checksum.empty()) {
 		return false;
 	}
 
-	wxLongLong filesize = CLocalFileSystem::GetSize(file);
-	if( filesize < 0 || static_cast<unsigned long long>(filesize.GetValue()) != size ) {
+	auto filesize = fz::local_filesys::get_size(fz::to_native(file));
+	if (filesize < 0 || filesize != size) {
 		return false;
 	}
 
-	SHA512_State state;
-	SHA512_Init(&state);
+	sha512_ctx state;
+	sha512_init(&state);
 
 	{
-		wxLogNull null;
-		wxFile f;
-		if (!f.Open(file)) {
+		fz::file f(fz::to_native(file), fz::file::reading);
+		if (!f.opened()) {
 			return false;
 		}
-		char buffer[65536];
-		ssize_t read;
-		while ((read = f.Read(buffer, sizeof(buffer))) > 0) {
-			SHA512_Bytes(&state, buffer, read);
+		unsigned char buffer[65536];
+		int64_t read;
+		while ((read = f.read(buffer, sizeof(buffer))) > 0) {
+			sha512_update(&state, static_cast<size_t>(read), buffer);
 		}
 		if (read < 0) {
 			return false;
@@ -753,10 +731,10 @@ bool CUpdater::VerifyChecksum( wxString const& file, wxULongLong size, wxString
 	}
 
 	unsigned char raw_digest[64];
-	SHA512_Final(&state, raw_digest);
+	sha512_digest(&state, 64, raw_digest);
 
 	wxString digest;
-	for( unsigned int i = 0; i < sizeof(raw_digest); i++ ) {
+	for (unsigned int i = 0; i < sizeof(raw_digest); ++i) {
 		unsigned char l = raw_digest[i] >> 4;
 		unsigned char r = raw_digest[i] & 0x0F;
 
@@ -799,7 +777,7 @@ wxString CUpdater::GetFilename( wxString const& url) const
 {
 	wxString ret;
 	int pos = url.Find('/', true);
-	if( pos != -1 ) {
+	if (pos != -1) {
 		ret = url.Mid(pos + 1);
 	}
 	size_t p = ret.find_first_of(_T("?#"));
@@ -866,24 +844,21 @@ void CUpdater::RemoveHandler( CUpdateHandler& handler )
 	}
 }
 
-wxULongLong CUpdater::BytesDownloaded() const
+int64_t CUpdater::BytesDownloaded() const
 {
-	wxLongLong ret;
-	if( state_ == UpdaterState::newversion_ready ) {
-		if( !local_file_.empty() ) {
-			ret = CLocalFileSystem::GetSize(local_file_);
+	int64_t ret{-1};
+	if (state_ == UpdaterState::newversion_ready) {
+		if (!local_file_.empty()) {
+			ret = fz::local_filesys::get_size(fz::to_native(local_file_));
 		}
 	}
 	else if( state_ == UpdaterState::newversion_downloading ) {
 		wxString const temp = GetTempFile();
 		if( !temp.empty() ) {
-			ret = CLocalFileSystem::GetSize(temp);
+			ret = fz::local_filesys::get_size(fz::to_native(temp));
 		}
 	}
-	if( ret < 0 ) {
-		ret = 0;
-	}
-	return static_cast<unsigned long long>(ret.GetValue());
+	return ret;
 }
 
 bool CUpdater::UpdatableBuild() const
