@@ -8,6 +8,7 @@
 #include "psftp.h"
 #include "ssh.h"
 #include "int64.h"
+#include "winsecur.h"
 
 char *get_ttymode(void *frontend, const char *mode) { return NULL; }
 
@@ -30,50 +31,6 @@ const int platform_uses_x11_unix_by_default = TRUE;
  * File access abstraction.
  */
 
-static wchar_t* utf8_to_wide(const char* utf8)
-{
-    wchar_t *w;
-
-    int len = MultiByteToWideChar(CP_UTF8, 0, utf8, -1, 0, 0);
-    if (len <= 0)
-	return NULL;
-
-    w = snewn(len, wchar_t);
-
-    if (!w)
-	return NULL;
-
-    if (MultiByteToWideChar(CP_UTF8, 0, utf8, -1, w, len) <= 0)
-    {
-	sfree(w);
-	return NULL;
-    }
-
-    return w;
-}
-
-static char* wide_to_utf8(const wchar_t* w)
-{
-    char* utf8;
-
-    int len = WideCharToMultiByte(CP_UTF8, 0, w, -1, 0, 0, 0, 0);
-    if (len <= 0)
-	return NULL;
-
-    utf8 = snewn(len, char);
-
-    if (!utf8)
-	return NULL;
-
-    if (WideCharToMultiByte(CP_UTF8, 0, w, -1, utf8, len, 0, 0) <= 0)
-    {
-	sfree(utf8);
-	return NULL;
-    }
-    
-    return utf8;
-}
-
 /*
  * Set local current directory. Returns NULL on success, or else an
  * error message which must be freed after printing.
@@ -161,8 +118,12 @@ RFile *open_existing_file(const char *name, uint64 *size,
     ret = snew(RFile);
     ret->h = h;
 
-    if (size)
-        size->lo=GetFileSize(h, &(size->hi));
+    if (size) {
+        DWORD lo, hi;
+        lo = GetFileSize(h, &hi);
+        size->lo = lo;
+        size->hi = hi;
+    }
 
     if (mtime || atime) {
 	FILETIME actime, wrtime;
@@ -239,8 +200,12 @@ WFile *open_existing_wfile(const char *name, uint64 *size)
     ret = snew(WFile);
     ret->h = h;
 
-    if (size)
-	size->lo=GetFileSize(h, &(size->hi));
+    if (size) {
+        DWORD lo, hi;
+        lo = GetFileSize(h, &hi);
+        size->lo = lo;
+        size->hi = hi;
+    }
 
     return ret;
 }
@@ -290,7 +255,10 @@ int seek_file(WFile *f, uint64 offset, int whence)
 	return -1;
     }
 
-    SetFilePointer(f->h, offset.lo, &(offset.hi), movemethod);
+    {
+        LONG lo = offset.lo, hi = offset.hi;
+        SetFilePointer(f->h, lo, &hi, movemethod);
+    }
     
     if (GetLastError() != NO_ERROR)
 	return -1;
@@ -301,9 +269,11 @@ int seek_file(WFile *f, uint64 offset, int whence)
 uint64 get_file_posn(WFile *f)
 {
     uint64 ret;
+    LONG lo, hi;
 
-    ret.hi = 0L;
-    ret.lo = SetFilePointer(f->h, 0L, &(ret.hi), FILE_CURRENT);
+    lo = SetFilePointer(f->h, 0L, &hi, FILE_CURRENT);
+    ret.lo = lo;
+    ret.hi = hi;
 
     return ret;
 }
@@ -434,14 +404,14 @@ struct WildcardMatcher {
     char *srcpath;
 };
 
-/*
- * Return a pointer to the portion of str that comes after the last
- * slash (or backslash or colon, if `local' is TRUE).
- */
-static char *stripslashes(char *str, int local)
+char *stripslashes(const char *str, int local)
 {
     char *p;
 
+    /*
+     * On Windows, \ / : are all path component separators.
+     */
+
     if (local) {
         p = strchr(str, ':');
         if (p) str = p+1;
@@ -455,7 +425,7 @@ static char *stripslashes(char *str, int local)
 	if (p) str = p+1;
     }
 
-    return str;
+    return (char *)str;
 }
 
 WildcardMatcher *begin_wildcard_matching(const char *name)
@@ -838,6 +808,25 @@ char *ssh_sftp_get_cmdline(const char *prompt, int no_fds_ok)
     return ctx->line;
 }
 
+void platform_psftp_post_option_setup(void)
+{
+#if !defined UNPROTECT && !defined NO_SECURITY
+    /*
+     * Protect our process.
+     */
+    {
+        char *error = NULL;
+        if (!setprocessacl(error)) {
+            char *message = dupprintf("Could not restrict process ACL: %s",
+                                      error);
+            logevent(NULL, message);
+            sfree(message);
+            sfree(error);
+        }
+    }
+#endif
+}
+
 /* ----------------------------------------------------------------------
  * Main program. Parse arguments etc.
  */
